------------------------------------------------------------------------------------------
-- CalcStat file
-- Written by Giseldah
-- Version 2.2.2
-- ****************************** Start CalcStat ******************************

-- Floating point numbers bring errors into the calculation, both inside the Lotro-client and in this function collection. This is why a 100% match with the stats in Lotro is impossible.
-- Anyway, to compensate for some errors, we use a calculation deviation correction value. This makes for instance 24.49999999 round to 25, as it's assumed that 24.5 was intended as outcome of a formula.
DblCalcDev = 0.00000001;

function CalcStat(SName, SLvl, SParam)

	-- process parameters and parameter defaults

	if type(SName) ~= "string" then
		return "Missing stat name";
	end
	local SN = string.match(SName:upper(),"^%s*(-?%a+%d*%a+)%s*$"); -- allow nested digits in stat name and a starting -
	if SN == nil then
		SN = string.match(SName,"^%s*(-?%a+)%s*$"); -- to allow single character stat name
		if SN == nil then
			return "Illegal stat name";
		end
	end
	SN = string.upper(SN);

	local L = 1; -- default L
	if type(SLvl) == "number" then
		L = SLvl;
	elseif type(SLvl) ~= "nil" then
		return "Illegal level";
	end
	local Lm = L-DblCalcDev;
	local Lp = L+DblCalcDev;

	local N = 1; -- default N
	local C = ""; -- default C
	if type(SParam) == "number" then
		N = SParam;
	elseif type(SParam) == "string" then
		C = SParam;
	elseif type(SParam) ~= "nil" then
		return "Illegal N or C";
	end

	-- binary search tree (generated code)

	if SN < "MINSTRELCDFATETOCRITHIT" then
		if SN < "CREEPBATPROMOUTHEALP" then
			if SN < "BRWRETPRECISIONFINESSE" then
				if SN < "BEORNINGCDBASEVITALITY" then
					if SN < "AWARDILVLGA" then
						if SN < "AWARDILVLAC" then
							if SN < "ARMOURPROGB" then
								if SN < "ARMCATPROGB" then
									if SN < "ACIDMITT" then
										if SN > "-VERSION" then
											if SN == "ACIDMIT" then
												return CalcStat("DmgTypeMit",L,N);
											else
												return 0;
											end
										elseif SN == "-VERSION" then
											return "2.2.2f";
										else
											return 0;
										end
									elseif SN > "ACIDMITT" then
										if SN < "AGILITYT" then
											if SN == "AGILITY" then
												return CalcStat("Main",L,N);
											else
												return 0;
											end
										elseif SN > "AGILITYT" then
											if SN == "ARMCATMP" then
												return CalcStat("PntMPArmour",L);
											else
												return 0;
											end
										else
											return CalcStat("MainT",L,N);
										end
									else
										return CalcStat("DmgTypeMitT",L,N);
									end
								elseif SN > "ARMCATPROGB" then
									if SN < "ARMOURLOWPNTMP" then
										if SN > "ARMOUR" then
											if SN == "ARMOURLOW" then
												return RoundDblDown(StatLinInter("ArmourLowPntMP","ItemProg","ArmourProgB","",L,C,0));
											else
												return 0;
											end
										elseif SN == "ARMOUR" then
											return RoundDblDown(StatLinInter("ArmourPntMP","ItemProg","ArmourProgB","",L,C,0));
										else
											return 0;
										end
									elseif SN > "ARMOURLOWPNTMP" then
										if SN < "ARMOURPENTADJ" then
											if SN == "ARMOURPENT" then
												return StatLinInter("PntMPArmourPenT","TraitProg","ProgBArmour","ArmourPenTAdj",L,N,0);
											else
												return 0;
											end
										elseif SN > "ARMOURPENTADJ" then
											if SN == "ARMOURPNTMP" then
												return CalcStat("ArmTypeMP",ArmCodeIndex(C,2))*CalcStat("ArmQtyMP",ArmCodeIndex(C,3),ArmCodeIndex(C,2))*CalcStat("ArmCatMP",ArmCodeIndex(C,1));
											else
												return 0;
											end
										else
											if Lm <= 49 then
												return 1;
											elseif Lm <= 50 then
												return 0.8;
											else
												return 1;
											end
										end
									else
										return CalcStat("ArmTypeMP",ArmCodeIndex(C,2))*CalcStat("ArmQtyLowMP",ArmCodeIndex(C,2))*CalcStat("ArmCatMP",ArmCodeIndex(C,1));
									end
								else
									if Lm <= 0 then
										return 0;
									elseif Lm <= 1 then
										return CalcStat("ProgBArmourHeavy",N);
									elseif Lm <= 2 then
										return CalcStat("ProgBArmourMedium",N);
									else
										return CalcStat("ProgBArmourLight",N);
									end
								end
							elseif SN > "ARMOURPROGB" then
								if SN < "ARMQTYRAREMP" then
									if SN < "ARMQTYEPICMP" then
										if SN > "ARMOURT" then
											if SN == "ARMQTYCOMMMP" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({22/30,22/30,0.8,0.8,0.76,0.804,0.8,0.7},L);
												end
											else
												return 0;
											end
										elseif SN == "ARMOURT" then
											return StatLinInter("PntMPArmourT","TraitProg","ProgBArmour","",L,N,0);
										else
											return 0;
										end
									elseif SN > "ARMQTYEPICMP" then
										if SN < "ARMQTYLOWMP" then
											if SN == "ARMQTYINCOMPMP" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({28/30,28/30,0.95,0.95,0.94,0.948,0.95,0.95},L);
												end
											else
												return 0;
											end
										elseif SN > "ARMQTYLOWMP" then
											if SN == "ARMQTYMP" then
												if Lm <= 0 then
													return 0;
												elseif Lm <= 1 then
													return CalcStat("ArmQtyCommMP",N);
												elseif Lm <= 2 then
													return CalcStat("ArmQtyUncomMP",N);
												elseif Lm <= 3 then
													return CalcStat("ArmQtyRareMP",N);
												elseif Lm <= 4 then
													return CalcStat("ArmQtyIncompMP",N);
												else
													return CalcStat("ArmQtyEpicMP",N);
												end
											else
												return 0;
											end
										else
											if Lm <= 0 then
												return 0;
											else
												return DataTableValue({0.3635,0.3635,0.3635,0.3635,0.3635,0.36215,0.3635,0.3635},L);
											end
										end
									else
										if Lm <= 0 then
											return 0;
										else
											return DataTableValue({1,1,1,1,1,1.002,1,1},L);
										end
									end
								elseif SN > "ARMQTYRAREMP" then
									if SN < "AUTOLVLTOILVL" then
										if SN > "ARMQTYUNCOMMP" then
											if SN == "ARMTYPEMP" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({9,9,20,30,15,25,12,32.725},L);
												end
											else
												return 0;
											end
										elseif SN == "ARMQTYUNCOMMP" then
											if Lm <= 0 then
												return 0;
											else
												return DataTableValue({22/30,22/30,0.85,0.85,0.82,0.852,0.85,0.75},L);
											end
										else
											return 0;
										end
									elseif SN > "AUTOLVLTOILVL" then
										if SN < "AWARDILVLAA" then
											if SN == "AWARDILVLA" then
												if Lm <= 75 then
													return L;
												elseif Lm <= 94 then
													return 5*L-304;
												elseif Lm <= 95 then
													return 173;
												elseif Lm <= 100 then
													return 5*L-304;
												elseif Lm <= 101 then
													return 197;
												elseif Lm <= 104 then
													return 4*L-206;
												elseif Lm <= 105 then
													return 218;
												elseif Lm <= 114 then
													return 3*L-30;
												elseif Lm <= 115 then
													return 314;
												elseif Lm <= 120 then
													return 5*L-245;
												elseif Lm <= 130 then
													return 2*L+143;
												elseif Lm <= 140 then
													return RoundDbl(LinFmod(1,435.4,445.4,131,140,L));
												else
													return CalcStat("AwardILvlA",140);
												end
											else
												return 0;
											end
										elseif SN > "AWARDILVLAA" then
											if SN == "AWARDILVLAB" then
												if Lm <= 74 then
													return CalcStat("AwardILvlA",L);
												elseif Lm <= 75 then
													return CalcStat("AwardILvlA",L)+1;
												else
													return CalcStat("AwardILvlA",L);
												end
											else
												return 0;
											end
										else
											if Lm <= 74 then
												return CalcStat("AwardILvlAOld",L);
											elseif Lm <= 75 then
												return CalcStat("AwardILvlAOld",L)+3;
											else
												return CalcStat("AwardILvlAOld",L);
											end
										end
									else
										return CalcStat("AwardILvlCC",L);
									end
								else
									if Lm <= 0 then
										return 0;
									else
										return DataTableValue({26/30,26/30,0.9,0.9,0.88,0.9,0.9,0.8},L);
									end
								end
							else
								return CalcStat("ArmCatProgB",ArmCodeIndex(C,1),L);
							end
						elseif SN > "AWARDILVLAC" then
							if SN < "AWARDILVLDD" then
								if SN < "AWARDILVLCB" then
									if SN < "AWARDILVLBA" then
										if SN > "AWARDILVLAOLD" then
											if SN == "AWARDILVLB" then
												if Lm <= 95 then
													return CalcStat("AwardILvlBOld",L);
												elseif Lm <= 96 then
													return CalcStat("AwardILvlBOld",95);
												else
													return CalcStat("AwardILvlBOld",L);
												end
											else
												return 0;
											end
										elseif SN == "AWARDILVLAOLD" then
											if Lm <= 120 then
												return CalcStat("AwardILvlA",L);
											elseif Lm <= 130 then
												return RoundDbl((14/15)*L+4081/15);
											elseif Lm <= 140 then
												return RoundDbl(LinFmod(1,435.4,445.4,131,140,L));
											else
												return CalcStat("AwardILvlAOld",140);
											end
										else
											return 0;
										end
									elseif SN > "AWARDILVLBA" then
										if SN < "AWARDILVLC" then
											if SN == "AWARDILVLBOLD" then
												if Lm <= 4 then
													return CalcStat("AwardILvlA",1);
												elseif Lm <= 50 then
													return CalcStat("AwardILvlA",L-4);
												elseif Lm <= 54 then
													return CalcStat("AwardILvlA",51);
												elseif Lm <= 75 then
													return CalcStat("AwardILvlA",L-4);
												elseif Lm <= 95 then
													return CalcStat("AwardILvlA",L)+4;
												elseif Lm <= 130 then
													return CalcStat("AwardILvlA",L);
												elseif Lm <= 135 then
													return RoundDbl(LinFmod(1,438.4,443.4,131,135,L));
												elseif Lm <= 140 then
													return RoundDbl(LinFmod(1,444.4,446.4,136,140,L));
												else
													return CalcStat("AwardILvlBOld",140);
												end
											else
												return 0;
											end
										elseif SN > "AWARDILVLC" then
											if SN == "AWARDILVLCA" then
												if Lm <= 95 then
													return CalcStat("AwardILvlC",L);
												elseif Lm <= 100 then
													return CalcStat("AwardILvlC",L)+4;
												else
													return CalcStat("AwardILvlC",L);
												end
											else
												return 0;
											end
										else
											if Lm <= 75 then
												return CalcStat("AwardILvlBOld",L);
											elseif Lm <= 95 then
												return CalcStat("AwardILvlA",L);
											else
												return CalcStat("AwardILvlBOld",L);
											end
										end
									else
										return CalcStat("AwardILvlB",L);
									end
								elseif SN > "AWARDILVLCB" then
									if SN < "AWARDILVLD" then
										if SN > "AWARDILVLCC" then
											if SN == "AWARDILVLCD" then
												if Lm <= 100 then
													return CalcStat("AwardILvlC",L);
												elseif Lm <= 105 then
													return CalcStat("AwardILvlC",L)+4;
												else
													return CalcStat("AwardILvlC",L);
												end
											else
												return 0;
											end
										elseif SN == "AWARDILVLCC" then
											if Lm <= 75 then
												return CalcStat("AwardILvlC",L);
											elseif Lm <= 95 then
												return CalcStat("AwardILvlC",L)+2;
											else
												return CalcStat("AwardILvlC",L);
											end
										else
											return 0;
										end
									elseif SN > "AWARDILVLD" then
										if SN < "AWARDILVLDB" then
											if SN == "AWARDILVLDA" then
												if Lm <= 105 then
													return CalcStat("AwardILvlD",L);
												elseif Lm <= 115 then
													return RoundDbl(LinFmod(1,300,336.5,106,115,L));
												else
													return CalcStat("AwardILvlD",L);
												end
											else
												return 0;
											end
										elseif SN > "AWARDILVLDB" then
											if SN == "AWARDILVLDC" then
												if Lm <= 105 then
													return CalcStat("AwardILvlD",L);
												elseif Lm <= 110 then
													return RoundDbl(LinFmod(1,300,320,106,115,L));
												elseif Lm <= 115 then
													return CalcStat("AwardILvlDC",110);
												else
													return CalcStat("AwardILvlD",L);
												end
											else
												return 0;
											end
										else
											if Lm <= 105 then
												return CalcStat("AwardILvlD",L);
											elseif Lm <= 115 then
												return RoundDbl(LinFmod(1,300,345,106,115,L));
											else
												return CalcStat("AwardILvlD",L);
											end
										end
									else
										if Lm <= 105 then
											return CalcStat("AwardILvlC",L);
										elseif Lm <= 115 then
											return 300;
										elseif Lm <= 130 then
											return CalcStat("AwardILvlC",L)+3;
										else
											return CalcStat("AwardILvlC",L);
										end
									end
								else
									if Lm <= 95 then
										return CalcStat("AwardILvlC",L);
									elseif Lm <= 99 then
										return CalcStat("AwardILvlC",L)+4;
									elseif Lm <= 100 then
										return CalcStat("AwardILvlC",L)+6;
									else
										return CalcStat("AwardILvlC",L);
									end
								end
							elseif SN > "AWARDILVLDD" then
								if SN < "AWARDILVLEC" then
									if SN < "AWARDILVLDG" then
										if SN > "AWARDILVLDE" then
											if SN == "AWARDILVLDF" then
												if Lm <= 105 then
													return CalcStat("AwardILvlD",L);
												elseif Lm <= 110 then
													return CalcStat("AwardILvlDF",111);
												elseif Lm <= 115 then
													return RoundDbl(LinFmod(1,299.5,320.5,106,115,L));
												else
													return CalcStat("AwardILvlD",L);
												end
											else
												return 0;
											end
										elseif SN == "AWARDILVLDE" then
											if Lm <= 105 then
												return CalcStat("AwardILvlD",L);
											elseif Lm <= 115 then
												return RoundDbl(LinFmod(1,300.4,326.4,106,115,L));
											else
												return CalcStat("AwardILvlD",L);
											end
										else
											return 0;
										end
									elseif SN > "AWARDILVLDG" then
										if SN < "AWARDILVLEA" then
											if SN == "AWARDILVLE" then
												if Lm <= 115 then
													return CalcStat("AwardILvlC",L);
												elseif Lm <= 120 then
													return 350;
												else
													return CalcStat("AwardILvlD",L);
												end
											else
												return 0;
											end
										elseif SN > "AWARDILVLEA" then
											if SN == "AWARDILVLEB" then
												if Lm <= 115 then
													return CalcStat("AwardILvlE",L);
												elseif Lm <= 119 then
													return RoundDbl(LinFmod(1,350,390,116,120,L));
												elseif Lm <= 120 then
													return CalcStat("AwardILvlE",120)+38;
												else
													return CalcStat("AwardILvlE",L);
												end
											else
												return 0;
											end
										else
											if Lm <= 115 then
												return CalcStat("AwardILvlE",L);
											elseif Lm <= 119 then
												return RoundDbl(LinFmod(1,350,378,116,120,L));
											elseif Lm <= 120 then
												return CalcStat("AwardILvlE",120)+26;
											else
												return CalcStat("AwardILvlE",L);
											end
										end
									else
										if Lm <= 105 then
											return CalcStat("AwardILvlD",L);
										elseif Lm <= 114 then
											return RoundDbl(LinFmod(1,300,336,106,115,L));
										elseif Lm <= 115 then
											return CalcStat("AwardILvlD",115)+40;
										else
											return CalcStat("AwardILvlD",L);
										end
									end
								elseif SN > "AWARDILVLEC" then
									if SN < "AWARDILVLF" then
										if SN > "AWARDILVLED" then
											if SN == "AWARDILVLEE" then
												if Lm <= 115 then
													return CalcStat("AwardILvlE",L);
												elseif Lm <= 119 then
													return RoundDbl(LinFmod(1,350,366,116,120,L));
												elseif Lm <= 120 then
													return CalcStat("AwardILvlE",120)+14;
												else
													return CalcStat("AwardILvlE",L);
												end
											else
												return 0;
											end
										elseif SN == "AWARDILVLED" then
											if Lm <= 115 then
												return CalcStat("AwardILvlE",L);
											elseif Lm <= 120 then
												return RoundDbl(LinFmod(1,350,370,116,120,L));
											else
												return CalcStat("AwardILvlE",L);
											end
										else
											return 0;
										end
									elseif SN > "AWARDILVLF" then
										if SN < "AWARDILVLFB" then
											if SN == "AWARDILVLFA" then
												if Lm <= 130 then
													return CalcStat("AwardILvlF",L);
												elseif Lm <= 140 then
													return RoundDbl(LinFmod(1,438.4,447.4,131,140,L));
												else
													return CalcStat("AwardILvlFA",140);
												end
											else
												return 0;
											end
										elseif SN > "AWARDILVLFB" then
											if SN == "AWARDILVLG" then
												if Lm <= 115 then
													return CalcStat("AwardILvlC",L);
												elseif Lm <= 120 then
													return CalcStat("AwardILvlC",L)+3;
												elseif Lm <= 130 then
													return 400;
												else
													return CalcStat("AwardILvlC",L);
												end
											else
												return 0;
											end
										else
											if Lm <= 119 then
												return CalcStat("AwardILvlF",L);
											elseif Lm <= 120 then
												return 380;
											elseif Lm <= 129 then
												return CalcStat("AwardILvlF",L);
											elseif Lm <= 130 then
												return 432;
											elseif Lm <= 140 then
												return RoundDbl(LinFmod(1,450.4,478.4,131,140,L));
											else
												return CalcStat("AwardILvlFB",140);
											end
										end
									else
										if Lm <= 40 then
											return LinFmod(1,1,40,1,40,L);
										elseif Lm <= 41 then
											return CalcStat("AwardILvlF",L-1);
										elseif Lm <= 49 then
											return LinFmod(1,41,48,42,49,L);
										elseif Lm <= 50 then
											return CalcStat("AwardILvlF",L-1);
										elseif Lm <= 55 then
											return LinFmod(1,51,55,51,55,L);
										elseif Lm <= 56 then
											return CalcStat("AwardILvlF",L-1);
										elseif Lm <= 59 then
											return LinFmod(1,56,58,57,59,L);
										elseif Lm <= 60 then
											return CalcStat("AwardILvlF",L-1);
										elseif Lm <= 64 then
											return RoundDbl(LinFmod(1,60.6,63,61,64,L));
										elseif Lm <= 65 then
											return CalcStat("AwardILvlF",L-1);
										elseif Lm <= 70 then
											return LinFmod(1,66,70,66,70,L);
										elseif Lm <= 71 then
											return CalcStat("AwardILvlF",L-1);
										elseif Lm <= 74 then
											return LinFmod(1,71,73,72,74,L);
										elseif Lm <= 75 then
											return CalcStat("AwardILvlF",L-1);
										elseif Lm <= 85 then
											return RoundDbl(LinFmod(1,80.4,123.4,76,85,L));
										elseif Lm <= 94 then
											return RoundDbl(LinFmod(1,130,169,86,94,L));
										elseif Lm <= 100 then
											return RoundDbl(LinFmod(1,175,198.4,95,100,L));
										elseif Lm <= 104 then
											return LinFmod(1,201,213,101,104,L);
										elseif Lm <= 105 then
											return 220;
										elseif Lm <= 114 then
											return RoundDbl(LinFmod(1,300,323,106,114,L));
										elseif Lm <= 115 then
											return 324;
										elseif Lm <= 120 then
											return RoundDbl(LinFmod(1,349.6,368.4,116,120,L));
										elseif Lm <= 130 then
											return RoundDbl(LinFmod(1,400.4,416.4,121,130,L));
										else
											return CalcStat("AwardILvlF",130);
										end
									end
								else
									if Lm <= 115 then
										return CalcStat("AwardILvlE",L);
									elseif Lm <= 120 then
										return RoundDbl(LinFmod(1,350,382,116,120,L));
									else
										return CalcStat("AwardILvlE",L);
									end
								end
							else
								if Lm <= 105 then
									return CalcStat("AwardILvlD",L);
								elseif Lm <= 114 then
									return RoundDbl(LinFmod(1,300,327,106,115,L));
								elseif Lm <= 115 then
									return CalcStat("AwardILvlD",115)+30;
								else
									return CalcStat("AwardILvlD",L);
								end
							end
						else
							if Lm <= 74 then
								return CalcStat("AwardILvlA",L);
							elseif Lm <= 75 then
								return CalcStat("AwardILvlA",L)+2;
							else
								return CalcStat("AwardILvlA",L);
							end
						end
					elseif SN > "AWARDILVLGA" then
						if SN < "BASEMORALE" then
							if SN < "AWARDLVLCAP" then
								if SN < "AWARDILVLI" then
									if SN < "AWARDILVLGD" then
										if SN > "AWARDILVLGB" then
											if SN == "AWARDILVLGC" then
												if Lm <= 120 then
													return CalcStat("AwardILvlG",L);
												elseif Lm <= 130 then
													return RoundDbl(LinFmod(1,400.4,408.3,121,130,L));
												else
													return CalcStat("AwardILvlG",L);
												end
											else
												return 0;
											end
										elseif SN == "AWARDILVLGB" then
											if Lm <= 120 then
												return CalcStat("AwardILvlG",L);
											elseif Lm <= 130 then
												return RoundDbl(LinFmod(1,400.4,418.4,121,130,L));
											else
												return CalcStat("AwardILvlG",L);
											end
										else
											return 0;
										end
									elseif SN > "AWARDILVLGD" then
										if SN < "AWARDILVLGF" then
											if SN == "AWARDILVLGE" then
												if Lm <= 120 then
													return CalcStat("AwardILvlG",L);
												elseif Lm <= 130 then
													return RoundDbl(LinFmod(1,400.4,434.4,121,130,L));
												else
													return CalcStat("AwardILvlG",L);
												end
											else
												return 0;
											end
										elseif SN > "AWARDILVLGF" then
											if SN == "AWARDILVLH" then
												if Lm <= 4 then
													return CalcStat("AwardILvlA",1);
												elseif Lm <= 50 then
													return CalcStat("AwardILvlA",L-4);
												elseif Lm <= 54 then
													return CalcStat("AwardILvlA",51);
												elseif Lm <= 75 then
													return CalcStat("AwardILvlA",L-4);
												else
													return L-4;
												end
											else
												return 0;
											end
										else
											if Lm <= 120 then
												return CalcStat("AwardILvlG",L);
											elseif Lm <= 130 then
												return RoundDbl(LinFmod(1,400.4,440.4,121,130,L));
											else
												return CalcStat("AwardILvlG",L);
											end
										end
									else
										if Lm <= 120 then
											return CalcStat("AwardILvlG",L);
										elseif Lm <= 130 then
											return RoundDbl(LinFmod(1,400.4,426.4,121,130,L));
										else
											return CalcStat("AwardILvlG",L);
										end
									end
								elseif SN > "AWARDILVLI" then
									if SN < "AWARDILVLJA" then
										if SN > "AWARDILVLIA" then
											if SN == "AWARDILVLJ" then
												if Lm <= 19 then
													return CalcStat("AwardILvlB",L);
												elseif Lm <= 40 then
													return CalcStat("AwardILvlA",L);
												elseif Lm <= 50 then
													return RoundDbl(LinFmod(1,40.4,47.6,41,50,L));
												elseif Lm <= 120 then
													return CalcStat("AwardILvlG",L);
												elseif Lm <= 130 then
													return CalcStat("AwardILvlD",L);
												elseif Lm <= 140 then
													return RoundDbl(LinFmod(1,450.4,460.4,131,140,L));
												else
													return CalcStat("AwardILvlJ",140);
												end
											else
												return 0;
											end
										elseif SN == "AWARDILVLIA" then
											if Lm <= 44 then
												return CalcStat("AwardILvlI",L);
											elseif Lm <= 55 then
												return RoundDblDown((L-45)/6)+51;
											elseif Lm <= 90 then
												return CalcStat("AwardILvlI",L);
											elseif Lm <= 104 then
												return RoundDblDown((L-91)/5)*25+165;
											elseif Lm <= 105 then
												return CalcStat("LvlToILvl",L);
											elseif Lm <= 110 then
												return CalcStat("LvlToILvl",106)+15;
											elseif Lm <= 115 then
												return CalcStat("LvlToILvl",115)-20;
											elseif Lm <= 120 then
												return CalcStat("LvlToILvl",116)+15;
											elseif Lm <= 125 then
												return CalcStat("LvlToILvl",121)+15;
											elseif Lm <= 130 then
												return CalcStat("LvlToILvl",130)-20;
											elseif Lm <= 135 then
												return CalcStat("LvlToILvl",131)+15;
											elseif Lm <= 140 then
												return CalcStat("LvlToILvl",140)-4;
											else
												return CalcStat("AwardILvlIA",140);
											end
										else
											return 0;
										end
									elseif SN > "AWARDILVLJA" then
										if SN < "AWARDILVLJC" then
											if SN == "AWARDILVLJB" then
												if Lm <= 130 then
													return CalcStat("AwardILvlJ",L);
												elseif Lm <= 140 then
													return RoundDbl(LinFmod(1,450.4,470.4,131,140,L));
												else
													return CalcStat("AwardILvlJB",140);
												end
											else
												return 0;
											end
										elseif SN > "AWARDILVLJC" then
											if SN == "AWARDILVLJD" then
												if Lm <= 130 then
													return CalcStat("AwardILvlJ",L);
												elseif Lm <= 140 then
													return RoundDbl(LinFmod(1,450.4,480.4,131,140,L));
												else
													return CalcStat("AwardILvlJD",140);
												end
											else
												return 0;
											end
										else
											if Lm <= 130 then
												return CalcStat("AwardILvlJ",L);
											elseif Lm <= 140 then
												return RoundDbl(LinFmod(1,450.4,475.4,131,140,L));
											else
												return CalcStat("AwardILvlJC",140);
											end
										end
									else
										if Lm <= 130 then
											return CalcStat("AwardILvlJ",L);
										elseif Lm <= 140 then
											return RoundDbl(LinFmod(1,450.4,465.4,131,140,L));
										else
											return CalcStat("AwardILvlJA",140);
										end
									end
								else
									if Lm <= 44 then
										return 1;
									elseif Lm <= 55 then
										return 52;
									elseif Lm <= 75 then
										return RoundDblDown((L-56)/5)*5+60;
									elseif Lm <= 85 then
										return RoundDblDown((L-76)/5)*29+100;
									elseif Lm <= 95 then
										return RoundDblDown((L-86)/5)*20+155;
									elseif Lm <= 100 then
										return RoundDblDown((L-96)/4)*10+190;
									elseif Lm <= 105 then
										return RoundDblDown((L-101)/4)*35+215;
									elseif Lm <= 110 then
										return CalcStat("LvlToILvl",106)+15;
									elseif Lm <= 115 then
										return CalcStat("LvlToILvl",115);
									elseif Lm <= 119 then
										return CalcStat("LvlToILvl",116)+15;
									elseif Lm <= 120 then
										return CalcStat("LvlToILvl",120);
									elseif Lm <= 125 then
										return CalcStat("LvlToILvl",121)+15;
									elseif Lm <= 130 then
										return CalcStat("LvlToILvl",130);
									elseif Lm <= 135 then
										return CalcStat("LvlToILvl",131)+15;
									elseif Lm <= 140 then
										return CalcStat("LvlToILvl",140);
									else
										return CalcStat("AwardILvlI",140);
									end
								end
							elseif SN > "AWARDLVLCAP" then
								if SN < "BASEAGILITY" then
									if SN < "AXEARMRENDPOS" then
										if SN > "AWARDLVLTOILVL" then
											if SN == "AXEARMOURREND" then
												return -CalcStat("AxeArmRendPos",L);
											else
												return 0;
											end
										elseif SN == "AWARDLVLTOILVL" then
											return CalcStat("AwardILvlC",L);
										else
											return 0;
										end
									elseif SN > "AXEARMRENDPOS" then
										if SN < "BALANCEOFMANEVADE" then
											if SN == "BALANCEOFMANBLOCK" then
												return CalcStat("BlockT",L,0.8);
											else
												return 0;
											end
										elseif SN > "BALANCEOFMANEVADE" then
											if SN == "BALANCEOFMANPARRY" then
												return CalcStat("ParryT",L,0.8);
											else
												return 0;
											end
										else
											return CalcStat("EvadeT",L,1.0);
										end
									else
										if Lm <= 7 then
											return RoundDbl(LinFmod(1,0.5,3.25,1,7,L));
										elseif Lm <= 14 then
											return RoundDbl(LinFmod(1,3.75,8.5,8,14,L));
										elseif Lm <= 30 then
											return RoundDbl(LinFmod(1,9,24.7,15,30,L));
										elseif Lm <= 39 then
											return RoundDbl(LinFmod(1,26.5,37,31,39,L));
										elseif Lm <= 50 then
											return RoundDbl(LinFmod(1,38.5,53,40,50,L));
										elseif Lm <= 52 then
											return 53;
										elseif Lm <= 60 then
											return RoundDbl(LinFmod(1,53.2,64.8,53,60,L));
										elseif Lm <= 77 then
											return RoundDbl(LinFmod(1,66.15,94.8,61,77,L));
										elseif Lm <= 113 then
											return RoundDbl(RoundDbl(L/3-0.5)*1.95+44.1);
										elseif Lm <= 141 then
											return 116;
										elseif Lm <= 181 then
											return RoundDbl(RoundDbl(L/4)*2+46);
										elseif Lm <= 217 then
											return RoundDbl(RoundDbl(L/4)*2+45);
										elseif Lm <= 221 then
											return 153;
										elseif Lm <= 299 then
											return 155;
										elseif Lm <= 308 then
											return RoundDbl(RoundDbl(L/2.6-0.4)*2-73);
										elseif Lm <= 317 then
											return RoundDbl(RoundDbl(L/2.74-0.75)*2-60);
										elseif Lm <= 325 then
											return 172;
										elseif Lm <= 326 then
											return 174;
										elseif Lm <= 500 then
											return ExpFmod(CalcStat("AxeArmRendPos",326),327,1,L,1);
										else
											return CalcStat("AxeArmRendPos",500);
										end
									end
								elseif SN > "BASEAGILITY" then
									if SN < "BASEFATE" then
										if SN > "BASEARMOUR" then
											if SN == "BASEENERGY" then
												if Lm <= 50 then
													return LinFmod(1,10,25,1,50,L)*N;
												elseif Lm <= 60 then
													return LinFmod(1,25,38,50,60,L)*N;
												elseif Lm <= 65 then
													return LinFmod(1,38,51,60,65,L)*N;
												elseif Lm <= 75 then
													return LinFmod(1,51,77,65,75,L)*N;
												elseif Lm <= 85 then
													return LinFmod(1,77,116,75,85,L)*N;
												elseif Lm <= 95 then
													return LinFmod(1,116,168,85,95,L)*N;
												elseif Lm <= 100 then
													return LinFmod(1,168,218,95,100,L)*N;
												elseif Lm <= 105 then
													return LinFmod(1,218,283,100,105,L)*N;
												elseif Lm <= 115 then
													return LinFmod(1,311,440,106,115,L,0)*N;
												elseif Lm <= 120 then
													return LinFmod(1,510,550,116,120,L,0)*N;
												elseif Lm <= 130 then
													return LinFmod(1,630,840,121,130,L,0)*N;
												else
													return LinFmod(1,970,1660,131,140,L,0)*N;
												end
											else
												return 0;
											end
										elseif SN == "BASEARMOUR" then
											return RoundDblUp(CalcStat("BaseMain",L,N));
										else
											return 0;
										end
									elseif SN > "BASEFATE" then
										if SN < "BASEMAIN" then
											if SN == "BASEHEALTH" then
												if Lm <= 50 then
													return LinFmod(1,4,30,1,50,L)*N;
												elseif Lm <= 60 then
													return LinFmod(1,30,45,50,60,L)*N;
												elseif Lm <= 65 then
													return LinFmod(1,45,60,60,65,L)*N;
												elseif Lm <= 75 then
													return LinFmod(1,60,90,65,75,L)*N;
												elseif Lm <= 85 then
													return LinFmod(1,90,135,75,85,L)*N;
												elseif Lm <= 95 then
													return LinFmod(1,135,195,85,95,L)*N;
												elseif Lm <= 100 then
													return LinFmod(1,195,300,95,100,L)*N;
												elseif Lm <= 105 then
													return LinFmod(1,300,390,100,105,L)*N;
												elseif Lm <= 115 then
													return LinFmod(1,430,600,106,115,L,0)*N;
												elseif Lm <= 120 then
													return LinFmod(1,690,750,116,120,L,0)*N;
												elseif Lm <= 130 then
													return LinFmod(1,860,1140,121,130,L,0)*N;
												else
													return LinFmod(1,1310,2250,131,140,L,0)*N;
												end
											else
												return 0;
											end
										elseif SN > "BASEMAIN" then
											if SN == "BASEMIGHT" then
												return RoundDbl(CalcStat("BaseMain",L,N));
											else
												return 0;
											end
										else
											if Lm <= 50 then
												return LinFmod(1,1,10,1,50,L)*N;
											elseif Lm <= 60 then
												return LinFmod(1,10,15,50,60,L)*N;
											elseif Lm <= 65 then
												return LinFmod(1,15,20,60,65,L)*N;
											elseif Lm <= 75 then
												return LinFmod(1,20,30,65,75,L)*N;
											elseif Lm <= 85 then
												return LinFmod(1,30,45,75,85,L)*N;
											elseif Lm <= 95 then
												return LinFmod(1,45,65,85,95,L)*N;
											elseif Lm <= 100 then
												return LinFmod(1,65,100,95,100,L)*N;
											elseif Lm <= 105 then
												return LinFmod(1,100,130,100,105,L)*N;
											elseif Lm <= 115 then
												return LinFmod(1,143,200,106,115,L,0)*N;
											elseif Lm <= 120 then
												return LinFmod(1,230,250,116,120,L,0)*N;
											elseif Lm <= 130 then
												return LinFmod(1,288,380,121,130,L,0)*N;
											else
												return LinFmod(1,440,750,131,140,L,0)*N;
											end
										end
									else
										return RoundDbl(CalcStat("BaseEnergy",L,N));
									end
								else
									return RoundDbl(CalcStat("BaseMain",L,N));
								end
							else
								return 140;
							end
						elseif SN > "BASEMORALE" then
							if SN < "BEORNINGCDAGILITYTOEVADE" then
								if SN < "BEOBEARFORMCRITDEF" then
									if SN < "BASEWILL" then
										if SN > "BASEPOWER" then
											if SN == "BASEVITALITY" then
												return RoundDbl(CalcStat("BaseHealth",L,N));
											else
												return 0;
											end
										elseif SN == "BASEPOWER" then
											return RoundDblDown(CalcStat("BaseEnergy",L,N));
										else
											return 0;
										end
									elseif SN > "BASEWILL" then
										if SN < "BATTLELOREPHYMAS" then
											if SN == "BATTLELOREMAS" then
												if Lm <= 105 then
													return CalcStat("Mastery",L,2.5);
												elseif Lm <= 119 then
													return CalcStat("MasteryT",L,1.2);
												elseif Lm <= 120 then
													return CalcStat("MasteryT",L,1.6);
												elseif Lm <= 129 then
													return CalcStat("MasteryT",L,1.2);
												elseif Lm <= 130 then
													return CalcStat("MasteryT",L,1.6);
												elseif Lm <= 139 then
													return CalcStat("MasteryT",L,1.2);
												else
													return CalcStat("MasteryT",L,1.6);
												end
											else
												return 0;
											end
										elseif SN > "BATTLELOREPHYMAS" then
											if SN == "BATTLELORETACMAS" then
												return CalcStat("BattleLoreMas",L);
											else
												return 0;
											end
										else
											return CalcStat("BattleLoreMas",L);
										end
									else
										return RoundDbl(CalcStat("BaseMain",L,N));
									end
								elseif SN > "BEOBEARFORMCRITDEF" then
									if SN < "BEOFERALPRESFATE" then
										if SN > "BEOEMISSARYFATE" then
											if SN == "BEOFATE" then
												return CalcStat("FateT",L,CalcStat("Trait567810Choice",N)*0.4);
											else
												return 0;
											end
										elseif SN == "BEOEMISSARYFATE" then
											return CalcStat("FateT",L,1.0);
										else
											return 0;
										end
									elseif SN > "BEOFERALPRESFATE" then
										if SN < "BEOMIGHTOFTHEWILDMIGHT" then
											if SN == "BEOFEWINNUMBERFATE" then
												return -CalcStat("FateT",L,0.4);
											else
												return 0;
											end
										elseif SN > "BEOMIGHTOFTHEWILDMIGHT" then
											if SN == "BEORNINGCDAGILITYTOCRITHIT" then
												return 1;
											else
												return 0;
											end
										else
											return CalcStat("MightT",L,1.0);
										end
									else
										return CalcStat("FateT",L,1.0);
									end
								else
									return CalcStat("CritDefT",L,0.4);
								end
							elseif SN > "BEORNINGCDAGILITYTOEVADE" then
								if SN < "BEORNINGCDBASEFATE" then
									if SN < "BEORNINGCDARMOURTOPHYMIT" then
										if SN > "BEORNINGCDAGILITYTOPHYMAS" then
											if SN == "BEORNINGCDARMOURTOOFMIT" then
												return 0.2;
											else
												return 0;
											end
										elseif SN == "BEORNINGCDAGILITYTOPHYMAS" then
											return 2;
										else
											return 0;
										end
									elseif SN > "BEORNINGCDARMOURTOPHYMIT" then
										if SN < "BEORNINGCDARMOURTYPE" then
											if SN == "BEORNINGCDARMOURTOTACMIT" then
												return 0.2;
											else
												return 0;
											end
										elseif SN > "BEORNINGCDARMOURTYPE" then
											if SN == "BEORNINGCDBASEAGILITY" then
												return CalcStat("ClassBaseAgilityM",L);
											else
												return 0;
											end
										else
											return 3;
										end
									else
										return 1;
									end
								elseif SN > "BEORNINGCDBASEFATE" then
									if SN < "BEORNINGCDBASEMORALE" then
										if SN < "BEORNINGCDBASEICPR" then
											if SN == "BEORNINGCDBASEICMR" then
												return CalcStat("ClassBaseICMRM",L);
											else
												return 0;
											end
										elseif SN > "BEORNINGCDBASEICPR" then
											if SN == "BEORNINGCDBASEMIGHT" then
												return CalcStat("ClassBaseMightM",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseICPR",L);
										end
									elseif SN > "BEORNINGCDBASEMORALE" then
										if SN < "BEORNINGCDBASENCPR" then
											if SN == "BEORNINGCDBASENCMR" then
												return CalcStat("ClassBaseNCMRM",L);
											else
												return 0;
											end
										elseif SN > "BEORNINGCDBASENCPR" then
											if SN == "BEORNINGCDBASEPOWER" then
												return 10;
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseNCPR",L);
										end
									else
										return CalcStat("ClassBaseMorale",L);
									end
								else
									return CalcStat("ClassBaseFate",L);
								end
							else
								return 2;
							end
						else
							return RoundDblDown(CalcStat("BaseHealth",L,N));
						end
					else
						if Lm <= 120 then
							return CalcStat("AwardILvlG",L);
						elseif Lm <= 130 then
							return RoundDbl(LinFmod(1,400.4,412.4,121,130,L));
						else
							return CalcStat("AwardILvlG",L);
						end
					end
				elseif SN > "BEORNINGCDBASEVITALITY" then
					if SN < "BRATMITMEDIUM" then
						if SN < "BEORNINGRDTRAITVITALITY" then
							if SN < "BEORNINGCDPHYMITTOOFMIT" then
								if SN < "BEORNINGCDFATETOTACMIT" then
									if SN < "BEORNINGCDFATETOCRITHIT" then
										if SN > "BEORNINGCDBASEWILL" then
											if SN == "BEORNINGCDCANBLOCK" then
												if Lm <= 5 then
													return 0;
												else
													return 1;
												end
											else
												return 0;
											end
										elseif SN == "BEORNINGCDBASEWILL" then
											return CalcStat("ClassBaseWillM",L);
										else
											return 0;
										end
									elseif SN > "BEORNINGCDFATETOCRITHIT" then
										if SN < "BEORNINGCDFATETOICMR" then
											if SN == "BEORNINGCDFATETOFINESSE" then
												return 0.5;
											else
												return 0;
											end
										elseif SN > "BEORNINGCDFATETOICMR" then
											if SN == "BEORNINGCDFATETORESIST" then
												return 0.5;
											else
												return 0;
											end
										else
											return 0.05;
										end
									else
										return 1.5;
									end
								elseif SN > "BEORNINGCDFATETOTACMIT" then
									if SN < "BEORNINGCDMIGHTTOEVADE" then
										if SN > "BEORNINGCDMIGHTTOBLOCK" then
											if SN == "BEORNINGCDMIGHTTOCRITHIT" then
												return 1;
											else
												return 0;
											end
										elseif SN == "BEORNINGCDMIGHTTOBLOCK" then
											return 2;
										else
											return 0;
										end
									elseif SN > "BEORNINGCDMIGHTTOEVADE" then
										if SN < "BEORNINGCDMIGHTTOPHYMAS" then
											if SN == "BEORNINGCDMIGHTTOPARRY" then
												return 3;
											else
												return 0;
											end
										elseif SN > "BEORNINGCDMIGHTTOPHYMAS" then
											if SN == "BEORNINGCDMIGHTTOTACMAS" then
												return 2.5;
											else
												return 0;
											end
										else
											return 2.5;
										end
									else
										return 1;
									end
								else
									return 1;
								end
							elseif SN > "BEORNINGCDPHYMITTOOFMIT" then
								if SN < "BEORNINGCDWILLTOTACMAS" then
									if SN < "BEORNINGCDVITALITYTONCMR" then
										if SN > "BEORNINGCDTACMASTOOUTHEAL" then
											if SN == "BEORNINGCDVITALITYTOMORALE" then
												return 4.8;
											else
												return 0;
											end
										elseif SN == "BEORNINGCDTACMASTOOUTHEAL" then
											return 1;
										else
											return 0;
										end
									elseif SN > "BEORNINGCDVITALITYTONCMR" then
										if SN < "BEORNINGCDWILLTOEVADE" then
											if SN == "BEORNINGCDVITALITYTORESIST" then
												return 1;
											else
												return 0;
											end
										elseif SN > "BEORNINGCDWILLTOEVADE" then
											if SN == "BEORNINGCDWILLTORESIST" then
												return 1;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 0.12;
									end
								elseif SN > "BEORNINGCDWILLTOTACMAS" then
									if SN < "BEORNINGRDPSVONENAME" then
										if SN > "BEORNINGCDWILLTOTACMIT" then
											if SN == "BEORNINGRDPSVONEFATE" then
												return CalcStat("BeoEmissaryFate",L);
											else
												return 0;
											end
										elseif SN == "BEORNINGCDWILLTOTACMIT" then
											return 1;
										else
											return 0;
										end
									elseif SN > "BEORNINGRDPSVONENAME" then
										if SN < "BEORNINGRDTRAITFATE" then
											if SN == "BEORNINGRDPSVTWONAME" then
												return "";
											else
												return 0;
											end
										elseif SN > "BEORNINGRDTRAITFATE" then
											if SN == "BEORNINGRDTRAITMIGHT" then
												return CalcStat("BeoMightoftheWildMight",L);
											else
												return 0;
											end
										else
											return CalcStat("BeoFewinNumberFate",L);
										end
									else
										return "Emissary";
									end
								else
									return 2;
								end
							else
								return 1;
							end
						elseif SN > "BEORNINGRDTRAITVITALITY" then
							if SN < "BPEPBONUS" then
								if SN < "BLOCKPRATPA" then
									if SN < "BLOCK" then
										if SN > "BEOTHICKHIDEVITALITY" then
											if SN == "BEOVITALITYINCREASE" then
												return CalcStat("VitalityT",L,CalcStat("Trait567810Choice",N)*0.4);
											else
												return 0;
											end
										elseif SN == "BEOTHICKHIDEVITALITY" then
											return CalcStat("VitalityT",L,1.0);
										else
											return 0;
										end
									elseif SN > "BLOCK" then
										if SN < "BLOCKPPRAT" then
											if SN == "BLOCKPBONUS" then
												return CalcStat("BPEPBonus",L);
											else
												return 0;
											end
										elseif SN > "BLOCKPPRAT" then
											if SN == "BLOCKPRATP" then
												return CalcStat("BPEPRatP",L,N);
											else
												return 0;
											end
										else
											return CalcStat("BPEPPRat",L,N);
										end
									else
										return CalcStat("BPE",L,N);
									end
								elseif SN > "BLOCKPRATPA" then
									if SN < "BLOCKPRATPCAP" then
										if SN > "BLOCKPRATPB" then
											if SN == "BLOCKPRATPC" then
												return CalcStat("BPEPRatPC",L);
											else
												return 0;
											end
										elseif SN == "BLOCKPRATPB" then
											return CalcStat("BPEPRatPB",L);
										else
											return 0;
										end
									elseif SN > "BLOCKPRATPCAP" then
										if SN < "BLOCKT" then
											if SN == "BLOCKPRATPCAPR" then
												return CalcStat("BPEPRatPCapR",L);
											else
												return 0;
											end
										elseif SN > "BLOCKT" then
											if SN == "BPE" then
												return StatLinInter("PntMPBPE","ItemProg","ProgBBPE","",L,N,0);
											else
												return 0;
											end
										else
											return CalcStat("BPET",L,N);
										end
									else
										return CalcStat("BPEPRatPCap",L);
									end
								else
									return CalcStat("BPEPRatPA",L);
								end
							elseif SN > "BPEPBONUS" then
								if SN < "BPEPRATPCAPR" then
									if SN < "BPEPRATPA" then
										if SN > "BPEPPRAT" then
											if SN == "BPEPRATP" then
												return CalcPercAB(CalcStat("BPEPRatPA",L),CalcStat("BPEPRatPB",L),CalcStat("BPEPRatPCap",L),N);
											else
												return 0;
											end
										elseif SN == "BPEPPRAT" then
											return CalcRatAB(CalcStat("BPEPRatPA",L),CalcStat("BPEPRatPB",L),CalcStat("BPEPRatPCapR",L),N);
										else
											return 0;
										end
									elseif SN > "BPEPRATPA" then
										if SN < "BPEPRATPC" then
											if SN == "BPEPRATPB" then
												return CalcStat("BRatRounded",L,"BRatStandard");
											else
												return 0;
											end
										elseif SN > "BPEPRATPC" then
											if SN == "BPEPRATPCAP" then
												return 13;
											else
												return 0;
											end
										else
											return 0.5;
										end
									else
										return 39;
									end
								elseif SN > "BPEPRATPCAPR" then
									if SN < "BRATDEVHIT" then
										if SN > "BPET" then
											if SN == "BRATCRITMAGN" then
												if Lm <= 50 then
													return LinFmod(1,700,7000,1,50,L);
												elseif Lm <= 60 then
													return LinFmod(1,7000,10500,50,60,L);
												elseif Lm <= 65 then
													return LinFmod(1,10500,14000,60,65,L);
												elseif Lm <= 75 then
													return LinFmod(1,14000,21000,65,75,L);
												elseif Lm <= 85 then
													return LinFmod(1,21000,31500,75,85,L);
												elseif Lm <= 95 then
													return LinFmod(1,31500,46000,85,95,L);
												elseif Lm <= 100 then
													return LinFmod(1,46000,64000,95,100,L);
												elseif Lm <= 105 then
													return LinFmod(1,64000,85000,100,105,L);
												elseif Lm <= 115 then
													return LinFmod(1,94000,128000,106,115,L);
												elseif Lm <= 120 then
													return LinFmod(1,147000,160000,116,120,L);
												elseif Lm <= 130 then
													return LinFmod(1,184000,240000,121,130,L);
												elseif Lm <= 140 then
													return LinFmod(1,276000,480000,131,140,L);
												else
													return LinFmod(1,552000,960000,141,150,L);
												end
											else
												return 0;
											end
										elseif SN == "BPET" then
											return StatLinInter("PntMPBPE","TraitProg","ProgBBPE","",L,N,0);
										else
											return 0;
										end
									elseif SN > "BRATDEVHIT" then
										if SN < "BRATMITHEAVY" then
											if SN == "BRATEXTRA" then
												if Lm <= 50 then
													return LinFmod(1,300,3000,1,50,L);
												elseif Lm <= 60 then
													return LinFmod(1,3000,4500,50,60,L);
												elseif Lm <= 65 then
													return LinFmod(1,4500,6000,60,65,L);
												elseif Lm <= 75 then
													return LinFmod(1,6000,9000,65,75,L);
												elseif Lm <= 85 then
													return LinFmod(1,9000,13500,75,85,L);
												elseif Lm <= 95 then
													return LinFmod(1,13500,19500,85,95,L);
												elseif Lm <= 100 then
													return LinFmod(1,19500,27100,95,100,L);
												elseif Lm <= 105 then
													return LinFmod(1,27100,36000,100,105,L);
												elseif Lm <= 115 then
													return LinFmod(1,40000,54000,106,115,L);
												elseif Lm <= 120 then
													return LinFmod(1,62000,68000,116,120,L);
												elseif Lm <= 130 then
													return LinFmod(1,78000,102000,121,130,L);
												elseif Lm <= 140 then
													return LinFmod(1,117000,204000,131,140,L);
												else
													return LinFmod(1,235000,410000,141,150,L);
												end
											else
												return 0;
											end
										elseif SN > "BRATMITHEAVY" then
											if SN == "BRATMITLIGHT" then
												if Lm <= 50 then
													return LinFmod(1,105,1330,1,50,L);
												elseif Lm <= 60 then
													return LinFmod(1,1330,2000,50,60,L);
												elseif Lm <= 65 then
													return LinFmod(1,2000,2660,60,65,L);
												elseif Lm <= 75 then
													return LinFmod(1,2660,4000,65,75,L);
												elseif Lm <= 85 then
													return LinFmod(1,4000,6000,75,85,L);
												elseif Lm <= 95 then
													return LinFmod(1,6000,8700,85,95,L);
												elseif Lm <= 100 then
													return LinFmod(1,8700,12100,95,100,L);
												elseif Lm <= 105 then
													return LinFmod(1,12100,16100,100,105,L);
												elseif Lm <= 115 then
													return LinFmod(1,17600,24000,106,115,L);
												elseif Lm <= 120 then
													return LinFmod(1,27300,30000,116,120,L);
												elseif Lm <= 130 then
													return LinFmod(1,35000,45000,121,130,L);
												elseif Lm <= 140 then
													return LinFmod(1,52000,91000,131,140,L);
												else
													return LinFmod(1,104000,181000,141,150,L);
												end
											else
												return 0;
											end
										else
											return CalcStat("BRatStandard",L);
										end
									else
										if Lm <= 50 then
											return LinFmod(1,400,4000,1,50,L);
										elseif Lm <= 60 then
											return LinFmod(1,4000,6000,50,60,L);
										elseif Lm <= 65 then
											return LinFmod(1,6000,8000,60,65,L);
										elseif Lm <= 75 then
											return LinFmod(1,8000,12000,65,75,L);
										elseif Lm <= 85 then
											return LinFmod(1,12000,18000,75,85,L);
										elseif Lm <= 95 then
											return LinFmod(1,18000,26000,85,95,L);
										elseif Lm <= 100 then
											return LinFmod(1,26000,36000,95,100,L);
										elseif Lm <= 105 then
											return LinFmod(1,36000,48000,100,105,L);
										elseif Lm <= 115 then
											return LinFmod(1,53000,72000,106,115,L);
										elseif Lm <= 120 then
											return LinFmod(1,83000,90000,116,120,L);
										elseif Lm <= 130 then
											return LinFmod(1,104000,135000,121,130,L);
										elseif Lm <= 140 then
											return LinFmod(1,155000,270000,131,140,L);
										else
											return LinFmod(1,311000,540000,141,150,L);
										end
									end
								else
									return CalcStat("BPEPRatPB",L)*CalcStat("BPEPRatPC",L);
								end
							else
								return 0;
							end
						else
							return CalcStat("BeoThickHideVitality",L);
						end
					elseif SN > "BRATMITMEDIUM" then
						if SN < "BRAWLERCDFATETORESIST" then
							if SN < "BRAWLERCDBASEICMR" then
								if SN < "BRAWLERCDAGILITYTOPARRY" then
									if SN < "BRATPROGB" then
										if SN > "BRATOUTHEAL" then
											if SN == "BRATPARTBPE" then
												if Lm <= 50 then
													return LinFmod(1,350,3500,1,50,L);
												elseif Lm <= 60 then
													return LinFmod(1,3500,5300,50,60,L);
												elseif Lm <= 65 then
													return LinFmod(1,5300,7100,60,65,L);
												elseif Lm <= 75 then
													return LinFmod(1,7100,10700,65,75,L);
												elseif Lm <= 85 then
													return LinFmod(1,10700,16100,75,85,L);
												elseif Lm <= 95 then
													return LinFmod(1,16100,23300,85,95,L);
												elseif Lm <= 100 then
													return LinFmod(1,23300,32000,95,100,L);
												elseif Lm <= 105 then
													return LinFmod(1,32000,43000,100,105,L);
												elseif Lm <= 115 then
													return LinFmod(1,47000,65000,106,115,L);
												elseif Lm <= 120 then
													return LinFmod(1,75000,81000,116,120,L);
												elseif Lm <= 130 then
													return LinFmod(1,93000,122000,121,130,L);
												elseif Lm <= 140 then
													return LinFmod(1,140000,244000,131,140,L);
												else
													return LinFmod(1,281000,488000,141,150,L);
												end
											else
												return 0;
											end
										elseif SN == "BRATOUTHEAL" then
											if Lm <= 50 then
												return LinFmod(1,450,4500,1,50,L);
											elseif Lm <= 60 then
												return LinFmod(1,4500,6800,50,60,L);
											elseif Lm <= 65 then
												return LinFmod(1,6800,9100,60,65,L);
											elseif Lm <= 75 then
												return LinFmod(1,9100,13700,65,75,L);
											elseif Lm <= 85 then
												return LinFmod(1,13700,20600,75,85,L);
											elseif Lm <= 95 then
												return LinFmod(1,20600,29800,85,95,L);
											elseif Lm <= 100 then
												return LinFmod(1,29800,41000,95,100,L);
											elseif Lm <= 105 then
												return LinFmod(1,41000,55000,100,105,L);
											elseif Lm <= 115 then
												return LinFmod(1,61000,83000,106,115,L);
											elseif Lm <= 120 then
												return LinFmod(1,95000,104000,116,120,L);
											elseif Lm <= 130 then
												return LinFmod(1,120000,156000,121,130,L);
											elseif Lm <= 140 then
												return LinFmod(1,179000,312000,131,140,L);
											else
												return LinFmod(1,360000,620000,141,150,L);
											end
										else
											return 0;
										end
									elseif SN > "BRATPROGB" then
										if SN < "BRATSTANDARD" then
											if SN == "BRATROUNDED" then
												if Lm <= 50 then
													return RoundDbl(CalcStat(C,L));
												elseif Lm <= 105 then
													return RoundDbl(CalcStat(C,L),-1);
												elseif Lm <= 115 then
													return RoundDbl(CalcStat(C,L),-2);
												elseif Lm <= 130 then
													return RoundDbl(CalcStat(C,L),-1);
												else
													return RoundDbl(CalcStat(C,L),-2);
												end
											else
												return 0;
											end
										elseif SN > "BRATSTANDARD" then
											if SN == "BRAWLERCDAGILITYTOEVADE" then
												return 3;
											else
												return 0;
											end
										else
											if Lm <= 50 then
												return LinFmod(1,200,2000,1,50,L);
											elseif Lm <= 60 then
												return LinFmod(1,2000,3000,50,60,L);
											elseif Lm <= 65 then
												return LinFmod(1,3000,4000,60,65,L);
											elseif Lm <= 75 then
												return LinFmod(1,4000,6000,65,75,L);
											elseif Lm <= 85 then
												return LinFmod(1,6000,9000,75,85,L);
											elseif Lm <= 95 then
												return LinFmod(1,9000,13000,85,95,L);
											elseif Lm <= 100 then
												return LinFmod(1,13000,18100,95,100,L);
											elseif Lm <= 105 then
												return LinFmod(1,18100,24100,100,105,L);
											elseif Lm <= 115 then
												return LinFmod(1,26500,36000,106,115,L);
											elseif Lm <= 120 then
												return LinFmod(1,41000,45000,116,120,L);
											elseif Lm <= 130 then
												return LinFmod(1,52000,68000,121,130,L);
											elseif Lm <= 140 then
												return LinFmod(1,78000,136000,131,140,L);
											else
												return LinFmod(1,156000,272000,141,150,L);
											end
										end
									else
										if Lm <= 50 then
											return RoundDbl(CalcStat(C,L));
										else
											return CalcStat(C,L);
										end
									end
								elseif SN > "BRAWLERCDAGILITYTOPARRY" then
									if SN < "BRAWLERCDARMOURTOTACMIT" then
										if SN > "BRAWLERCDARMOURTOOFMIT" then
											if SN == "BRAWLERCDARMOURTOPHYMIT" then
												return 1;
											else
												return 0;
											end
										elseif SN == "BRAWLERCDARMOURTOOFMIT" then
											return 0.2;
										else
											return 0;
										end
									elseif SN > "BRAWLERCDARMOURTOTACMIT" then
										if SN < "BRAWLERCDBASEAGILITY" then
											if SN == "BRAWLERCDARMOURTYPE" then
												return 3;
											else
												return 0;
											end
										elseif SN > "BRAWLERCDBASEAGILITY" then
											if SN == "BRAWLERCDBASEFATE" then
												return CalcStat("ClassBaseFate",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseAgilityM",L);
										end
									else
										return 0.2;
									end
								else
									return 2;
								end
							elseif SN > "BRAWLERCDBASEICMR" then
								if SN < "BRAWLERCDBASEVITALITY" then
									if SN < "BRAWLERCDBASEMORALE" then
										if SN > "BRAWLERCDBASEICPR" then
											if SN == "BRAWLERCDBASEMIGHT" then
												return CalcStat("ClassBaseMightH",L);
											else
												return 0;
											end
										elseif SN == "BRAWLERCDBASEICPR" then
											return CalcStat("ClassBaseICPR",L);
										else
											return 0;
										end
									elseif SN > "BRAWLERCDBASEMORALE" then
										if SN < "BRAWLERCDBASENCPR" then
											if SN == "BRAWLERCDBASENCMR" then
												return CalcStat("ClassBaseNCMRL",L);
											else
												return 0;
											end
										elseif SN > "BRAWLERCDBASENCPR" then
											if SN == "BRAWLERCDBASEPOWER" then
												return CalcStat("ClassBasePower",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseNCPR",L);
										end
									else
										return CalcStat("ClassBaseMorale",L);
									end
								elseif SN > "BRAWLERCDBASEVITALITY" then
									if SN < "BRAWLERCDFATETOFINESSE" then
										if SN > "BRAWLERCDBASEWILL" then
											if SN == "BRAWLERCDFATETOCRITHIT" then
												return 1.5;
											else
												return 0;
											end
										elseif SN == "BRAWLERCDBASEWILL" then
											return CalcStat("ClassBaseWillL",L);
										else
											return 0;
										end
									elseif SN > "BRAWLERCDFATETOFINESSE" then
										if SN < "BRAWLERCDFATETONCPR" then
											if SN == "BRAWLERCDFATETOICMR" then
												return 0.025;
											else
												return 0;
											end
										elseif SN > "BRAWLERCDFATETONCPR" then
											if SN == "BRAWLERCDFATETOPOWER" then
												return 1;
											else
												return 0;
											end
										else
											return 0.05;
										end
									else
										return 0.5;
									end
								else
									return CalcStat("ClassBaseVitality",L);
								end
							else
								return CalcStat("ClassBaseICMRL",L);
							end
						elseif SN > "BRAWLERCDFATETORESIST" then
							if SN < "BRAWLERCDWILLTOTACMIT" then
								if SN < "BRAWLERCDPHYMITTOOFMIT" then
									if SN < "BRAWLERCDMIGHTTOEVADE" then
										if SN > "BRAWLERCDFATETOTACMIT" then
											if SN == "BRAWLERCDHASPOWER" then
												return 1;
											else
												return 0;
											end
										elseif SN == "BRAWLERCDFATETOTACMIT" then
											return 0.5;
										else
											return 0;
										end
									elseif SN > "BRAWLERCDMIGHTTOEVADE" then
										if SN < "BRAWLERCDMIGHTTOPARRY" then
											if SN == "BRAWLERCDMIGHTTOFINESSE" then
												return 1;
											else
												return 0;
											end
										elseif SN > "BRAWLERCDMIGHTTOPARRY" then
											if SN == "BRAWLERCDMIGHTTOPHYMAS" then
												return 3;
											else
												return 0;
											end
										else
											return 2;
										end
									else
										return 3;
									end
								elseif SN > "BRAWLERCDPHYMITTOOFMIT" then
									if SN < "BRAWLERCDVITALITYTONCMR" then
										if SN > "BRAWLERCDTACMASTOOUTHEAL" then
											if SN == "BRAWLERCDVITALITYTOMORALE" then
												return 5;
											else
												return 0;
											end
										elseif SN == "BRAWLERCDTACMASTOOUTHEAL" then
											return 1;
										else
											return 0;
										end
									elseif SN > "BRAWLERCDVITALITYTONCMR" then
										if SN < "BRAWLERCDWILLTORESIST" then
											if SN == "BRAWLERCDVITALITYTORESIST" then
												return 0.5;
											else
												return 0;
											end
										elseif SN > "BRAWLERCDWILLTORESIST" then
											if SN == "BRAWLERCDWILLTOTACMAS" then
												return 1;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 0.12;
									end
								else
									return 1;
								end
							elseif SN > "BRAWLERCDWILLTOTACMIT" then
								if SN < "BRGTRICKCNTDEFCRITDEF" then
									if SN < "BRGREVWEAKNCRITDEF" then
										if SN > "BRGALLINFINESSE" then
											if SN == "BRGALLINONEXPFINESSE" then
												return -CalcStat("Finesse",L,2);
											else
												return 0;
											end
										elseif SN == "BRGALLINFINESSE" then
											return CalcStat("Finesse",L,2);
										else
											return 0;
										end
									elseif SN > "BRGREVWEAKNCRITDEF" then
										if SN < "BRGREVWEAKNRESIST" then
											if SN == "BRGREVWEAKNFINESSE" then
												return -CalcStat("FinesseT",L,0.4);
											else
												return 0;
											end
										elseif SN > "BRGREVWEAKNRESIST" then
											if SN == "BRGTRICKCNTDEFBPE" then
												return -CalcStat("BPET",L,1.2);
											else
												return 0;
											end
										else
											return -CalcStat("ResistT",L,0.4);
										end
									else
										return -CalcStat("CritDefT",L,0.4);
									end
								elseif SN > "BRGTRICKCNTDEFCRITDEF" then
									if SN < "BRWINNSTRPRECISIONCRITHIT" then
										if SN < "BRWDEFPOSTUREPHYMAS" then
											if SN == "BRWAGGPOSTUREPHYMIT" then
												return -CalcStat("PhyMitT",L,3);
											else
												return 0;
											end
										elseif SN > "BRWDEFPOSTUREPHYMAS" then
											if SN == "BRWINNSTRCLEVTECHFINESSE" then
												return CalcStat("FinesseT",L,CalcStat("Trait13510Choice",N)*0.2);
											else
												return 0;
											end
										else
											return -CalcStat("PhyMasT",L,4);
										end
									elseif SN > "BRWINNSTRPRECISIONCRITHIT" then
										if SN < "BRWMIGHTINCREASE" then
											if SN == "BRWMAELSTROMMIGHT" then
												return CalcStat("MightT",L,2);
											else
												return 0;
											end
										elseif SN > "BRWMIGHTINCREASE" then
											if SN == "BRWRETINTENSITYCRITHIT" then
												return CalcStat("CritHitT",L,CalcStat("Trait234Choice",N));
											else
												return 0;
											end
										else
											return CalcStat("MightT",L,CalcStat("Trait567810Choice",N)*0.4);
										end
									else
										return CalcStat("CritHitT",L,CalcStat("Trait13510Choice",N)*0.2);
									end
								else
									return -CalcStat("CritDefT",L,0.8);
								end
							else
								return 2;
							end
						else
							return 0.5;
						end
					else
						if Lm <= 50 then
							return LinFmod(1,144,1670,1,50,L);
						elseif Lm <= 60 then
							return LinFmod(1,1670,2500,50,60,L);
						elseif Lm <= 65 then
							return LinFmod(1,2500,3300,60,65,L);
						elseif Lm <= 75 then
							return LinFmod(1,3300,5000,65,75,L);
						elseif Lm <= 85 then
							return LinFmod(1,5000,7500,75,85,L);
						elseif Lm <= 95 then
							return LinFmod(1,7500,10800,85,95,L);
						elseif Lm <= 100 then
							return LinFmod(1,10800,15100,95,100,L);
						elseif Lm <= 105 then
							return LinFmod(1,15100,20100,100,105,L);
						elseif Lm <= 115 then
							return LinFmod(1,22100,30000,106,115,L);
						elseif Lm <= 120 then
							return LinFmod(1,34000,37000,116,120,L);
						elseif Lm <= 130 then
							return LinFmod(1,43000,57000,121,130,L);
						elseif Lm <= 140 then
							return LinFmod(1,65000,113000,131,140,L);
						else
							return LinFmod(1,130000,227000,141,150,L);
						end
					end
				else
					return CalcStat("ClassBaseVitality",L);
				end
			elseif SN > "BRWRETPRECISIONFINESSE" then
				if SN < "CHAMPIONCDMIGHTTOPHYMIT" then
					if SN < "CAPTAINCDBASEMORALE" then
						if SN < "BURGLARCDFATETONCPR" then
							if SN < "BURGLARCDBASEAGILITY" then
								if SN < "BURGLARCDAGILITYTOEVADE" then
									if SN < "BRWSHATTFISTARMOUR" then
										if SN > "BRWSHAREISBALANCEFINESSE" then
											if SN == "BRWSHAREISHEAVYCRITHIT" then
												return CalcStat("BrwInnStrPrecisionCritHit",L,3);
											else
												return 0;
											end
										elseif SN == "BRWSHAREISBALANCEFINESSE" then
											return CalcStat("BrwInnStrClevTechFinesse",L,3);
										else
											return 0;
										end
									elseif SN > "BRWSHATTFISTARMOUR" then
										if SN < "BRWVITALITYINCREASE" then
											if SN == "BRWTACMIT" then
												return CalcStat("TacMitT",L,CalcStat("Trait12345Choice",N)*0.4);
											else
												return 0;
											end
										elseif SN > "BRWVITALITYINCREASE" then
											if SN == "BURGLARCDAGILITYTOCRITHIT" then
												return 1;
											else
												return 0;
											end
										else
											return CalcStat("VitalityT",L,CalcStat("Trait567810Choice",N)*0.4);
										end
									else
										return -CalcStat("ArmourT",L,1.6);
									end
								elseif SN > "BURGLARCDAGILITYTOEVADE" then
									if SN < "BURGLARCDARMOURTOOFMIT" then
										if SN > "BURGLARCDAGILITYTOPARRY" then
											if SN == "BURGLARCDAGILITYTOPHYMAS" then
												return 3;
											else
												return 0;
											end
										elseif SN == "BURGLARCDAGILITYTOPARRY" then
											return 2;
										else
											return 0;
										end
									elseif SN > "BURGLARCDARMOURTOOFMIT" then
										if SN < "BURGLARCDARMOURTOTACMIT" then
											if SN == "BURGLARCDARMOURTOPHYMIT" then
												return 1;
											else
												return 0;
											end
										elseif SN > "BURGLARCDARMOURTOTACMIT" then
											if SN == "BURGLARCDARMOURTYPE" then
												return 2;
											else
												return 0;
											end
										else
											return 0.2;
										end
									else
										return 0.2;
									end
								else
									return 3;
								end
							elseif SN > "BURGLARCDBASEAGILITY" then
								if SN < "BURGLARCDBASENCPR" then
									if SN < "BURGLARCDBASEICPR" then
										if SN > "BURGLARCDBASEFATE" then
											if SN == "BURGLARCDBASEICMR" then
												return CalcStat("ClassBaseICMRL",L);
											else
												return 0;
											end
										elseif SN == "BURGLARCDBASEFATE" then
											return CalcStat("ClassBaseFate",L);
										else
											return 0;
										end
									elseif SN > "BURGLARCDBASEICPR" then
										if SN < "BURGLARCDBASEMORALE" then
											if SN == "BURGLARCDBASEMIGHT" then
												return CalcStat("ClassBaseMightM",L);
											else
												return 0;
											end
										elseif SN > "BURGLARCDBASEMORALE" then
											if SN == "BURGLARCDBASENCMR" then
												return CalcStat("ClassBaseNCMRL",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseMorale",L);
										end
									else
										return CalcStat("ClassBaseICPR",L);
									end
								elseif SN > "BURGLARCDBASENCPR" then
									if SN < "BURGLARCDBASEWILL" then
										if SN > "BURGLARCDBASEPOWER" then
											if SN == "BURGLARCDBASEVITALITY" then
												return CalcStat("ClassBaseVitality",L);
											else
												return 0;
											end
										elseif SN == "BURGLARCDBASEPOWER" then
											return CalcStat("ClassBasePower",L);
										else
											return 0;
										end
									elseif SN > "BURGLARCDBASEWILL" then
										if SN < "BURGLARCDFATETOFINESSE" then
											if SN == "BURGLARCDFATETOCRITHIT" then
												return 1.5;
											else
												return 0;
											end
										elseif SN > "BURGLARCDFATETOFINESSE" then
											if SN == "BURGLARCDFATETOICMR" then
												return 0.025;
											else
												return 0;
											end
										else
											return 0.5;
										end
									else
										return CalcStat("ClassBaseWillL",L);
									end
								else
									return CalcStat("ClassBaseNCPR",L);
								end
							else
								return CalcStat("ClassBaseAgilityH",L);
							end
						elseif SN > "BURGLARCDFATETONCPR" then
							if SN < "BURGLARCDWILLTOTACMIT" then
								if SN < "BURGLARCDPHYMITTOOFMIT" then
									if SN < "BURGLARCDFATETOTACMIT" then
										if SN > "BURGLARCDFATETOPOWER" then
											if SN == "BURGLARCDFATETORESIST" then
												return 0.5;
											else
												return 0;
											end
										elseif SN == "BURGLARCDFATETOPOWER" then
											return 1;
										else
											return 0;
										end
									elseif SN > "BURGLARCDFATETOTACMIT" then
										if SN < "BURGLARCDMIGHTTOBLOCK" then
											if SN == "BURGLARCDHASPOWER" then
												return 1;
											else
												return 0;
											end
										elseif SN > "BURGLARCDMIGHTTOBLOCK" then
											if SN == "BURGLARCDMIGHTTOPARRY" then
												return 2;
											else
												return 0;
											end
										else
											return 3;
										end
									else
										return 0.5;
									end
								elseif SN > "BURGLARCDPHYMITTOOFMIT" then
									if SN < "BURGLARCDVITALITYTONCMR" then
										if SN > "BURGLARCDTACMASTOOUTHEAL" then
											if SN == "BURGLARCDVITALITYTOMORALE" then
												return 4.5;
											else
												return 0;
											end
										elseif SN == "BURGLARCDTACMASTOOUTHEAL" then
											return 1;
										else
											return 0;
										end
									elseif SN > "BURGLARCDVITALITYTONCMR" then
										if SN < "BURGLARCDWILLTORESIST" then
											if SN == "BURGLARCDVITALITYTORESIST" then
												return 1;
											else
												return 0;
											end
										elseif SN > "BURGLARCDWILLTORESIST" then
											if SN == "BURGLARCDWILLTOTACMAS" then
												return 1;
											else
												return 0;
											end
										else
											return 2;
										end
									else
										return 0.12;
									end
								else
									return 1;
								end
							elseif SN > "BURGLARCDWILLTOTACMIT" then
								if SN < "CAPTAINCDARMOURTOTACMIT" then
									if SN < "CAPTAINCDAGILITYTOEVADE" then
										if SN > "C" then
											if SN == "CAPTAINCDAGILITYTOCRITHIT" then
												return 1;
											else
												return 0;
											end
										elseif SN == "C" then
											return C;
										else
											return 0;
										end
									elseif SN > "CAPTAINCDAGILITYTOEVADE" then
										if SN < "CAPTAINCDARMOURTOOFMIT" then
											if SN == "CAPTAINCDAGILITYTOPARRY" then
												return 2;
											else
												return 0;
											end
										elseif SN > "CAPTAINCDARMOURTOOFMIT" then
											if SN == "CAPTAINCDARMOURTOPHYMIT" then
												return 1;
											else
												return 0;
											end
										else
											return 0.2;
										end
									else
										return 3;
									end
								elseif SN > "CAPTAINCDARMOURTOTACMIT" then
									if SN < "CAPTAINCDBASEFATE" then
										if SN > "CAPTAINCDARMOURTYPE" then
											if SN == "CAPTAINCDBASEAGILITY" then
												return CalcStat("ClassBaseAgilityL",L);
											else
												return 0;
											end
										elseif SN == "CAPTAINCDARMOURTYPE" then
											return 3;
										else
											return 0;
										end
									elseif SN > "CAPTAINCDBASEFATE" then
										if SN < "CAPTAINCDBASEICPR" then
											if SN == "CAPTAINCDBASEICMR" then
												return CalcStat("ClassBaseICMRM",L);
											else
												return 0;
											end
										elseif SN > "CAPTAINCDBASEICPR" then
											if SN == "CAPTAINCDBASEMIGHT" then
												return CalcStat("ClassBaseMightH",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseICPR",L);
										end
									else
										return CalcStat("ClassBaseFate",L);
									end
								else
									return 0.2;
								end
							else
								return 1;
							end
						else
							return 0.05;
						end
					elseif SN > "CAPTAINCDBASEMORALE" then
						if SN < "CHAMPIONCDAGILITYTOPARRY" then
							if SN < "CAPTAINCDMIGHTTOBLOCK" then
								if SN < "CAPTAINCDFATETOCRITHIT" then
									if SN < "CAPTAINCDBASEPOWER" then
										if SN > "CAPTAINCDBASENCMR" then
											if SN == "CAPTAINCDBASENCPR" then
												return CalcStat("ClassBaseNCPR",L);
											else
												return 0;
											end
										elseif SN == "CAPTAINCDBASENCMR" then
											return CalcStat("ClassBaseNCMRM",L);
										else
											return 0;
										end
									elseif SN > "CAPTAINCDBASEPOWER" then
										if SN < "CAPTAINCDBASEWILL" then
											if SN == "CAPTAINCDBASEVITALITY" then
												return CalcStat("ClassBaseVitality",L);
											else
												return 0;
											end
										elseif SN > "CAPTAINCDBASEWILL" then
											if SN == "CAPTAINCDCANBLOCK" then
												if Lm <= 14 then
													return 0;
												else
													return 1;
												end
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseWillM",L);
										end
									else
										return CalcStat("ClassBasePower",L);
									end
								elseif SN > "CAPTAINCDFATETOCRITHIT" then
									if SN < "CAPTAINCDFATETONCPR" then
										if SN > "CAPTAINCDFATETOFINESSE" then
											if SN == "CAPTAINCDFATETOICMR" then
												return 0.025;
											else
												return 0;
											end
										elseif SN == "CAPTAINCDFATETOFINESSE" then
											return 0.5;
										else
											return 0;
										end
									elseif SN > "CAPTAINCDFATETONCPR" then
										if SN < "CAPTAINCDFATETOTACMIT" then
											if SN == "CAPTAINCDFATETOPOWER" then
												return 1;
											else
												return 0;
											end
										elseif SN > "CAPTAINCDFATETOTACMIT" then
											if SN == "CAPTAINCDHASPOWER" then
												return 1;
											else
												return 0;
											end
										else
											return 0.5;
										end
									else
										return 0.05;
									end
								else
									return 1.5;
								end
							elseif SN > "CAPTAINCDMIGHTTOBLOCK" then
								if SN < "CAPTAINCDVITALITYTONCMR" then
									if SN < "CAPTAINCDMIGHTTOTACMAS" then
										if SN > "CAPTAINCDMIGHTTOPARRY" then
											if SN == "CAPTAINCDMIGHTTOPHYMAS" then
												return 2.5;
											else
												return 0;
											end
										elseif SN == "CAPTAINCDMIGHTTOPARRY" then
											return 2;
										else
											return 0;
										end
									elseif SN > "CAPTAINCDMIGHTTOTACMAS" then
										if SN < "CAPTAINCDTACMASTOOUTHEAL" then
											if SN == "CAPTAINCDPHYMITTOOFMIT" then
												return 1;
											else
												return 0;
											end
										elseif SN > "CAPTAINCDTACMASTOOUTHEAL" then
											if SN == "CAPTAINCDVITALITYTOMORALE" then
												return 4.5;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 2.5;
									end
								elseif SN > "CAPTAINCDVITALITYTONCMR" then
									if SN < "CAPTAINCDWILLTOTACMIT" then
										if SN > "CAPTAINCDVITALITYTORESIST" then
											if SN == "CAPTAINCDWILLTORESIST" then
												return 2;
											else
												return 0;
											end
										elseif SN == "CAPTAINCDVITALITYTORESIST" then
											return 1;
										else
											return 0;
										end
									elseif SN > "CAPTAINCDWILLTOTACMIT" then
										if SN < "CHAMPIONCDAGILITYTOCRITHIT" then
											if SN == "CATMINTBEARARMOUR" then
												return CalcStat("LMCatmintBearArmour",L);
											else
												return 0;
											end
										elseif SN > "CHAMPIONCDAGILITYTOCRITHIT" then
											if SN == "CHAMPIONCDAGILITYTOEVADE" then
												return 3;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 1;
									end
								else
									return 0.12;
								end
							else
								return 3;
							end
						elseif SN > "CHAMPIONCDAGILITYTOPARRY" then
							if SN < "CHAMPIONCDBASEVITALITY" then
								if SN < "CHAMPIONCDBASEICMR" then
									if SN < "CHAMPIONCDARMOURTOTACMIT" then
										if SN > "CHAMPIONCDARMOURTOOFMIT" then
											if SN == "CHAMPIONCDARMOURTOPHYMIT" then
												return 1;
											else
												return 0;
											end
										elseif SN == "CHAMPIONCDARMOURTOOFMIT" then
											return 0.2;
										else
											return 0;
										end
									elseif SN > "CHAMPIONCDARMOURTOTACMIT" then
										if SN < "CHAMPIONCDBASEAGILITY" then
											if SN == "CHAMPIONCDARMOURTYPE" then
												return 3;
											else
												return 0;
											end
										elseif SN > "CHAMPIONCDBASEAGILITY" then
											if SN == "CHAMPIONCDBASEFATE" then
												return CalcStat("ClassBaseFate",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseAgilityM",L);
										end
									else
										return 0.2;
									end
								elseif SN > "CHAMPIONCDBASEICMR" then
									if SN < "CHAMPIONCDBASEMORALE" then
										if SN > "CHAMPIONCDBASEICPR" then
											if SN == "CHAMPIONCDBASEMIGHT" then
												return CalcStat("ClassBaseMightH",L);
											else
												return 0;
											end
										elseif SN == "CHAMPIONCDBASEICPR" then
											return CalcStat("ClassBaseICPR",L);
										else
											return 0;
										end
									elseif SN > "CHAMPIONCDBASEMORALE" then
										if SN < "CHAMPIONCDBASENCPR" then
											if SN == "CHAMPIONCDBASENCMR" then
												return CalcStat("ClassBaseNCMRH",L);
											else
												return 0;
											end
										elseif SN > "CHAMPIONCDBASENCPR" then
											if SN == "CHAMPIONCDBASEPOWER" then
												return CalcStat("ClassBasePower",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseNCPR",L);
										end
									else
										return CalcStat("ClassBaseMorale",L);
									end
								else
									return CalcStat("ClassBaseICMRH",L);
								end
							elseif SN > "CHAMPIONCDBASEVITALITY" then
								if SN < "CHAMPIONCDFATETOPOWER" then
									if SN < "CHAMPIONCDFATETOCRITHIT" then
										if SN > "CHAMPIONCDBASEWILL" then
											if SN == "CHAMPIONCDCANBLOCK" then
												if Lm <= 9 then
													return 0;
												else
													return 1;
												end
											else
												return 0;
											end
										elseif SN == "CHAMPIONCDBASEWILL" then
											return CalcStat("ClassBaseWillL",L);
										else
											return 0;
										end
									elseif SN > "CHAMPIONCDFATETOCRITHIT" then
										if SN < "CHAMPIONCDFATETOICMR" then
											if SN == "CHAMPIONCDFATETOFINESSE" then
												return 0.5;
											else
												return 0;
											end
										elseif SN > "CHAMPIONCDFATETOICMR" then
											if SN == "CHAMPIONCDFATETONCPR" then
												return 0.05;
											else
												return 0;
											end
										else
											return 0.025;
										end
									else
										return 1.5;
									end
								elseif SN > "CHAMPIONCDFATETOPOWER" then
									if SN < "CHAMPIONCDMIGHTTOCRITHIT" then
										if SN < "CHAMPIONCDFATETOTACMIT" then
											if SN == "CHAMPIONCDFATETORESIST" then
												return 0.5;
											else
												return 0;
											end
										elseif SN > "CHAMPIONCDFATETOTACMIT" then
											if SN == "CHAMPIONCDHASPOWER" then
												return 1;
											else
												return 0;
											end
										else
											return 0.5;
										end
									elseif SN > "CHAMPIONCDMIGHTTOCRITHIT" then
										if SN < "CHAMPIONCDMIGHTTOPARRY" then
											if SN == "CHAMPIONCDMIGHTTOEVADE" then
												return 2;
											else
												return 0;
											end
										elseif SN > "CHAMPIONCDMIGHTTOPARRY" then
											if SN == "CHAMPIONCDMIGHTTOPHYMAS" then
												return 3;
											else
												return 0;
											end
										else
											return 2;
										end
									else
										return 1;
									end
								else
									return 1;
								end
							else
								return CalcStat("ClassBaseVitality",L);
							end
						else
							return 2;
						end
					else
						return CalcStat("ClassBaseMorale",L);
					end
				elseif SN > "CHAMPIONCDMIGHTTOPHYMIT" then
					if SN < "COMBATBASETACHPSBASE" then
						if SN < "CLASSBASEICMRM" then
							if SN < "CHISELFINESSEL" then
								if SN < "CHAMPIONCDWILLTORESIST" then
									if SN < "CHAMPIONCDVITALITYTOBLOCK" then
										if SN > "CHAMPIONCDPHYMITTOOFMIT" then
											if SN == "CHAMPIONCDTACMASTOOUTHEAL" then
												return 1;
											else
												return 0;
											end
										elseif SN == "CHAMPIONCDPHYMITTOOFMIT" then
											return 1;
										else
											return 0;
										end
									elseif SN > "CHAMPIONCDVITALITYTOBLOCK" then
										if SN < "CHAMPIONCDVITALITYTONCMR" then
											if SN == "CHAMPIONCDVITALITYTOMORALE" then
												return 4.5;
											else
												return 0;
											end
										elseif SN > "CHAMPIONCDVITALITYTONCMR" then
											if SN == "CHAMPIONCDVITALITYTORESIST" then
												return 0.5;
											else
												return 0;
											end
										else
											return 0.12;
										end
									else
										return 1;
									end
								elseif SN > "CHAMPIONCDWILLTORESIST" then
									if SN < "CHISELCRITHITH" then
										if SN > "CHAMPIONCDWILLTOTACMAS" then
											if SN == "CHAMPIONCDWILLTOTACMIT" then
												return 1;
											else
												return 0;
											end
										elseif SN == "CHAMPIONCDWILLTOTACMAS" then
											return 1;
										else
											return 0;
										end
									elseif SN > "CHISELCRITHITH" then
										if SN < "CHISELCRITHITL" then
											if SN == "CHISELCRITHITHOLD" then
												if Lm <= 105 then
													return RoundDbl(24.24*L);
												else
													return CalcStat("ProgExtHighLinExpRnd",L,CalcStat("ChiselCritHitHOld",105));
												end
											else
												return 0;
											end
										elseif SN > "CHISELCRITHITL" then
											if SN == "CHISELCRITHITLOLD" then
												if Lm <= 105 then
													return RoundDbl(16.16*L);
												else
													return CalcStat("ProgExtHighLinExpRnd",L,CalcStat("ChiselCritHitLOld",105));
												end
											else
												return 0;
											end
										else
											return CalcStat("U371LegacyStatFix",L,"ChiselCritHitLOld");
										end
									else
										return CalcStat("U371LegacyStatFix",L,"ChiselCritHitHOld");
									end
								else
									return 2;
								end
							elseif SN > "CHISELFINESSEL" then
								if SN < "CHPUNBREAKTACMIT" then
									if SN < "CHPFINESSEINCREASE" then
										if SN > "CHP2HWPNBLOCK" then
											if SN == "CHPCONTRBURNICPR" then
												return CalcStat("ICPRT",L,0.4);
											else
												return 0;
											end
										elseif SN == "CHP2HWPNBLOCK" then
											return CalcStat("BlockT",L,4);
										else
											return 0;
										end
									elseif SN > "CHPFINESSEINCREASE" then
										if SN < "CHPMIGHTINCREASE" then
											if SN == "CHPFLURRYINCRCRITHIT" then
												if Lm <= 80 then
													return RoundDbl(16.16*L);
												else
													return CalcStat("ChpFlurryIncrCritHit",80);
												end
											else
												return 0;
											end
										elseif SN > "CHPMIGHTINCREASE" then
											if SN == "CHPSTALWBLADEVITALITY" then
												return CalcStat("VitalityT",L,CalcStat("Trait567810Choice",N)*0.4);
											else
												return 0;
											end
										else
											return CalcStat("MightT",L,CalcStat("Trait567810Choice",N)*0.4);
										end
									else
										return CalcStat("FinesseT",L,CalcStat("Trait12345Choice",N)*0.4);
									end
								elseif SN > "CHPUNBREAKTACMIT" then
									if SN < "CLASSBASEAGILITYM" then
										if SN > "CLASSBASEAGILITYH" then
											if SN == "CLASSBASEAGILITYL" then
												return CalcStat("BaseAgility",L,0.5);
											else
												return 0;
											end
										elseif SN == "CLASSBASEAGILITYH" then
											return CalcStat("BaseAgility",L,1.5);
										else
											return 0;
										end
									elseif SN > "CLASSBASEAGILITYM" then
										if SN < "CLASSBASEICMRH" then
											if SN == "CLASSBASEFATE" then
												return CalcStat("BaseFate",L,0.70001);
											else
												return 0;
											end
										elseif SN > "CLASSBASEICMRH" then
											if SN == "CLASSBASEICMRL" then
												return 0.15;
											else
												return 0;
											end
										else
											return 0.2;
										end
									else
										return CalcStat("BaseAgility",L,1.0);
									end
								else
									return CalcStat("TacMitT",L,0.4);
								end
							else
								if Lm <= 105 then
									return RoundDbl(4.04*L);
								else
									return CalcStat("ProgExtHighLinExpRnd",L,CalcStat("ChiselFinesseL",105));
								end
							end
						elseif SN > "CLASSBASEICMRM" then
							if SN < "CLASSBASEVITALITY" then
								if SN < "CLASSBASENCMRH" then
									if SN < "CLASSBASEMIGHTH" then
										if SN > "CLASSBASEICPR" then
											if SN == "CLASSBASEICPRADJ" then
												if Lm <= 1 then
													return 1.5*N;
												elseif Lm <= 20 then
													return 1.1*N;
												else
													return N;
												end
											else
												return 0;
											end
										elseif SN == "CLASSBASEICPR" then
											return StatLinInter("PntMPClassBaseICPR","ClassBaseProgPowerRegen","ProgBICPR","ClassBaseICPRAdj",L,N,99);
										else
											return 0;
										end
									elseif SN > "CLASSBASEMIGHTH" then
										if SN < "CLASSBASEMIGHTM" then
											if SN == "CLASSBASEMIGHTL" then
												return CalcStat("BaseMight",L,0.5);
											else
												return 0;
											end
										elseif SN > "CLASSBASEMIGHTM" then
											if SN == "CLASSBASEMORALE" then
												return CalcStat("BaseMorale",L,10.0);
											else
												return 0;
											end
										else
											return CalcStat("BaseMight",L,1.0);
										end
									else
										return CalcStat("BaseMight",L,1.5);
									end
								elseif SN > "CLASSBASENCMRH" then
									if SN < "CLASSBASENCPR" then
										if SN > "CLASSBASENCMRL" then
											if SN == "CLASSBASENCMRM" then
												return 1;
											else
												return 0;
											end
										elseif SN == "CLASSBASENCMRL" then
											return 1;
										else
											return 0;
										end
									elseif SN > "CLASSBASENCPR" then
										if SN < "CLASSBASEPOWER" then
											if SN == "CLASSBASENCPRADJ" then
												if Lm <= 1 then
													return 1.5*N;
												elseif Lm <= 20 then
													return 1.1*N;
												else
													return N;
												end
											else
												return 0;
											end
										elseif SN > "CLASSBASEPOWER" then
											if SN == "CLASSBASEPROGPOWERREGEN" then
												return {{1,20,50,60,65,75,85,95,100,105,115,120,130,140,150},{1,20,50,60,65,75,85,95,100,105,115,120,130,140,150}};
											else
												return 0;
											end
										else
											return CalcStat("BasePower",L,1.0);
										end
									else
										return StatLinInter("PntMPClassBaseNCPR","ClassBaseProgPowerRegen","ProgBNCPR","ClassBaseNCPRAdj",L,N,99);
									end
								else
									return 2;
								end
							elseif SN > "CLASSBASEVITALITY" then
								if SN < "COMBATBASEH" then
									if SN < "CLASSBASEWILLM" then
										if SN > "CLASSBASEWILLH" then
											if SN == "CLASSBASEWILLL" then
												return CalcStat("BaseWill",L,0.5);
											else
												return 0;
											end
										elseif SN == "CLASSBASEWILLH" then
											return CalcStat("BaseWill",L,1.5);
										else
											return 0;
										end
									elseif SN > "CLASSBASEWILLM" then
										if SN < "COMBATBASE" then
											if SN == "CLOTHARMOUR" then
												if Lm <= 50 then
													return L;
												else
													return 50;
												end
											else
												return 0;
											end
										elseif SN > "COMBATBASE" then
											if SN == "COMBATBASECAT" then
												if Lm <= 0 then
													return 0;
												elseif Lm <= 1 then
													return CalcStat("CombatBaseH",N);
												else
													return CalcStat("CombatBaseL",N);
												end
											else
												return 0;
											end
										else
											return CalcStat("CombatBaseCat",WpnCodeIndex(C,1),L)*CalcStat("CombatBaseTypeMP",WpnCodeIndex(C,2))*CalcStat("CombatBaseQtyMPCat",WpnCodeIndex(C,1),WpnCodeIndex(C,3));
										end
									else
										return CalcStat("BaseWill",L,1.0);
									end
								elseif SN > "COMBATBASEH" then
									if SN < "COMBATBASEPHYDPS" then
										if SN < "COMBATBASEL" then
											if SN == "COMBATBASEHQTYMP" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({1,1.02,1.04,1.08,1.12},L);
												end
											else
												return 0;
											end
										elseif SN > "COMBATBASEL" then
											if SN == "COMBATBASELQTYMP" then
												return CalcStat("CombatBaseHQtyMP",L);
											else
												return 0;
											end
										else
											return 0.9*CalcStat("CombatBasePhyDPS",L)*1.08;
										end
									elseif SN > "COMBATBASEPHYDPS" then
										if SN < "COMBATBASETACDPS" then
											if SN == "COMBATBASEQTYMPCAT" then
												if Lm <= 0 then
													return 0;
												elseif Lm <= 1 then
													return CalcStat("CombatBaseHQtyMP",N);
												else
													return CalcStat("CombatBaseLQtyMP",N);
												end
											else
												return 0;
											end
										elseif SN > "COMBATBASETACDPS" then
											if SN == "COMBATBASETACHPS" then
												if Lm <= 46 then
													return 0;
												elseif Lm <= 49 then
													return DataTableValue({-0.091142856,-0.18971428,-0.2957143},L-46);
												elseif Lm <= 474 then
													return CalcStat("CombatBaseTacHPSBase",L)-5.659142856;
												elseif Lm <= 475 then
													return 460.62656;
												else
													return CalcStat("CombatBaseTacHPSBase",L)-5.659142856;
												end
											else
												return 0;
											end
										else
											if Lm <= 47 then
												return LinFmod(1,0.5,0.5,1,47,L);
											elseif Lm <= 50 then
												return LinFmod(1,0.5,0.8,47,50,L);
											elseif Lm <= 51 then
												return LinFmod(1,1,1,51,51,L);
											elseif Lm <= 60 then
												return LinFmod(1,1.32,6.6,52,60,L);
											elseif Lm <= 65 then
												return LinFmod(1,6.6,9.1,60,65,L);
											elseif Lm <= 75 then
												return LinFmod(1,9.1,19,65,75,L);
											elseif Lm <= 125 then
												return LinFmod(1,19,32,75,125,L);
											elseif Lm <= 175 then
												return LinFmod(1,32,49,125,175,L);
											elseif Lm <= 200 then
												return LinFmod(1,49,66,175,200,L);
											elseif Lm <= 222 then
												return LinFmod(1,66,88,200,222,L);
											elseif Lm <= 299 then
												return LinFmod(1,88,88,222,299,L);
											elseif Lm <= 349 then
												return LinFmod(1,99,124,300,349,L);
											elseif Lm <= 399 then
												return LinFmod(1,138,160,350,399,L);
											elseif Lm <= 449 then
												return LinFmod(1,178,219,400,449,L);
											elseif Lm <= 499 then
												return LinFmod(1,243,340,450,499,L);
											elseif Lm <= 549 then
												return LinFmod(1,380,520,500,549,L);
											elseif Lm <= 599 then
												return RoundDblDown(RoundDblUp((L+1)/5.5)*25-1955,-1);
											elseif Lm <= 649 then
												return RoundDblDown(RoundDblUp((L-49)/5.5)*36.7-2831,-1);
											else
												return CalcStat("CombatBaseTacDPS",649);
											end
										end
									else
										if Lm <= 50 then
											return LinFmod(1,2,20,1,50,L);
										elseif Lm <= 60 then
											return LinFmod(1,20,26.6,50,60,L);
										elseif Lm <= 65 then
											return LinFmod(1,26.6,29.1,60,65,L);
										elseif Lm <= 75 then
											return LinFmod(1,29.1,39,65,75,L);
										elseif Lm <= 125 then
											return LinFmod(1,39,52,75,125,L);
										elseif Lm <= 175 then
											return LinFmod(1,52,69,125,175,L);
										elseif Lm <= 200 then
											return LinFmod(1,69,86,175,200,L);
										elseif Lm <= 222 then
											return LinFmod(1,86,108,200,222,L);
										elseif Lm <= 300 then
											return LinFmod(1,108,108,222,300,L);
										elseif Lm <= 350 then
											return LinFmod(1,119,144,301,350,L);
										elseif Lm <= 400 then
											return LinFmod(1,158,180,351,400,L);
										elseif Lm <= 450 then
											return LinFmod(1,198,239,401,450,L);
										elseif Lm <= 500 then
											return LinFmod(1,263,360,451,500,L);
										elseif Lm <= 550 then
											return LinFmod(1,400,540,501,550,L);
										elseif Lm <= 600 then
											return LinFmod(1,590,810,551,600,L);
										elseif Lm <= 650 then
											return RoundDblDown(RoundDblUp((L-50)/5.5)*36.7-2811,-1);
										else
											return CalcStat("CombatBasePhyDPS",650);
										end
									end
								else
									return 1.0*CalcStat("CombatBasePhyDPS",L)*1.08;
								end
							else
								return CalcStat("BaseVitality",L,1.5);
							end
						else
							return 0.175;
						end
					elseif SN > "COMBATBASETACHPSBASE" then
						if SN < "CPTCRITDEF" then
							if SN < "COMBATDAMAGEMODPETSRND" then
								if SN < "COMBATDAMAGEMODHEALTHLOW" then
									if SN < "COMBATBASETYPEMP" then
										if SN > "COMBATBASETACHPSLVLTOILVL" then
											if SN == "COMBATBASETACHPSNOCLASS" then
												return CalcStat("CombatBaseTacHPSBase",CalcStat("CombatBaseTacHPSLvlToILvl",L));
											else
												return 0;
											end
										elseif SN == "COMBATBASETACHPSLVLTOILVL" then
											if Lm <= 75 then
												return LinFmod(1,1,75,1,75,L);
											elseif Lm <= 100 then
												return LinFmod(1,75,200,75,100,L);
											elseif Lm <= 104 then
												return RoundDbl(LinFmod(1,201.3,214.3,101,104,L));
											elseif Lm <= 105 then
												return 222;
											else
												return RoundDbl(CalcStat("LvlToILvl",L));
											end
										else
											return 0;
										end
									elseif SN > "COMBATBASETYPEMP" then
										if SN < "COMBATDAMAGEMODENERGY" then
											if SN == "COMBATDAMAGEMOD" then
												return StatLinInter("PntMPCombatDamageMod","TraitProgVital","ProgBCombatDamageMod","",L,N,2);
											else
												return 0;
											end
										elseif SN > "COMBATDAMAGEMODENERGY" then
											if SN == "COMBATDAMAGEMODHEALTH" then
												return StatLinInter("PntMPCombatDamageMod","TraitProgVital","ProgBHealth","",L,N,2);
											else
												return 0;
											end
										else
											return StatLinInter("PntMPCombatDamageMod","TraitProgVital","ProgBEnergy","",L,N,2);
										end
									else
										if Lm <= 0 then
											return 0;
										else
											return DataTableValue({1,1.4,1.4},L);
										end
									end
								elseif SN > "COMBATDAMAGEMODHEALTHLOW" then
									if SN < "COMBATDAMAGEMODHEALTHMEDIUMADJ" then
										if SN > "COMBATDAMAGEMODHEALTHLOWADJ" then
											if SN == "COMBATDAMAGEMODHEALTHMEDIUM" then
												return StatLinInter("PntMPCombatDamageMod","TraitProgVital","ProgBHealth","CombatDamageModHealthMediumAdj",L,N,99);
											else
												return 0;
											end
										elseif SN == "COMBATDAMAGEMODHEALTHLOWADJ" then
											if Lm <= 1 then
												return 0.144*N;
											elseif Lm <= 25 then
												return 0.324*N;
											elseif Lm <= 50 then
												return 0.576*N;
											else
												return 0.9*N;
											end
										else
											return 0;
										end
									elseif SN > "COMBATDAMAGEMODHEALTHMEDIUMADJ" then
										if SN < "COMBATDAMAGEMODPETS" then
											if SN == "COMBATDAMAGEMODNPCS" then
												return StatLinInter("PntMPCombatDamageMod","TraitProgVital","ProgBCombatDamageMod","",L,N,99);
											else
												return 0;
											end
										elseif SN > "COMBATDAMAGEMODPETS" then
											if SN == "COMBATDAMAGEMODPETSADJ" then
												if Lm <= 1 then
													return N;
												elseif Lm <= 25 then
													return 0.4*N;
												elseif Lm <= 50 then
													return 0.5*N;
												elseif Lm <= 60 then
													return 0.6*N;
												elseif Lm <= 65 then
													return 0.7*N;
												elseif Lm <= 75 then
													return 0.8*N;
												elseif Lm <= 85 then
													return 0.85*N;
												elseif Lm <= 95 then
													return 0.9*N;
												elseif Lm <= 100 then
													return 0.95*N;
												else
													return N;
												end
											else
												return 0;
											end
										else
											return StatLinInter("PntMPCombatDamageMod","TraitProgVital","ProgBCombatDamageMod","CombatDamageModPetsAdj",L,N,99);
										end
									else
										if Lm <= 1 then
											return 0.36*N;
										elseif Lm <= 25 then
											return 0.54*N;
										elseif Lm <= 50 then
											return 0.72*N;
										else
											return 0.9*N;
										end
									end
								else
									return StatLinInter("PntMPCombatDamageMod","TraitProgVital","ProgBHealth","CombatDamageModHealthLowAdj",L,N,99);
								end
							elseif SN > "COMBATDAMAGEMODPETSRND" then
								if SN < "CPTBLADECRITDEF" then
									if SN < "COMBATINHEAL" then
										if SN > "COMBATDAMAGEMODPLAYERS" then
											if SN == "COMBATDAMAGEMODPLAYERSADJ" then
												if Lm <= 1 then
													return 0.8*N;
												elseif Lm <= 25 then
													return 0.9*N;
												else
													return N;
												end
											else
												return 0;
											end
										elseif SN == "COMBATDAMAGEMODPLAYERS" then
											return StatLinInter("PntMPCombatDamageMod","TraitProgVital","ProgBCombatDamageMod","CombatDamageModPlayersAdj",L,N,99);
										else
											return 0;
										end
									elseif SN > "COMBATINHEAL" then
										if SN < "COMMONTACHPSITEMPROG" then
											if SN == "COMMONTACHPS" then
												return StatLinInter("PntMPTacHPS","CommonTacHPSItemProg","ProgBTacHPS","",L,N,99);
											else
												return 0;
											end
										elseif SN > "COMMONTACHPSITEMPROG" then
											if SN == "CONSTSTATC" then
												return CalcStat(C,1,L);
											else
												return 0;
											end
										else
											return {{1,50,60,65,75,125,175,200,222,299,300,349,350,399,400,449,450,499,500,549},{1,50,60,65,75,85,95,100,105,105,106,115,116,120,121,130,131,140,141,150}};
										end
									else
										return CalcStat("InHeal",L,2.0);
									end
								elseif SN > "CPTBLADECRITDEF" then
									if SN < "CPTCOVFATE" then
										if SN > "CPTBLADEPHYMAS" then
											if SN == "CPTBLADETACMAS" then
												return CalcStat("TacMasT",L,0.8);
											else
												return 0;
											end
										elseif SN == "CPTBLADEPHYMAS" then
											return CalcStat("PhyMasT",L,1.2);
										else
											return 0;
										end
									elseif SN > "CPTCOVFATE" then
										if SN < "CPTCOVPHYMIT" then
											if SN == "CPTCOVMAIN" then
												return CalcStat("MainT",L,0.4);
											else
												return 0;
											end
										elseif SN > "CPTCOVPHYMIT" then
											if SN == "CPTCOVVITALITY" then
												return CalcStat("VitalityT",L,0.4);
											else
												return 0;
											end
										else
											return CalcStat("PhyMitT",L,2.4);
										end
									else
										return CalcStat("FateT",L,0.4);
									end
								else
									return CalcStat("CritDefT",L,0.8);
								end
							else
								return StatLinInter("PntMPCombatDamageMod","TraitProgVital","ProgBCombatDamageMod","CombatDamageModPetsAdj",L,N,2);
							end
						elseif SN > "CPTCRITDEF" then
							if SN < "CREEPAUDACITYCOSTBASE" then
								if SN < "CPTSONGCRITDEF" then
									if SN < "CPTIDOMEVITALITY" then
										if SN > "CPTIDOMEFATE" then
											if SN == "CPTIDOMEMAIN" then
												return CalcStat("MainT",L,0.4);
											else
												return 0;
											end
										elseif SN == "CPTIDOMEFATE" then
											return CalcStat("FateT",L,0.4);
										else
											return 0;
										end
									elseif SN > "CPTIDOMEVITALITY" then
										if SN < "CPTSHIELDPHYMAS" then
											if SN == "CPTSHIELDCRITDEF" then
												return CalcStat("CritDefT",L,1.2);
											else
												return 0;
											end
										elseif SN > "CPTSHIELDPHYMAS" then
											if SN == "CPTSHIELDTACMAS" then
												return CalcStat("TacMasT",L,0.8);
											else
												return 0;
											end
										else
											return CalcStat("PhyMasT",L,0.8);
										end
									else
										return CalcStat("VitalityT",L,0.4);
									end
								elseif SN > "CPTSONGCRITDEF" then
									if SN < "CPTSTANDALONEPHYMAS" then
										if SN > "CPTSONGPHYMAS" then
											if SN == "CPTSONGTACMAS" then
												return CalcStat("TacMasT",L,1.2);
											else
												return 0;
											end
										elseif SN == "CPTSONGPHYMAS" then
											return CalcStat("PhyMasT",L,0.8);
										else
											return 0;
										end
									elseif SN > "CPTSTANDALONEPHYMAS" then
										if SN < "CREEPAUDACITYCCDP" then
											if SN == "CPTSTANDALONETACMAS" then
												return CalcStat("TacMasT",L,0.8);
											else
												return 0;
											end
										elseif SN > "CREEPAUDACITYCCDP" then
											if SN == "CREEPAUDACITYCOST" then
												return CalcStat("CreepAudacityCostBase",L)*25;
											else
												return 0;
											end
										else
											if Lm <= 0 then
												return 0;
											elseif Lm <= 15 then
												return LinFmod(1,0.5,0.5,1,15,L);
											elseif Lm <= 16 then
												return LinFmod(1,0.5,0.4,15,16,L);
											elseif Lm <= 41 then
												return LinFmod(1,0.4,0.4,16,41,L);
											elseif Lm <= 60 then
												return LinFmod(1,0.4,0.4,41,60,L);
											else
												return CalcStat("CreepAudacityCCDP",60);
											end
										end
									else
										return CalcStat("PhyMasT",L,1.2);
									end
								else
									return CalcStat("CritDefT",L,0.8);
								end
							elseif SN > "CREEPAUDACITYCOSTBASE" then
								if SN < "CREEPAUDACITYTACDMGP" then
									if SN < "CREEPAUDACITYMELREDP" then
										if SN > "CREEPAUDACITYDMGP" then
											if SN == "CREEPAUDACITYMELDMGP" then
												return CalcStat("CreepAudacityDmgP",L);
											else
												return 0;
											end
										elseif SN == "CREEPAUDACITYDMGP" then
											if Lm <= 0 then
												return 0;
											elseif Lm <= 10 then
												return LinFmod(1,1,1.2,1,10,L);
											elseif Lm <= 36 then
												return LinFmod(1,1.2,1.25,10,36,L);
											elseif Lm <= 41 then
												return LinFmod(1,1.25,1.3,36,41,L);
											elseif Lm <= 60 then
												return LinFmod(1,1.3,1.25,41,60,L);
											else
												return CalcStat("CreepAudacityDmgP",60);
											end
										else
											return 0;
										end
									elseif SN > "CREEPAUDACITYMELREDP" then
										if SN < "CREEPAUDACITYRNGDMGP" then
											if SN == "CREEPAUDACITYREDP" then
												if Lm <= 0 then
													return 0;
												elseif Lm <= 10 then
													return LinFmod(1,0.55,0.55,1,10,L);
												elseif Lm <= 36 then
													return LinFmod(1,0.55,0.55,10,36,L);
												elseif Lm <= 41 then
													return LinFmod(1,0.55,0.55,36,41,L);
												elseif Lm <= 60 then
													return LinFmod(1,0.55,0.55,41,60,L);
												else
													return CalcStat("CreepAudacityRedP",60);
												end
											else
												return 0;
											end
										elseif SN > "CREEPAUDACITYRNGDMGP" then
											if SN == "CREEPAUDACITYRNGREDP" then
												return CalcStat("CreepAudacityRedP",L);
											else
												return 0;
											end
										else
											return CalcStat("CreepAudacityDmgP",L);
										end
									else
										return CalcStat("CreepAudacityRedP",L);
									end
								elseif SN > "CREEPAUDACITYTACDMGP" then
									if SN < "CREEPBATPROMFINESSE" then
										if SN < "CREEPBASEPROG" then
											if SN == "CREEPAUDACITYTACREDP" then
												return CalcStat("CreepAudacityRedP",L);
											else
												return 0;
											end
										elseif SN > "CREEPBASEPROG" then
											if SN == "CREEPBATPROMDMGP" then
												if Lm <= 0 then
													return 0;
												elseif Lm <= 5 then
													return LinFmod(1,0.005,0.02,1,5,L);
												elseif Lm <= 10 then
													return LinFmod(1,0.02,0.045,5,10,L);
												elseif Lm <= 15 then
													return LinFmod(1,0.045,0.075,10,15,L);
												else
													return CalcStat("CreepBatPromDmgP",15);
												end
											else
												return 0;
											end
										else
											return LinFmod(1,RoundDblLotro(N),RoundDblLotro(100*N),1,140,L);
										end
									elseif SN > "CREEPBATPROMFINESSE" then
										if SN < "CREEPBATPROMINHEALP" then
											if SN == "CREEPBATPROMHEALTHP" then
												return CalcStat("CreepBatPromVitalP",L);
											else
												return 0;
											end
										elseif SN > "CREEPBATPROMINHEALP" then
											if SN == "CREEPBATPROMMELDMGP" then
												return CalcStat("CreepBatPromDmgP",L);
											else
												return 0;
											end
										else
											return 0;
										end
									else
										return 0;
									end
								else
									return CalcStat("CreepAudacityDmgP",L);
								end
							else
								if Lm <= 0 then
									return 0;
								elseif Lm <= 2 then
									return 2*L;
								elseif Lm <= 9 then
									return 2*L-1;
								elseif Lm <= 15 then
									return 3*L;
								elseif Lm <= 25 then
									return 6*L;
								elseif Lm <= 29 then
									return 9*L;
								elseif Lm <= 36 then
									return 300;
								else
									return 0;
								end
							end
						else
							return CalcStat("CritDef",L,0.6);
						end
					else
						if Lm <= 24 then
							return (-1/300)*L*L+(741739.1304347/3000000)*L+1.75608695652177;
						elseif Lm <= 49 then
							return -0.003725*L*L+0.255*L+1.812153556;
						elseif Lm <= 55 then
							return 0.5+5.659142856;
						elseif Lm <= 549 then
							return CalcStat("CommonTacHPS",L);
						elseif Lm <= 599 then
							return (RoundDblUp((4*L+3)/22)*378-RoundDblUp((L+7)/22))*0.175-6081.6;
						elseif Lm <= 649 then
							return (RoundDblUp((4*L+2)/22)*678-RoundDblUp((L+1)/22))*0.175-11734.1;
						else
							return CalcStat("CombatBaseTacHPSBase",649);
						end
					end
				else
					return 1;
				end
			else
				return CalcStat("FinesseT",L,CalcStat("Trait234Choice",N));
			end
		elseif SN > "CREEPBATPROMOUTHEALP" then
			if SN < "HUNTERCDBASEAGILITY" then
				if SN < "FOODNCPRBASEPROG" then
					if SN < "DWARFRDTRAITICPR" then
						if SN < "CRITMAGNPRATP" then
							if SN < "CRITDEFT" then
								if SN < "CRITDEFPPRAT" then
									if SN < "CREEPBATPROMTACDMGP" then
										if SN > "CREEPBATPROMPOWERP" then
											if SN == "CREEPBATPROMRNGDMGP" then
												return CalcStat("CreepBatPromDmgP",L);
											else
												return 0;
											end
										elseif SN == "CREEPBATPROMPOWERP" then
											return CalcStat("CreepBatPromVitalP",L);
										else
											return 0;
										end
									elseif SN > "CREEPBATPROMTACDMGP" then
										if SN < "CRITDEF" then
											if SN == "CREEPBATPROMVITALP" then
												if Lm <= 0 then
													return 0;
												elseif Lm <= 5 then
													return LinFmod(1,1.02,1.1,1,5,L);
												elseif Lm <= 10 then
													return LinFmod(1,1.1,1.15,5,10,L);
												elseif Lm <= 15 then
													return LinFmod(1,1.15,1.2,10,15,L);
												else
													return CalcStat("CreepBatPromVitalP",15);
												end
											else
												return 0;
											end
										elseif SN > "CRITDEF" then
											if SN == "CRITDEFPBONUS" then
												return 0;
											else
												return 0;
											end
										else
											return StatLinInter("PntMPCritDef","ItemProg","ProgBCritDef","",L,N,0);
										end
									else
										return CalcStat("CreepBatPromDmgP",L);
									end
								elseif SN > "CRITDEFPPRAT" then
									if SN < "CRITDEFPRATPB" then
										if SN > "CRITDEFPRATP" then
											if SN == "CRITDEFPRATPA" then
												return 240;
											else
												return 0;
											end
										elseif SN == "CRITDEFPRATP" then
											return CalcPercAB(CalcStat("CritDefPRatPA",L),CalcStat("CritDefPRatPB",L),CalcStat("CritDefPRatPCap",L),N);
										else
											return 0;
										end
									elseif SN > "CRITDEFPRATPB" then
										if SN < "CRITDEFPRATPCAP" then
											if SN == "CRITDEFPRATPC" then
												return 0.5;
											else
												return 0;
											end
										elseif SN > "CRITDEFPRATPCAP" then
											if SN == "CRITDEFPRATPCAPR" then
												return CalcStat("CritDefPRatPB",L)*CalcStat("CritDefPRatPC",L);
											else
												return 0;
											end
										else
											return 80;
										end
									else
										return CalcStat("BRatRounded",L,"BRatStandard");
									end
								else
									return CalcRatAB(CalcStat("CritDefPRatPA",L),CalcStat("CritDefPRatPB",L),CalcStat("CritDefPRatPCapR",L),N);
								end
							elseif SN > "CRITDEFT" then
								if SN < "CRITHITPRATPB" then
									if SN < "CRITHITPBONUS" then
										if SN > "CRITHIT" then
											if SN == "CRITHITOLD" then
												return CalcStat("CritHit",L,N);
											else
												return 0;
											end
										elseif SN == "CRITHIT" then
											return StatLinInter("PntMPCritHit","ItemProg","ProgBCritHit","",L,N,0);
										else
											return 0;
										end
									elseif SN > "CRITHITPBONUS" then
										if SN < "CRITHITPRATP" then
											if SN == "CRITHITPPRAT" then
												return CalcRatAB(CalcStat("CritHitPRatPA",L),CalcStat("CritHitPRatPB",L),CalcStat("CritHitPRatPCapR",L),N);
											else
												return 0;
											end
										elseif SN > "CRITHITPRATP" then
											if SN == "CRITHITPRATPA" then
												return 75;
											else
												return 0;
											end
										else
											return CalcPercAB(CalcStat("CritHitPRatPA",L),CalcStat("CritHitPRatPB",L),CalcStat("CritHitPRatPCap",L),N);
										end
									else
										return 0;
									end
								elseif SN > "CRITHITPRATPB" then
									if SN < "CRITHITPRATPCAPR" then
										if SN > "CRITHITPRATPC" then
											if SN == "CRITHITPRATPCAP" then
												return 25;
											else
												return 0;
											end
										elseif SN == "CRITHITPRATPC" then
											return 0.5;
										else
											return 0;
										end
									elseif SN > "CRITHITPRATPCAPR" then
										if SN < "CRITMAGNPBONUS" then
											if SN == "CRITHITT" then
												return StatLinInter("PntMPCritHit","TraitProg","ProgBCritHit","",L,N,0);
											else
												return 0;
											end
										elseif SN > "CRITMAGNPBONUS" then
											if SN == "CRITMAGNPPRAT" then
												return CalcRatAB(CalcStat("CritMagnPRatPA",L),CalcStat("CritMagnPRatPB",L),CalcStat("CritMagnPRatPCapR",L),N);
											else
												return 0;
											end
										else
											return 0;
										end
									else
										return CalcStat("CritHitPRatPB",L)*CalcStat("CritHitPRatPC",L);
									end
								else
									return CalcStat("BRatRounded",L,"BRatStandard");
								end
							else
								return StatLinInter("PntMPCritDef","TraitProg","ProgBCritDef","",L,N,0);
							end
						elseif SN > "CRITMAGNPRATP" then
							if SN < "DEVHITPRATPCAPR" then
								if SN < "DEVHITPBONUS" then
									if SN < "CRITMAGNPRATPC" then
										if SN > "CRITMAGNPRATPA" then
											if SN == "CRITMAGNPRATPB" then
												return CalcStat("BRatRounded",L,"BRatCritMagn");
											else
												return 0;
											end
										elseif SN == "CRITMAGNPRATPA" then
											return 225;
										else
											return 0;
										end
									elseif SN > "CRITMAGNPRATPC" then
										if SN < "CRITMAGNPRATPCAPR" then
											if SN == "CRITMAGNPRATPCAP" then
												return 75;
											else
												return 0;
											end
										elseif SN > "CRITMAGNPRATPCAPR" then
											if SN == "CRYRESISTT" then
												return CalcStat("ResistAddT",L,N);
											else
												return 0;
											end
										else
											return CalcStat("CritMagnPRatPB",L)*CalcStat("CritMagnPRatPC",L);
										end
									else
										return 0.5;
									end
								elseif SN > "DEVHITPBONUS" then
									if SN < "DEVHITPRATPA" then
										if SN > "DEVHITPPRAT" then
											if SN == "DEVHITPRATP" then
												return CalcPercAB(CalcStat("DevHitPRatPA",L),CalcStat("DevHitPRatPB",L),CalcStat("DevHitPRatPCap",L),N);
											else
												return 0;
											end
										elseif SN == "DEVHITPPRAT" then
											return CalcRatAB(CalcStat("DevHitPRatPA",L),CalcStat("DevHitPRatPB",L),CalcStat("DevHitPRatPCapR",L),N);
										else
											return 0;
										end
									elseif SN > "DEVHITPRATPA" then
										if SN < "DEVHITPRATPC" then
											if SN == "DEVHITPRATPB" then
												return CalcStat("BRatRounded",L,"BRatDevHit");
											else
												return 0;
											end
										elseif SN > "DEVHITPRATPC" then
											if SN == "DEVHITPRATPCAP" then
												return 10;
											else
												return 0;
											end
										else
											return 0.5;
										end
									else
										return 30;
									end
								else
									return 0;
								end
							elseif SN > "DEVHITPRATPCAPR" then
								if SN < "DWARFRDPSVONEFATE" then
									if SN < "DMGTYPEMITT" then
										if SN > "DISEASERESISTT" then
											if SN == "DMGTYPEMIT" then
												return StatLinInter("PntMPDmgTypeMit","ItemProg","ProgBMitigation","",L,N,0);
											else
												return 0;
											end
										elseif SN == "DISEASERESISTT" then
											return CalcStat("ResistAddT",L,N);
										else
											return 0;
										end
									elseif SN > "DMGTYPEMITT" then
										if SN < "DWARFFATEFULDWARFFATE" then
											if SN == "DWARFENDURVITALITY" then
												return CalcStat("VitalityT",L,1.0);
											else
												return 0;
											end
										elseif SN > "DWARFFATEFULDWARFFATE" then
											if SN == "DWARFLOSTDWARFKDSFATE" then
												return -CalcStat("FateT",L,0.4);
											else
												return 0;
											end
										else
											return CalcStat("FateT",L,1.0);
										end
									else
										return StatLinInter("PntMPDmgTypeMit","TraitProg","ProgBMitigation","",L,N,0);
									end
								elseif SN > "DWARFRDPSVONEFATE" then
									if SN < "DWARFRDPSVTWONAME" then
										if SN > "DWARFRDPSVONENAME" then
											if SN == "DWARFRDPSVTWOBLOCK" then
												return CalcStat("DwarfShieldBrwlBlock",L);
											else
												return 0;
											end
										elseif SN == "DWARFRDPSVONENAME" then
											return "Fateful Dwarf";
										else
											return 0;
										end
									elseif SN > "DWARFRDPSVTWONAME" then
										if SN < "DWARFRDTRAITFATE" then
											if SN == "DWARFRDTRAITAGILITY" then
												return CalcStat("DwarfStockyAgility",L);
											else
												return 0;
											end
										elseif SN > "DWARFRDTRAITFATE" then
											if SN == "DWARFRDTRAITICMR" then
												return CalcStat("DwarfUnwearBattleICMR",L);
											else
												return 0;
											end
										else
											return CalcStat("DwarfLostDwarfKdsFate",L);
										end
									else
										return "Shield Brawler";
									end
								else
									return CalcStat("DwarfFatefulDwarfFate",L);
								end
							else
								return CalcStat("DevHitPRatPB",L)*CalcStat("DevHitPRatPC",L);
							end
						else
							return CalcPercAB(CalcStat("CritMagnPRatPA",L),CalcStat("CritMagnPRatPB",L),CalcStat("CritMagnPRatPCap",L),N);
						end
					elseif SN > "DWARFRDTRAITICPR" then
						if SN < "EVADEPBONUS" then
							if SN < "DWARFUNWEARBATTLENCPR" then
								if SN < "DWARFSTOCKYAGILITY" then
									if SN < "DWARFRDTRAITNCPR" then
										if SN > "DWARFRDTRAITMIGHT" then
											if SN == "DWARFRDTRAITNCMR" then
												return CalcStat("DwarfUnwearBattleNCMR",L);
											else
												return 0;
											end
										elseif SN == "DWARFRDTRAITMIGHT" then
											return CalcStat("DwarfSturdinessMight",L);
										else
											return 0;
										end
									elseif SN > "DWARFRDTRAITNCPR" then
										if SN < "DWARFRDTRAITVITALITY" then
											if SN == "DWARFRDTRAITPHYMITP" then
												return CalcStat("DwarfSturdinessPhyMitP",L);
											else
												return 0;
											end
										elseif SN > "DWARFRDTRAITVITALITY" then
											if SN == "DWARFSHIELDBRWLBLOCK" then
												return CalcStat("BlockT",L,0.8);
											else
												return 0;
											end
										else
											return CalcStat("DwarfSturdinessVitality",L);
										end
									else
										return CalcStat("DwarfUnwearBattleNCPR",L);
									end
								elseif SN > "DWARFSTOCKYAGILITY" then
									if SN < "DWARFSTURDINESSVITALITY" then
										if SN > "DWARFSTURDINESSMIGHT" then
											if SN == "DWARFSTURDINESSPHYMITP" then
												return 1;
											else
												return 0;
											end
										elseif SN == "DWARFSTURDINESSMIGHT" then
											return CalcStat("MightT",L,1.0);
										else
											return 0;
										end
									elseif SN > "DWARFSTURDINESSVITALITY" then
										if SN < "DWARFUNWEARBATTLEICPR" then
											if SN == "DWARFUNWEARBATTLEICMR" then
												return CalcStat("ICMRT",L,0.6);
											else
												return 0;
											end
										elseif SN > "DWARFUNWEARBATTLEICPR" then
											if SN == "DWARFUNWEARBATTLENCMR" then
												return -CalcStat("NCMRT",L,0.4);
											else
												return 0;
											end
										else
											return CalcStat("ICPRT",L,0.6);
										end
									else
										return CalcStat("VitalityT",L,1.0);
									end
								else
									return CalcStat("AgilityT",L,0.4);
								end
							elseif SN > "DWARFUNWEARBATTLENCPR" then
								if SN < "ELFRDTRAITAGILITY" then
									if SN < "ELFFRIENDOFMANFATE" then
										if SN > "ELFAGILITYWOODSAGILITY" then
											if SN == "ELFFADINGFIRSTBORNFATE" then
												return -CalcStat("FateT",L,0.4);
											else
												return 0;
											end
										elseif SN == "ELFAGILITYWOODSAGILITY" then
											return CalcStat("AgilityT",L,1.0);
										else
											return 0;
										end
									elseif SN > "ELFFRIENDOFMANFATE" then
										if SN < "ELFRDPSVONENAME" then
											if SN == "ELFRDPSVONEFATE" then
												return CalcStat("ElfFriendOfManFate",L);
											else
												return 0;
											end
										elseif SN > "ELFRDPSVONENAME" then
											if SN == "ELFRDPSVTWONAME" then
												return "";
											else
												return 0;
											end
										else
											return "Friend Of Man";
										end
									else
										return CalcStat("FateT",L,1.0);
									end
								elseif SN > "ELFRDTRAITAGILITY" then
									if SN < "ELFRDTRAITNCMR" then
										if SN > "ELFRDTRAITFATE" then
											if SN == "ELFRDTRAITMORALE" then
												return CalcStat("ElfSorrowFirstbornMorale",L);
											else
												return 0;
											end
										elseif SN == "ELFRDTRAITFATE" then
											return CalcStat("ElfFadingFirstbornFate",L);
										else
											return 0;
										end
									elseif SN > "ELFRDTRAITNCMR" then
										if SN < "ELFSORROWFIRSTBORNNCMR" then
											if SN == "ELFSORROWFIRSTBORNMORALE" then
												return -CalcStat("MoraleT",L,0.4);
											else
												return 0;
											end
										elseif SN > "ELFSORROWFIRSTBORNNCMR" then
											if SN == "EVADE" then
												return CalcStat("BPE",L,N);
											else
												return 0;
											end
										else
											return -CalcStat("NCMRT",L,0.4);
										end
									else
										return CalcStat("ElfSorrowFirstbornNCMR",L);
									end
								else
									return CalcStat("ElfAgilityWoodsAgility",L);
								end
							else
								return -CalcStat("NCPRT",L,0.4);
							end
						elseif SN > "EVADEPBONUS" then
							if SN < "FINESSEPPRAT" then
								if SN < "EVADEPRATPCAPR" then
									if SN < "EVADEPRATPA" then
										if SN > "EVADEPPRAT" then
											if SN == "EVADEPRATP" then
												return CalcStat("BPEPRatP",L,N);
											else
												return 0;
											end
										elseif SN == "EVADEPPRAT" then
											return CalcStat("BPEPPRat",L,N);
										else
											return 0;
										end
									elseif SN > "EVADEPRATPA" then
										if SN < "EVADEPRATPC" then
											if SN == "EVADEPRATPB" then
												return CalcStat("BPEPRatPB",L);
											else
												return 0;
											end
										elseif SN > "EVADEPRATPC" then
											if SN == "EVADEPRATPCAP" then
												return CalcStat("BPEPRatPCap",L);
											else
												return 0;
											end
										else
											return CalcStat("BPEPRatPC",L);
										end
									else
										return CalcStat("BPEPRatPA",L);
									end
								elseif SN > "EVADEPRATPCAPR" then
									if SN < "FATET" then
										if SN > "EVADET" then
											if SN == "FATE" then
												return RoundDblDown(StatLinInter("PntMPFate","ItemProgVital","ProgBFate","",L,N,0));
											else
												return 0;
											end
										elseif SN == "EVADET" then
											return CalcStat("BPET",L,N);
										else
											return 0;
										end
									elseif SN > "FATET" then
										if SN < "FINESSE" then
											if SN == "FEARRESISTT" then
												return CalcStat("ResistAddT",L,N);
											else
												return 0;
											end
										elseif SN > "FINESSE" then
											if SN == "FINESSEPBONUS" then
												return 0;
											else
												return 0;
											end
										else
											return StatLinInter("PntMPFinesse","ItemProg","ProgBFinesse","",L,N,0);
										end
									else
										return RoundDblDown(StatLinInter("PntMPFate","TraitProgVital","ProgBFate","",L,N,0));
									end
								else
									return CalcStat("BPEPRatPCapR",L);
								end
							elseif SN > "FINESSEPPRAT" then
								if SN < "FINESSET" then
									if SN < "FINESSEPRATPB" then
										if SN > "FINESSEPRATP" then
											if SN == "FINESSEPRATPA" then
												return 150;
											else
												return 0;
											end
										elseif SN == "FINESSEPRATP" then
											return CalcPercAB(CalcStat("FinessePRatPA",L),CalcStat("FinessePRatPB",L),CalcStat("FinessePRatPCap",L),N);
										else
											return 0;
										end
									elseif SN > "FINESSEPRATPB" then
										if SN < "FINESSEPRATPCAP" then
											if SN == "FINESSEPRATPC" then
												return 0.5;
											else
												return 0;
											end
										elseif SN > "FINESSEPRATPCAP" then
											if SN == "FINESSEPRATPCAPR" then
												return CalcStat("FinessePRatPB",L)*CalcStat("FinessePRatPC",L);
											else
												return 0;
											end
										else
											return 50;
										end
									else
										return CalcStat("BRatRounded",L,"BRatStandard");
									end
								elseif SN > "FINESSET" then
									if SN < "FOODNCMRH" then
										if SN < "FIREMITT" then
											if SN == "FIREMIT" then
												return CalcStat("DmgTypeMit",L,N);
											else
												return 0;
											end
										elseif SN > "FIREMITT" then
											if SN == "FOODNCMRBASEPROG" then
												if Lm <= 1 then
													return (N*70.2+210)/60;
												elseif Lm <= 2 then
													return (N*84+216)/120;
												else
													return (N*83.7+270)/90;
												end
											else
												return 0;
											end
										else
											return CalcStat("DmgTypeMitT",L,N);
										end
									elseif SN > "FOODNCMRH" then
										if SN < "FOODNCMRM" then
											if SN == "FOODNCMRL" then
												return CalcStat("FoodNCMRProg",L,2)*N;
											else
												return 0;
											end
										elseif SN > "FOODNCMRM" then
											if SN == "FOODNCMRPROG" then
												if Lm <= 50 then
													return CalcStat("FoodNCMRBaseProg",N,L);
												elseif Lm <= 52 then
													return CalcStat("FoodNCMRBaseProg",N,50);
												elseif Lm <= 77 then
													return CalcStat("FoodNCMRBaseProg",N,L-3);
												elseif Lm <= 111 then
													return CalcStat("FoodNCMRBaseProg",N,RoundDbl((L+143)/3));
												elseif Lm <= 140 then
													return CalcStat("FoodNCMRBaseProg",N,85);
												elseif Lm <= 217 then
													return CalcStat("FoodNCMRBaseProg",N,RoundDbl((L+200)/4));
												elseif Lm <= 221 then
													return CalcStat("FoodNCMRBaseProg",N,104);
												elseif Lm <= 299 then
													return CalcStat("FoodNCMRBaseProg",N,105);
												elseif Lm <= 320 then
													return CalcStat("FoodNCMRBaseProg",N,RoundDbl((L-53)/(7/3)));
												elseif Lm <= 325 then
													return CalcStat("FoodNCMRBaseProg",N,114);
												elseif Lm <= 326 then
													return CalcStat("FoodNCMRBaseProg",N,115);
												else
													return ExpFmod(CalcStat("FoodNCMRProg",326,N),327,1,L,1);
												end
											else
												return 0;
											end
										else
											return CalcStat("FoodNCMRProg",L,3)*N;
										end
									else
										return CalcStat("FoodNCMRProg",L,1)*N;
									end
								else
									return StatLinInter("PntMPFinesseT","TraitProg","ProgBFinesse","",L,N,0);
								end
							else
								return CalcRatAB(CalcStat("FinessePRatPA",L),CalcStat("FinessePRatPB",L),CalcStat("FinessePRatPCapR",L),N);
							end
						else
							return CalcStat("BPEPBonus",L);
						end
					else
						return CalcStat("DwarfUnwearBattleICPR",L);
					end
				elseif SN > "FOODNCPRBASEPROG" then
					if SN < "GUARDIANCDFATETOPOWER" then
						if SN < "GRDTENDERIZET2CRITHIT" then
							if SN < "FREEPAUDACITYTACDMGP" then
								if SN < "FREEPAUDACITYDMGP" then
									if SN < "FOODNCPRM" then
										if SN > "FOODNCPRH" then
											if SN == "FOODNCPRL" then
												return CalcStat("FoodNCPRProg",L,2)*N;
											else
												return 0;
											end
										elseif SN == "FOODNCPRH" then
											return CalcStat("FoodNCPRProg",L,1)*N;
										else
											return 0;
										end
									elseif SN > "FOODNCPRM" then
										if SN < "FOODRESIST" then
											if SN == "FOODNCPRPROG" then
												if Lm <= 1 then
													return CalcStat("FoodNCPRBaseProg",N,L);
												elseif Lm <= 6 then
													return CalcStat("FoodNCPRBaseProg",N,L)-0.25;
												elseif Lm <= 24 then
													return CalcStat("FoodNCPRBaseProg",N,L)-0.5;
												elseif Lm <= 48 then
													return CalcStat("FoodNCPRBaseProg",N,L)-0.75;
												elseif Lm <= 61 then
													return RoundDblDown(CalcStat("FoodNCPRProg",48,N))+1;
												elseif Lm <= 66 then
													return RoundDblDown(CalcStat("FoodNCPRProg",48,N))+2;
												elseif Lm <= 72 then
													return RoundDblDown(CalcStat("FoodNCPRProg",48,N))+3;
												elseif Lm <= 326 then
													return RoundDblDown(CalcStat("FoodNCPRProg",48,N))+4;
												else
													return ExpFmod(CalcStat("FoodNCPRProg",326,N),327,1,L,1);
												end
											else
												return 0;
											end
										elseif SN > "FOODRESIST" then
											if SN == "FREEPAUDACITYCCDP" then
												if Lm <= 0 then
													return 0;
												elseif Lm <= 15 then
													return LinFmod(1,0.5,0.5,1,15,L);
												elseif Lm <= 16 then
													return LinFmod(1,0.5,0.4,15,16,L);
												elseif Lm <= 41 then
													return LinFmod(1,0.4,0.4,16,41,L);
												elseif Lm <= 60 then
													return LinFmod(1,0.4,0.4,41,60,L);
												else
													return CalcStat("FreepAudacityCCDP",60);
												end
											else
												return 0;
											end
										else
											return StatLinInter("PntMPFoodResist","ItemProg","ProgBResist","",L,N,0);
										end
									else
										return CalcStat("FoodNCPRProg",L,3)*N;
									end
								elseif SN > "FREEPAUDACITYDMGP" then
									if SN < "FREEPAUDACITYMORALEP" then
										if SN > "FREEPAUDACITYMELDMGP" then
											if SN == "FREEPAUDACITYMELREDP" then
												return CalcStat("FreepAudacityRedP",L);
											else
												return 0;
											end
										elseif SN == "FREEPAUDACITYMELDMGP" then
											return CalcStat("FreepAudacityDmgP",L);
										else
											return 0;
										end
									elseif SN > "FREEPAUDACITYMORALEP" then
										if SN < "FREEPAUDACITYRNGDMGP" then
											if SN == "FREEPAUDACITYREDP" then
												if Lm <= 0 then
													return 0;
												elseif Lm <= 15 then
													return LinFmod(1,1.5,1.5,1,15,L);
												elseif Lm <= 16 then
													return LinFmod(1,1.5,0.55,15,16,L);
												elseif Lm <= 36 then
													return LinFmod(1,0.55,0.55,16,36,L);
												elseif Lm <= 60 then
													return LinFmod(1,0.55,0.55,36,60,L);
												else
													return CalcStat("FreepAudacityRedP",60);
												end
											else
												return 0;
											end
										elseif SN > "FREEPAUDACITYRNGDMGP" then
											if SN == "FREEPAUDACITYRNGREDP" then
												return CalcStat("FreepAudacityRedP",L);
											else
												return 0;
											end
										else
											return CalcStat("FreepAudacityDmgP",L);
										end
									else
										if Lm <= 0 then
											return 0;
										elseif Lm <= 15 then
											return LinFmod(1,0.5,1.4,1,15,L);
										elseif Lm <= 16 then
											return LinFmod(1,1.4,1.5,15,16,L);
										elseif Lm <= 36 then
											return LinFmod(1,1.5,1.5,16,36,L);
										elseif Lm <= 41 then
											return LinFmod(1,1.5,1.5,36,41,L);
										else
											return CalcStat("FreepAudacityMoraleP",41);
										end
									end
								else
									if Lm <= 0 then
										return 0;
									elseif Lm <= 15 then
										return LinFmod(1,0.75,0.75,1,15,L);
									elseif Lm <= 16 then
										return LinFmod(1,0.75,1.2,15,16,L);
									elseif Lm <= 36 then
										return LinFmod(1,1.2,1.25,16,36,L);
									elseif Lm <= 60 then
										return LinFmod(1,1.25,1.25,36,60,L);
									else
										return CalcStat("FreepAudacityDmgP",60);
									end
								end
							elseif SN > "FREEPAUDACITYTACDMGP" then
								if SN < "FREEPBATPROMTACDMGP" then
									if SN < "FREEPBATPROMHEALTHP" then
										if SN > "FREEPAUDACITYTACREDP" then
											if SN == "FREEPBATPROMDMGP" then
												if Lm <= 0 then
													return 0;
												elseif Lm <= 1 then
													return 0.005;
												elseif Lm <= 14 then
													return LinFmod(1,0.005,0.065,2,14,L);
												else
													return 0.075;
												end
											else
												return 0;
											end
										elseif SN == "FREEPAUDACITYTACREDP" then
											return CalcStat("FreepAudacityRedP",L);
										else
											return 0;
										end
									elseif SN > "FREEPBATPROMHEALTHP" then
										if SN < "FREEPBATPROMPOWERP" then
											if SN == "FREEPBATPROMMELDMGP" then
												return CalcStat("FreepBatPromDmgP",L);
											else
												return 0;
											end
										elseif SN > "FREEPBATPROMPOWERP" then
											if SN == "FREEPBATPROMRNGDMGP" then
												return CalcStat("FreepBatPromDmgP",L);
											else
												return 0;
											end
										else
											return CalcStat("FreepBatPromVitalP",L);
										end
									else
										return CalcStat("FreepBatPromVitalP",L);
									end
								elseif SN > "FREEPBATPROMTACDMGP" then
									if SN < "FROSTMITT" then
										if SN > "FREEPBATPROMVITALP" then
											if SN == "FROSTMIT" then
												return CalcStat("DmgTypeMit",L,N);
											else
												return 0;
											end
										elseif SN == "FREEPBATPROMVITALP" then
											if Lm <= 0 then
												return 0;
											elseif Lm <= 5 then
												return LinFmod(1,1.02,1.1,1,5,L);
											elseif Lm <= 10 then
												return LinFmod(1,1.1,1.15,5,10,L);
											elseif Lm <= 15 then
												return LinFmod(1,1.15,1.2,10,15,L);
											else
												return CalcStat("FreepBatPromVitalP",15);
											end
										else
											return 0;
										end
									elseif SN > "FROSTMITT" then
										if SN < "GRDRELENTLASSFINESSE" then
											if SN == "GRDCRITDEF" then
												return CalcStat("CritDef",L);
											else
												return 0;
											end
										elseif SN > "GRDRELENTLASSFINESSE" then
											if SN == "GRDTENDERIZECRITHIT" then
												return CalcStat("CritHitT",L,CalcStat("Trait12345Choice",N)*0.2);
											else
												return 0;
											end
										else
											return CalcStat("FinesseT",L,CalcStat("Trait12345Choice",N)*0.2);
										end
									else
										return CalcStat("DmgTypeMitT",L,N);
									end
								else
									return CalcStat("FreepBatPromDmgP",L);
								end
							else
								return CalcStat("FreepAudacityDmgP",L);
							end
						elseif SN > "GRDTENDERIZET2CRITHIT" then
							if SN < "GUARDIANCDBASEICMR" then
								if SN < "GUARDIANCDAGILITYTOPARRY" then
									if SN < "GRDTENDERIZET5CRITHIT" then
										if SN > "GRDTENDERIZET3CRITHIT" then
											if SN == "GRDTENDERIZET4CRITHIT" then
												return CalcStat("CritHitT",L,CalcStat("Trait47101316Choice",N)*0.2);
											else
												return 0;
											end
										elseif SN == "GRDTENDERIZET3CRITHIT" then
											return CalcStat("CritHitT",L,CalcStat("Trait357912Choice",N)*0.2);
										else
											return 0;
										end
									elseif SN > "GRDTENDERIZET5CRITHIT" then
										if SN < "GUARDIANCDAGILITYTOCRITHIT" then
											if SN == "GRDWARDTACTTACMIT" then
												return CalcStat("TacMitT",L,CalcStat("Trait12345Choice",N)*0.2);
											else
												return 0;
											end
										elseif SN > "GUARDIANCDAGILITYTOCRITHIT" then
											if SN == "GUARDIANCDAGILITYTOEVADE" then
												return 3;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return CalcStat("CritHitT",L,CalcStat("Trait58121620Choice",N)*0.2);
									end
								elseif SN > "GUARDIANCDAGILITYTOPARRY" then
									if SN < "GUARDIANCDARMOURTOTACMIT" then
										if SN > "GUARDIANCDARMOURTOOFMIT" then
											if SN == "GUARDIANCDARMOURTOPHYMIT" then
												return 1;
											else
												return 0;
											end
										elseif SN == "GUARDIANCDARMOURTOOFMIT" then
											return 0.2;
										else
											return 0;
										end
									elseif SN > "GUARDIANCDARMOURTOTACMIT" then
										if SN < "GUARDIANCDBASEAGILITY" then
											if SN == "GUARDIANCDARMOURTYPE" then
												return 3;
											else
												return 0;
											end
										elseif SN > "GUARDIANCDBASEAGILITY" then
											if SN == "GUARDIANCDBASEFATE" then
												return CalcStat("ClassBaseFate",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseAgilityM",L);
										end
									else
										return 0.2;
									end
								else
									return 2;
								end
							elseif SN > "GUARDIANCDBASEICMR" then
								if SN < "GUARDIANCDBASEVITALITY" then
									if SN < "GUARDIANCDBASEMORALE" then
										if SN > "GUARDIANCDBASEICPR" then
											if SN == "GUARDIANCDBASEMIGHT" then
												return CalcStat("ClassBaseMightH",L);
											else
												return 0;
											end
										elseif SN == "GUARDIANCDBASEICPR" then
											return CalcStat("ClassBaseICPR",L);
										else
											return 0;
										end
									elseif SN > "GUARDIANCDBASEMORALE" then
										if SN < "GUARDIANCDBASENCPR" then
											if SN == "GUARDIANCDBASENCMR" then
												return CalcStat("ClassBaseNCMRH",L);
											else
												return 0;
											end
										elseif SN > "GUARDIANCDBASENCPR" then
											if SN == "GUARDIANCDBASEPOWER" then
												return CalcStat("ClassBasePower",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseNCPR",L);
										end
									else
										return CalcStat("ClassBaseMorale",L);
									end
								elseif SN > "GUARDIANCDBASEVITALITY" then
									if SN < "GUARDIANCDFATETOCRITHIT" then
										if SN > "GUARDIANCDBASEWILL" then
											if SN == "GUARDIANCDCANBLOCK" then
												return 1;
											else
												return 0;
											end
										elseif SN == "GUARDIANCDBASEWILL" then
											return CalcStat("ClassBaseWillL",L);
										else
											return 0;
										end
									elseif SN > "GUARDIANCDFATETOCRITHIT" then
										if SN < "GUARDIANCDFATETOICMR" then
											if SN == "GUARDIANCDFATETOFINESSE" then
												return 0.5;
											else
												return 0;
											end
										elseif SN > "GUARDIANCDFATETOICMR" then
											if SN == "GUARDIANCDFATETONCPR" then
												return 0.05;
											else
												return 0;
											end
										else
											return 0.025;
										end
									else
										return 1.5;
									end
								else
									return CalcStat("ClassBaseVitality",L);
								end
							else
								return CalcStat("ClassBaseICMRH",L);
							end
						else
							return CalcStat("CritHitT",L,CalcStat("Trait12345Choice",N)*0.4);
						end
					elseif SN > "GUARDIANCDFATETOPOWER" then
						if SN < "HNTARMOURRENDEVADE" then
							if SN < "GUARDIANCDWILLTOTACMIT" then
								if SN < "GUARDIANCDPHYMITTOOFMIT" then
									if SN < "GUARDIANCDHASPOWER" then
										if SN > "GUARDIANCDFATETORESIST" then
											if SN == "GUARDIANCDFATETOTACMIT" then
												return 0.5;
											else
												return 0;
											end
										elseif SN == "GUARDIANCDFATETORESIST" then
											return 0.5;
										else
											return 0;
										end
									elseif SN > "GUARDIANCDHASPOWER" then
										if SN < "GUARDIANCDMIGHTTOPARRY" then
											if SN == "GUARDIANCDMIGHTTOBLOCK" then
												return 3;
											else
												return 0;
											end
										elseif SN > "GUARDIANCDMIGHTTOPARRY" then
											if SN == "GUARDIANCDMIGHTTOPHYMAS" then
												return 3;
											else
												return 0;
											end
										else
											return 2;
										end
									else
										return 1;
									end
								elseif SN > "GUARDIANCDPHYMITTOOFMIT" then
									if SN < "GUARDIANCDVITALITYTONCMR" then
										if SN > "GUARDIANCDTACMASTOOUTHEAL" then
											if SN == "GUARDIANCDVITALITYTOMORALE" then
												return 4.8;
											else
												return 0;
											end
										elseif SN == "GUARDIANCDTACMASTOOUTHEAL" then
											return 1;
										else
											return 0;
										end
									elseif SN > "GUARDIANCDVITALITYTONCMR" then
										if SN < "GUARDIANCDWILLTORESIST" then
											if SN == "GUARDIANCDVITALITYTORESIST" then
												return 0.5;
											else
												return 0;
											end
										elseif SN > "GUARDIANCDWILLTORESIST" then
											if SN == "GUARDIANCDWILLTOTACMAS" then
												return 1;
											else
												return 0;
											end
										else
											return 2;
										end
									else
										return 0.12;
									end
								else
									return 1;
								end
							elseif SN > "GUARDIANCDWILLTOTACMIT" then
								if SN < "HIGHELFRDPSVONEWILL" then
									if SN < "HELFPEACEELDARNCMR" then
										if SN > "HELFFADINGFIRSTBORNFATE" then
											if SN == "HELFPEACEELDARMORALE" then
												return CalcStat("MoraleT",L,1.0);
											else
												return 0;
											end
										elseif SN == "HELFFADINGFIRSTBORNFATE" then
											return -CalcStat("FateT",L,0.4);
										else
											return 0;
										end
									elseif SN > "HELFPEACEELDARNCMR" then
										if SN < "HELFTHOSEWHOREMAINWILL" then
											if SN == "HELFSORROWUNDYINGWILL" then
												return -CalcStat("WillT",L,0.4);
											else
												return 0;
											end
										elseif SN > "HELFTHOSEWHOREMAINWILL" then
											if SN == "HIGHELFRDPSVONENAME" then
												return "Those Who Remain";
											else
												return 0;
											end
										else
											return CalcStat("WillT",L,1.0);
										end
									else
										return CalcStat("NCMRT",L,0.6);
									end
								elseif SN > "HIGHELFRDPSVONEWILL" then
									if SN < "HIGHELFRDTRAITMORALE" then
										if SN > "HIGHELFRDPSVTWONAME" then
											if SN == "HIGHELFRDTRAITFATE" then
												return CalcStat("HElfFadingFirstbornFate",L);
											else
												return 0;
											end
										elseif SN == "HIGHELFRDPSVTWONAME" then
											return "";
										else
											return 0;
										end
									elseif SN > "HIGHELFRDTRAITMORALE" then
										if SN < "HIGHELFRDTRAITWILL" then
											if SN == "HIGHELFRDTRAITNCMR" then
												return CalcStat("HElfPeaceEldarNCMR",L);
											else
												return 0;
											end
										elseif SN > "HIGHELFRDTRAITWILL" then
											if SN == "HNTARMOURRENDBLOCK" then
												return -CalcStat("ShieldBlock",L,CalcStat("Trait23456Choice",N)*0.4);
											else
												return 0;
											end
										else
											return CalcStat("HElfSorrowUndyingWill",L);
										end
									else
										return CalcStat("HElfPeaceEldarMorale",L);
									end
								else
									return CalcStat("HElfThoseWhoRemainWill",L);
								end
							else
								return 1;
							end
						elseif SN > "HNTARMOURRENDEVADE" then
							if SN < "HOBBITRDTRAITNCMR" then
								if SN < "HNTRAPIDFIREPHYMAS" then
									if SN < "HNTCAMPFIRENCMR" then
										if SN > "HNTARMOURRENDPARRY" then
											if SN == "HNTBREACHFINDERRNGMIT" then
												return (-20)*L;
											else
												return 0;
											end
										elseif SN == "HNTARMOURRENDPARRY" then
											return -CalcStat("ParryT",L,CalcStat("Trait23456Choice",N)*0.4);
										else
											return 0;
										end
									elseif SN > "HNTCAMPFIRENCMR" then
										if SN < "HNTPRECISIONSTANCEFINESSE" then
											if SN == "HNTCAMPFIRENCPR" then
												return CalcStat("NCPRT",L,1.2);
											else
												return 0;
											end
										elseif SN > "HNTPRECISIONSTANCEFINESSE" then
											if SN == "HNTPURGEPOISONRESIST" then
												return CalcStat("PoisonResistT",L,4);
											else
												return 0;
											end
										else
											return CalcStat("FinesseT",L,1.6);
										end
									else
										return CalcStat("NCMRT",L,1.2);
									end
								elseif SN > "HNTRAPIDFIREPHYMAS" then
									if SN < "HOBBITRDPSVONEMIGHT" then
										if SN > "HNTRAPIDFIRESEL" then
											if SN == "HNTSTRENGTHSTANCECRITHIT" then
												return CalcStat("CritHitT",L,4);
											else
												return 0;
											end
										elseif SN == "HNTRAPIDFIRESEL" then
											if Lm <= 0 then
												return 0;
											elseif Lm <= 8 then
												return L+2;
											elseif Lm <= 10 then
												return 2*L-6;
											else
												return CalcStat("HntRapidFireSel",10);
											end
										else
											return 0;
										end
									elseif SN > "HOBBITRDPSVONEMIGHT" then
										if SN < "HOBBITRDPSVTWONAME" then
											if SN == "HOBBITRDPSVONENAME" then
												return "Hobbit-stature";
											else
												return 0;
											end
										elseif SN > "HOBBITRDPSVTWONAME" then
											if SN == "HOBBITRDTRAITMIGHT" then
												return CalcStat("HobSmallSizeMight",L);
											else
												return 0;
											end
										else
											return "";
										end
									else
										return CalcStat("HobHobbitStatureMight",L);
									end
								else
									return CalcStat("PhyMasT",L,CalcStat("HntRapidFireSel",N)*0.2);
								end
							elseif SN > "HOBBITRDTRAITNCMR" then
								if SN < "HUNTERCDAGILITYTOCRITHIT" then
									if SN < "HOBHOBBITTOUGHNVITALITY" then
										if SN > "HOBBITRDTRAITVITALITY" then
											if SN == "HOBHOBBITSTATUREMIGHT" then
												return CalcStat("MightT",L,1.0);
											else
												return 0;
											end
										elseif SN == "HOBBITRDTRAITVITALITY" then
											return CalcStat("HobHobbitToughnVitality",L);
										else
											return 0;
										end
									elseif SN > "HOBHOBBITTOUGHNVITALITY" then
										if SN < "HOBSMALLSIZEMIGHT" then
											if SN == "HOBRAPIDRECOVERYNCMR" then
												return CalcStat("NCMRT",L,0.6);
											else
												return 0;
											end
										elseif SN > "HOBSMALLSIZEMIGHT" then
											if SN == "HOPEMORALEP" then
												if Lm <= 5 then
													return DataTableValue({-0.99,-0.97,-0.95,-0.9,-0.85,-0.8,-0.65,-0.6,-0.5,-0.4,-0.3,-0.2,-0.15,-0.1,-0.05,0,0.01,0.02,0.03,0.04,0.05},L+16);
												else
													return 0.05;
												end
											else
												return 0;
											end
										else
											return -CalcStat("MightT",L,0.4);
										end
									else
										return CalcStat("VitalityT",L,1.0);
									end
								elseif SN > "HUNTERCDAGILITYTOCRITHIT" then
									if SN < "HUNTERCDARMOURTOOFMIT" then
										if SN < "HUNTERCDAGILITYTOPARRY" then
											if SN == "HUNTERCDAGILITYTOEVADE" then
												return 3;
											else
												return 0;
											end
										elseif SN > "HUNTERCDAGILITYTOPARRY" then
											if SN == "HUNTERCDAGILITYTOPHYMAS" then
												return 3;
											else
												return 0;
											end
										else
											return 2;
										end
									elseif SN > "HUNTERCDARMOURTOOFMIT" then
										if SN < "HUNTERCDARMOURTOTACMIT" then
											if SN == "HUNTERCDARMOURTOPHYMIT" then
												return 1;
											else
												return 0;
											end
										elseif SN > "HUNTERCDARMOURTOTACMIT" then
											if SN == "HUNTERCDARMOURTYPE" then
												return 2;
											else
												return 0;
											end
										else
											return 0.2;
										end
									else
										return 0.2;
									end
								else
									return 1;
								end
							else
								return CalcStat("HobRapidRecoveryNCMR",L);
							end
						else
							return -CalcStat("EvadeT",L,CalcStat("Trait23456Choice",N)*0.4);
						end
					else
						return 1;
					end
				else
					if Lm <= 1 then
						return (N*75+300)/60;
					elseif Lm <= 2 then
						return (N*100+350)/120;
					else
						return (N*90+337.5)/90;
					end
				end
			elseif SN > "HUNTERCDBASEAGILITY" then
				if SN < "LOREMASTERCDFATETOTACMIT" then
					if SN < "ITEMPROGVIRTUEMASTERY" then
						if SN < "HUNTERCDWILLTOTACMAS" then
							if SN < "HUNTERCDFATETONCPR" then
								if SN < "HUNTERCDBASENCPR" then
									if SN < "HUNTERCDBASEICPR" then
										if SN > "HUNTERCDBASEFATE" then
											if SN == "HUNTERCDBASEICMR" then
												return CalcStat("ClassBaseICMRM",L);
											else
												return 0;
											end
										elseif SN == "HUNTERCDBASEFATE" then
											return CalcStat("ClassBaseFate",L);
										else
											return 0;
										end
									elseif SN > "HUNTERCDBASEICPR" then
										if SN < "HUNTERCDBASEMORALE" then
											if SN == "HUNTERCDBASEMIGHT" then
												return CalcStat("ClassBaseMightM",L);
											else
												return 0;
											end
										elseif SN > "HUNTERCDBASEMORALE" then
											if SN == "HUNTERCDBASENCMR" then
												return CalcStat("ClassBaseNCMRM",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseMorale",L);
										end
									else
										return CalcStat("ClassBaseICPR",L);
									end
								elseif SN > "HUNTERCDBASENCPR" then
									if SN < "HUNTERCDBASEWILL" then
										if SN > "HUNTERCDBASEPOWER" then
											if SN == "HUNTERCDBASEVITALITY" then
												return CalcStat("ClassBaseVitality",L);
											else
												return 0;
											end
										elseif SN == "HUNTERCDBASEPOWER" then
											return CalcStat("ClassBasePower",L);
										else
											return 0;
										end
									elseif SN > "HUNTERCDBASEWILL" then
										if SN < "HUNTERCDFATETOFINESSE" then
											if SN == "HUNTERCDFATETOCRITHIT" then
												return 1.5;
											else
												return 0;
											end
										elseif SN > "HUNTERCDFATETOFINESSE" then
											if SN == "HUNTERCDFATETOICMR" then
												return 0.025;
											else
												return 0;
											end
										else
											return 0.5;
										end
									else
										return CalcStat("ClassBaseWillL",L);
									end
								else
									return CalcStat("ClassBaseNCPR",L);
								end
							elseif SN > "HUNTERCDFATETONCPR" then
								if SN < "HUNTERCDPHYMITTOOFMIT" then
									if SN < "HUNTERCDFATETOTACMIT" then
										if SN > "HUNTERCDFATETOPOWER" then
											if SN == "HUNTERCDFATETORESIST" then
												return 0.5;
											else
												return 0;
											end
										elseif SN == "HUNTERCDFATETOPOWER" then
											return 1;
										else
											return 0;
										end
									elseif SN > "HUNTERCDFATETOTACMIT" then
										if SN < "HUNTERCDMIGHTTOBLOCK" then
											if SN == "HUNTERCDHASPOWER" then
												return 1;
											else
												return 0;
											end
										elseif SN > "HUNTERCDMIGHTTOBLOCK" then
											if SN == "HUNTERCDMIGHTTOPARRY" then
												return 2;
											else
												return 0;
											end
										else
											return 3;
										end
									else
										return 0.5;
									end
								elseif SN > "HUNTERCDPHYMITTOOFMIT" then
									if SN < "HUNTERCDVITALITYTONCMR" then
										if SN > "HUNTERCDTACMASTOOUTHEAL" then
											if SN == "HUNTERCDVITALITYTOMORALE" then
												return 4;
											else
												return 0;
											end
										elseif SN == "HUNTERCDTACMASTOOUTHEAL" then
											return 1;
										else
											return 0;
										end
									elseif SN > "HUNTERCDVITALITYTONCMR" then
										if SN < "HUNTERCDVITALITYTOTACMIT" then
											if SN == "HUNTERCDVITALITYTORESIST" then
												return 0.5;
											else
												return 0;
											end
										elseif SN > "HUNTERCDVITALITYTOTACMIT" then
											if SN == "HUNTERCDWILLTORESIST" then
												return 2;
											else
												return 0;
											end
										else
											return 0.5;
										end
									else
										return 0.12;
									end
								else
									return 1;
								end
							else
								return 0.05;
							end
						elseif SN > "HUNTERCDWILLTOTACMAS" then
							if SN < "INHEAL" then
								if SN < "INDMGPPRAT" then
									if SN < "ICMRT" then
										if SN > "HUNTERCDWILLTOTACMIT" then
											if SN == "ICMR" then
												return StatLinInter("PntMPICMR","ItemProgVital","ProgBICMR","",L,N,1);
											else
												return 0;
											end
										elseif SN == "HUNTERCDWILLTOTACMIT" then
											return 1;
										else
											return 0;
										end
									elseif SN > "ICMRT" then
										if SN < "ICPRT" then
											if SN == "ICPR" then
												return StatLinInter("PntMPICPR","ItemProgVital","ProgBICPR","",L,N,99);
											else
												return 0;
											end
										elseif SN > "ICPRT" then
											if SN == "INDMGPBONUS" then
												return 0;
											else
												return 0;
											end
										else
											return StatLinInter("PntMPICPR","TraitProgVital","ProgBICPR","",L,N,99);
										end
									else
										return StatLinInter("PntMPICMR","TraitProgVital","ProgBICMR","",L,N,1);
									end
								elseif SN > "INDMGPPRAT" then
									if SN < "INDMGPRATPB" then
										if SN > "INDMGPRATP" then
											if SN == "INDMGPRATPA" then
												return 1200;
											else
												return 0;
											end
										elseif SN == "INDMGPRATP" then
											return CalcPercAB(CalcStat("InDmgPRatPA",L),CalcStat("InDmgPRatPB",L),CalcStat("InDmgPRatPCap",L),N);
										else
											return 0;
										end
									elseif SN > "INDMGPRATPB" then
										if SN < "INDMGPRATPCAP" then
											if SN == "INDMGPRATPC" then
												return 0.5;
											else
												return 0;
											end
										elseif SN > "INDMGPRATPCAP" then
											if SN == "INDMGPRATPCAPR" then
												return CalcStat("InDmgPRatPB",L)*CalcStat("InDmgPRatPC",L);
											else
												return 0;
											end
										else
											return 400;
										end
									else
										return CalcStat("BRatRounded",L,"BRatStandard");
									end
								else
									return CalcRatAB(CalcStat("InDmgPRatPA",L),CalcStat("InDmgPRatPB",L),CalcStat("InDmgPRatPCapR",L),N);
								end
							elseif SN > "INHEAL" then
								if SN < "INHEALPRATPCAP" then
									if SN < "INHEALPRATP" then
										if SN > "INHEALPBONUS" then
											if SN == "INHEALPPRAT" then
												return CalcRatAB(CalcStat("InHealPRatPA",L),CalcStat("InHealPRatPB",L),CalcStat("InHealPRatPCapR",L),N);
											else
												return 0;
											end
										elseif SN == "INHEALPBONUS" then
											return 0;
										else
											return 0;
										end
									elseif SN > "INHEALPRATP" then
										if SN < "INHEALPRATPB" then
											if SN == "INHEALPRATPA" then
												return 75;
											else
												return 0;
											end
										elseif SN > "INHEALPRATPB" then
											if SN == "INHEALPRATPC" then
												return 0.5;
											else
												return 0;
											end
										else
											return CalcStat("BRatRounded",L,"BRatStandard");
										end
									else
										return CalcPercAB(CalcStat("InHealPRatPA",L),CalcStat("InHealPRatPB",L),CalcStat("InHealPRatPCap",L),N);
									end
								elseif SN > "INHEALPRATPCAP" then
									if SN < "INSTRMORALE" then
										if SN > "INHEALPRATPCAPR" then
											if SN == "INHEALT" then
												return StatLinInter("PntMPInHeal","TraitProg","ProgBInHeal","",L,N,0);
											else
												return 0;
											end
										elseif SN == "INHEALPRATPCAPR" then
											return CalcStat("InHealPRatPB",L)*CalcStat("InHealPRatPC",L);
										else
											return 0;
										end
									elseif SN > "INSTRMORALE" then
										if SN < "ITEMPROG" then
											if SN == "INSTRPOWER" then
												if Lm <= 9 then
													return RoundDbl(0.95*L+12.6);
												elseif Lm <= 18 then
													return RoundDbl(1.95*L+3);
												elseif Lm <= 26 then
													return RoundDbl(2.95*L-15.55);
												elseif Lm <= 35 then
													return RoundDbl(3.95*L-42.15);
												elseif Lm <= 42 then
													return RoundDbl(4.95*L-77.7);
												elseif Lm <= 45 then
													return RoundDbl(5.95*L-120.35);
												elseif Lm <= 105 then
													return RoundDbl(6*L-124);
												else
													return CalcStat("ProgExtMpExpRnd",L,CalcStat("InstrPower",105));
												end
											else
												return 0;
											end
										elseif SN > "ITEMPROG" then
											if SN == "ITEMPROGCLASSIC" then
												return {{1,79,80,200,225,300,349,350,399,400,449,450,499,500,549},{1,75,76,100,105,106,115,116,120,121,130,131,140,141,150}};
											else
												return 0;
											end
										else
											return {{1,25,79,80,200,225,300,349,350,399,400,449,450,499,500,549},{1,25,75,76,100,105,106,115,116,120,121,130,131,140,141,150}};
										end
									else
										if Lm <= 105 then
											return RoundDbl(4.04*L);
										else
											return CalcStat("ProgExtLowExpRnd",L,CalcStat("InstrMorale",105));
										end
									end
								else
									return 25;
								end
							else
								return StatLinInter("PntMPInHeal","ItemProg","ProgBInHeal","",L,N,0);
							end
						else
							return 1;
						end
					elseif SN > "ITEMPROGVIRTUEMASTERY" then
						if SN < "LMRINGOFFIREPARRY" then
							if SN < "LMCATMINTBEARARMOUR" then
								if SN < "LIGHTMIT" then
									if SN < "L" then
										if SN > "ITEMPROGVIRTUEMORALE" then
											if SN == "ITEMPROGVITAL" then
												return {{1,25,50,60,79,80,200,225,300,349,350,399,400,449,450,499},{1,25,50,60,75,76,100,105,106,115,116,120,121,130,131,140}};
											else
												return 0;
											end
										elseif SN == "ITEMPROGVIRTUEMORALE" then
											return {{0,1,2,50,80,200,225,300,349,350,399,400,449,450,499,500},{0,1,2,50,76,100,105,106,115,116,120,121,130,131,140,141}};
										else
											return 0;
										end
									elseif SN > "L" then
										if SN < "LI2ILVLCAP" then
											if SN == "LEVELCAP" then
												return 140;
											else
												return 0;
											end
										elseif SN > "LI2ILVLCAP" then
											if SN == "LI2REFORGEILVL" then
												if Lm <= 135 then
													return CalcStat("AwardILvlI",L);
												else
													return N;
												end
											else
												return 0;
											end
										else
											return 495;
										end
									else
										return L;
									end
								elseif SN > "LIGHTMIT" then
									if SN < "LIGHTNINGMITT" then
										if SN > "LIGHTMITT" then
											if SN == "LIGHTNINGMIT" then
												return CalcStat("DmgTypeMit",L,N);
											else
												return 0;
											end
										elseif SN == "LIGHTMITT" then
											return CalcStat("DmgTypeMitT",L,N);
										else
											return 0;
										end
									elseif SN > "LIGHTNINGMITT" then
										if SN < "LMANCIENTCRAFTARMOUR" then
											if SN == "LINEAR1TOCAP" then
												return LinFmod(1,1,N,1,CalcStat("LevelCap",L),L);
											else
												return 0;
											end
										elseif SN > "LMANCIENTCRAFTARMOUR" then
											if SN == "LMANCIENTWISDOMWILL" then
												if Lm <= 105 then
													return RoundDbl(1.1*L);
												else
													return CalcStat("ProgExtLowExpRnd",L,CalcStat("LmAncientWisdomWill",105));
												end
											else
												return 0;
											end
										else
											return -CalcStat("ArmourT",L,2.4);
										end
									else
										return CalcStat("DmgTypeMitT",L,N);
									end
								else
									return CalcStat("DmgTypeMit",L,N);
								end
							elseif SN > "LMCATMINTBEARARMOUR" then
								if SN < "LMKNOWLEDGELMTACRES" then
									if SN < "LMHEARTYDIETMORALE" then
										if SN > "LMEAGLENOBILITYICPR" then
											if SN == "LMEVASIONRAVENARMOUR" then
												return CalcStat("ArmourT",L,0.8);
											else
												return 0;
											end
										elseif SN == "LMEAGLENOBILITYICPR" then
											return CalcStat("ICPRT",L,0.8);
										else
											return 0;
										end
									elseif SN > "LMHEARTYDIETMORALE" then
										if SN < "LMKNOWLEDGELMPHYRES" then
											if SN == "LMKNOWLEDGELMCRYRES" then
												return -CalcStat("CryResistT",L,0.6);
											else
												return 0;
											end
										elseif SN > "LMKNOWLEDGELMPHYRES" then
											if SN == "LMKNOWLEDGELMSONGRES" then
												return -CalcStat("SongResistT",L,0.6);
											else
												return 0;
											end
										else
											return -CalcStat("PhyResistT",L,0.6);
										end
									else
										return CalcStat("Morale",L,CalcStat("Trait12345Choice",N)*0.4);
									end
								elseif SN > "LMKNOWLEDGELMTACRES" then
									if SN < "LMRAVENSHOTRAVWINGFIXTACMIT" then
										if SN > "LMPREPFORWARPETTACMAS" then
											if SN == "LMPREPFORWARTACMAS" then
												return CalcStat("TacMasT",L,CalcStat("Trait12345Choice",N)*0.4);
											else
												return 0;
											end
										elseif SN == "LMPREPFORWARPETTACMAS" then
											return CalcStat("TacMas",L,3.0);
										else
											return 0;
										end
									elseif SN > "LMRAVENSHOTRAVWINGFIXTACMIT" then
										if SN < "LMRINGOFFIREBLOCK" then
											if SN == "LMRAVENSHOTRAVWINGTACMIT" then
												return CalcStat("TacMit",L,2);
											else
												return 0;
											end
										elseif SN > "LMRINGOFFIREBLOCK" then
											if SN == "LMRINGOFFIREEVADE" then
												return -CalcStat("EvadeT",L,2.4);
											else
												return 0;
											end
										else
											return -CalcStat("BlockT",L,2.4);
										end
									else
										return CalcStat("TacMitT",L,2);
									end
								else
									return -CalcStat("TacResistT",L,0.6);
								end
							else
								return CalcStat("BaseArmour",L,15);
							end
						elseif SN > "LMRINGOFFIREPARRY" then
							if SN < "LOREMASTERCDBASEFATE" then
								if SN < "LOREMASTERCDAGILITYTOEVADE" then
									if SN < "LMSWSTAFFBUGPARRYOLD" then
										if SN > "LMSWSTAFFBUGMORALE" then
											if SN == "LMSWSTAFFBUGPARRY" then
												return CalcStat("U371LegacyStatFix",L,"LmSwStaffBugParryOld");
											else
												return 0;
											end
										elseif SN == "LMSWSTAFFBUGMORALE" then
											if Lm <= 105 then
												return RoundDbl(LinFmod(1,16.2,696.9,1,105,L));
											else
												return CalcStat("ProgExtLowExpRnd",L,CalcStat("LmSwStaffBugMorale",105));
											end
										else
											return 0;
										end
									elseif SN > "LMSWSTAFFBUGPARRYOLD" then
										if SN < "LOEPASSIVE" then
											if SN == "LOE" then
												return 2*N;
											else
												return 0;
											end
										elseif SN > "LOEPASSIVE" then
											if SN == "LOREMASTERCDAGILITYTOCRITHIT" then
												return 1;
											else
												return 0;
											end
										else
											if Lm <= 115 then
												return 0;
											elseif Lm <= 119 then
												return 20*L-2300;
											else
												return CalcStat("LoEPassive",119);
											end
										end
									else
										if Lm <= 105 then
											return RoundDbl(44.44*L);
										else
											return CalcStat("ProgExtHighLinExpRnd",L,CalcStat("LmSwStaffBugParryOld",105));
										end
									end
								elseif SN > "LOREMASTERCDAGILITYTOEVADE" then
									if SN < "LOREMASTERCDARMOURTOPHYMIT" then
										if SN > "LOREMASTERCDAGILITYTOPARRY" then
											if SN == "LOREMASTERCDARMOURTOOFMIT" then
												return 0.2;
											else
												return 0;
											end
										elseif SN == "LOREMASTERCDAGILITYTOPARRY" then
											return 2;
										else
											return 0;
										end
									elseif SN > "LOREMASTERCDARMOURTOPHYMIT" then
										if SN < "LOREMASTERCDARMOURTYPE" then
											if SN == "LOREMASTERCDARMOURTOTACMIT" then
												return 0.2;
											else
												return 0;
											end
										elseif SN > "LOREMASTERCDARMOURTYPE" then
											if SN == "LOREMASTERCDBASEAGILITY" then
												return CalcStat("ClassBaseAgilityL",L);
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 1;
									end
								else
									return 3;
								end
							elseif SN > "LOREMASTERCDBASEFATE" then
								if SN < "LOREMASTERCDBASEPOWER" then
									if SN < "LOREMASTERCDBASEMIGHT" then
										if SN > "LOREMASTERCDBASEICMR" then
											if SN == "LOREMASTERCDBASEICPR" then
												return CalcStat("ClassBaseICPR",L);
											else
												return 0;
											end
										elseif SN == "LOREMASTERCDBASEICMR" then
											return CalcStat("ClassBaseICMRL",L);
										else
											return 0;
										end
									elseif SN > "LOREMASTERCDBASEMIGHT" then
										if SN < "LOREMASTERCDBASENCMR" then
											if SN == "LOREMASTERCDBASEMORALE" then
												return CalcStat("ClassBaseMorale",L);
											else
												return 0;
											end
										elseif SN > "LOREMASTERCDBASENCMR" then
											if SN == "LOREMASTERCDBASENCPR" then
												return CalcStat("ClassBaseNCPR",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseNCMRL",L);
										end
									else
										return CalcStat("ClassBaseMightM",L);
									end
								elseif SN > "LOREMASTERCDBASEPOWER" then
									if SN < "LOREMASTERCDFATETOICMR" then
										if SN < "LOREMASTERCDBASEWILL" then
											if SN == "LOREMASTERCDBASEVITALITY" then
												return CalcStat("ClassBaseVitality",L);
											else
												return 0;
											end
										elseif SN > "LOREMASTERCDBASEWILL" then
											if SN == "LOREMASTERCDFATETOCRITHIT" then
												return 1.5;
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseWillH",L);
										end
									elseif SN > "LOREMASTERCDFATETOICMR" then
										if SN < "LOREMASTERCDFATETOPOWER" then
											if SN == "LOREMASTERCDFATETONCPR" then
												return 0.05;
											else
												return 0;
											end
										elseif SN > "LOREMASTERCDFATETOPOWER" then
											if SN == "LOREMASTERCDFATETORESIST" then
												return 0.5;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 0.025;
									end
								else
									return CalcStat("ClassBasePower",L);
								end
							else
								return CalcStat("ClassBaseFate",L);
							end
						else
							return -CalcStat("ParryT",L,2.4);
						end
					else
						return {{0,1,25,79,80,200,225,300,349,350,400,449,450,499,500,549},{0,1,25,75,76,100,105,106,115,116,121,130,131,140,141,150}};
					end
				elseif SN > "LOREMASTERCDFATETOTACMIT" then
					if SN < "MARINERCDBASEVITALITY" then
						if SN < "MANRDPSVONENAME" then
							if SN < "LOREMASTERCDWILLTOTACMIT" then
								if SN < "LOREMASTERCDVITALITYTONCMR" then
									if SN < "LOREMASTERCDMIGHTTOPARRY" then
										if SN > "LOREMASTERCDHASPOWER" then
											if SN == "LOREMASTERCDMIGHTTOBLOCK" then
												return 3;
											else
												return 0;
											end
										elseif SN == "LOREMASTERCDHASPOWER" then
											return 1;
										else
											return 0;
										end
									elseif SN > "LOREMASTERCDMIGHTTOPARRY" then
										if SN < "LOREMASTERCDTACMASTOOUTHEAL" then
											if SN == "LOREMASTERCDPHYMITTOOFMIT" then
												return 1;
											else
												return 0;
											end
										elseif SN > "LOREMASTERCDTACMASTOOUTHEAL" then
											if SN == "LOREMASTERCDVITALITYTOMORALE" then
												return 4;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 2;
									end
								elseif SN > "LOREMASTERCDVITALITYTONCMR" then
									if SN < "LOREMASTERCDWILLTOEVADE" then
										if SN > "LOREMASTERCDVITALITYTORESIST" then
											if SN == "LOREMASTERCDVITALITYTOTACMIT" then
												return 0.5;
											else
												return 0;
											end
										elseif SN == "LOREMASTERCDVITALITYTORESIST" then
											return 0.5;
										else
											return 0;
										end
									elseif SN > "LOREMASTERCDWILLTOEVADE" then
										if SN < "LOREMASTERCDWILLTORESIST" then
											if SN == "LOREMASTERCDWILLTOFINESSE" then
												return 1;
											else
												return 0;
											end
										elseif SN > "LOREMASTERCDWILLTORESIST" then
											if SN == "LOREMASTERCDWILLTOTACMAS" then
												return 3;
											else
												return 0;
											end
										else
											return 2;
										end
									else
										return 1;
									end
								else
									return 0.12;
								end
							elseif SN > "LOREMASTERCDWILLTOTACMIT" then
								if SN < "LVLTOILVL" then
									if SN < "LVLBONUSPOWRES" then
										if SN > "LVLBONUSMORRES" then
											if SN == "LVLBONUSPHYDMG" then
												return 0.1;
											else
												return 0;
											end
										elseif SN == "LVLBONUSMORRES" then
											return 0.1;
										else
											return 0;
										end
									elseif SN > "LVLBONUSPOWRES" then
										if SN < "LVLEXPCOST" then
											if SN == "LVLBONUSTACDMG" then
												return 0.1;
											else
												return 0;
											end
										elseif SN > "LVLEXPCOST" then
											if SN == "LVLEXPCOSTTOT" then
												if Lm <= 1 then
													return 0;
												else
													return CalcStat("LvlExpCostTot",L-1)+CalcStat("LvlExpCost",L);
												end
											else
												return 0;
											end
										else
											if Lm <= 1 then
												return 0;
											elseif Lm <= 5 then
												return RoundDbl(12.5*L*L+12.5666666666667*L+24.8666666666667);
											elseif Lm <= 10 then
												return RoundDbl(33.8*L*L-179.48*L+452.6);
											elseif Lm <= 15 then
												return RoundDbl(55.05*L*L-583.77*L+2370.5);
											elseif Lm <= 20 then
												return RoundDbl(76.2*L*L-1196.96*L+6809);
											elseif Lm <= 25 then
												return RoundDbl(97.4*L*L-2023*L+14849.8);
											elseif Lm <= 30 then
												return RoundDbl(118.7*L*L-3066.02 *L+27612.8);
											elseif Lm <= 35 then
												return RoundDbl(139.95*L*L-4319.23*L+46084.1);
											elseif Lm <= 40 then
												return RoundDbl(161.2*L*L-5785.04*L+71356.2);
											elseif Lm <= 45 then
												return RoundDbl(182.5*L*L-7467.38*L+104569.8);
											elseif Lm <= 50 then
												return RoundDbl(203.8*L*L-9363.48*L+146761.8);
											elseif Lm <= 55 then
												return RoundDbl(225.05*L*L-11467.77*L+198851.3);
											elseif Lm <= 60 then
												return RoundDbl(246.3*L*L-13784.46*L+261988);
											elseif Lm <= 70 then
												return RoundDbl(ExpFmod(CalcStat("LvlExpCost",60),61,5.071,L,nil,3.485));
											elseif Lm <= 75 then
												return RoundDbl(ExpFmod(CalcStat("LvlExpCost",70),71,5.072,L,nil,-0.95));
											else
												return ExpFmod(CalcStat("LvlExpCost",75),76,5,L,0,-0.5);
											end
										end
									else
										return CalcStat("SkillPowerCost",L,N);
									end
								elseif SN > "LVLTOILVL" then
									if SN < "MANDIMMANKINDWILL" then
										if SN > "MAIN" then
											if SN == "MAINT" then
												return RoundDblDown(StatLinInter("PntMPMain","TraitProg","ProgBMain","",L,N,0));
											else
												return 0;
											end
										elseif SN == "MAIN" then
											return RoundDblDown(StatLinInter("PntMPMain","ItemProg","ProgBMain","",L,N,0));
										else
											return 0;
										end
									elseif SN > "MANDIMMANKINDWILL" then
										if SN < "MANFOURTHAGEWILL" then
											if SN == "MANEASILYINSPINHEALP" then
												return 5;
											else
												return 0;
											end
										elseif SN > "MANFOURTHAGEWILL" then
											if SN == "MANGIFTOFMENFATE" then
												return CalcStat("FateT",L,1.0);
											else
												return 0;
											end
										else
											return CalcStat("WillT",L,1.0);
										end
									else
										return -CalcStat("WillT",L,0.4);
									end
								else
									if Lm <= 106 then
										return LinFmod(1,225,300,105,106,L);
									elseif Lm <= 115 then
										return LinFmod(1,300,349,106,115,L);
									elseif Lm <= 116 then
										return LinFmod(1,349,350,115,116,L);
									elseif Lm <= 120 then
										return LinFmod(1,350,399,116,120,L);
									elseif Lm <= 121 then
										return LinFmod(1,399,400,120,121,L);
									elseif Lm <= 130 then
										return LinFmod(1,400,449,121,130,L);
									elseif Lm <= 131 then
										return LinFmod(1,449,450,130,131,L);
									elseif Lm <= 140 then
										return LinFmod(1,450,499,131,140,L);
									elseif Lm <= 141 then
										return LinFmod(1,499,500,140,141,L);
									else
										return LinFmod(1,500,549,141,150,L);
									end
								end
							else
								return 1;
							end
						elseif SN > "MANRDPSVONENAME" then
							if SN < "MARINERCDAGILITYTOPARRY" then
								if SN < "MANRDTRAITINHEALP" then
									if SN < "MANRDPSVTWOEVADE" then
										if SN > "MANRDPSVONEWILL" then
											if SN == "MANRDPSVTWOBLOCK" then
												return CalcStat("BalanceOfManBlock",L);
											else
												return 0;
											end
										elseif SN == "MANRDPSVONEWILL" then
											return CalcStat("ManFourthAgeWill",L);
										else
											return 0;
										end
									elseif SN > "MANRDPSVTWOEVADE" then
										if SN < "MANRDPSVTWOPARRY" then
											if SN == "MANRDPSVTWONAME" then
												return "Balance of Man";
											else
												return 0;
											end
										elseif SN > "MANRDPSVTWOPARRY" then
											if SN == "MANRDTRAITFATE" then
												return CalcStat("ManGiftOfMenFate",L);
											else
												return 0;
											end
										else
											return CalcStat("BalanceOfManParry",L);
										end
									else
										return CalcStat("BalanceOfManEvade",L);
									end
								elseif SN > "MANRDTRAITINHEALP" then
									if SN < "MANSTRONGMENMIGHT" then
										if SN > "MANRDTRAITMIGHT" then
											if SN == "MANRDTRAITWILL" then
												return CalcStat("ManDimMankindWill",L);
											else
												return 0;
											end
										elseif SN == "MANRDTRAITMIGHT" then
											return CalcStat("ManStrongMenMight",L);
										else
											return 0;
										end
									elseif SN > "MANSTRONGMENMIGHT" then
										if SN < "MARINERCDAGILITYTOEVADE" then
											if SN == "MARINERCDAGILITYTOCRITHIT" then
												return 4;
											else
												return 0;
											end
										elseif SN > "MARINERCDAGILITYTOEVADE" then
											if SN == "MARINERCDAGILITYTOFINESSE" then
												return 1;
											else
												return 0;
											end
										else
											return 4;
										end
									else
										return CalcStat("MightT",L,1.0);
									end
								else
									return CalcStat("ManEasilyInspInHealP",L);
								end
							elseif SN > "MARINERCDAGILITYTOPARRY" then
								if SN < "MARINERCDBASEFATE" then
									if SN < "MARINERCDARMOURTOPHYMIT" then
										if SN > "MARINERCDAGILITYTOPHYMAS" then
											if SN == "MARINERCDARMOURTOOFMIT" then
												return 0.2;
											else
												return 0;
											end
										elseif SN == "MARINERCDAGILITYTOPHYMAS" then
											return 3;
										else
											return 0;
										end
									elseif SN > "MARINERCDARMOURTOPHYMIT" then
										if SN < "MARINERCDARMOURTYPE" then
											if SN == "MARINERCDARMOURTOTACMIT" then
												return 0.2;
											else
												return 0;
											end
										elseif SN > "MARINERCDARMOURTYPE" then
											if SN == "MARINERCDBASEAGILITY" then
												return CalcStat("ClassBaseAgilityM",L);
											else
												return 0;
											end
										else
											return 2;
										end
									else
										return 1;
									end
								elseif SN > "MARINERCDBASEFATE" then
									if SN < "MARINERCDBASEMORALE" then
										if SN < "MARINERCDBASEICPR" then
											if SN == "MARINERCDBASEICMR" then
												return CalcStat("ClassBaseICMRL",L);
											else
												return 0;
											end
										elseif SN > "MARINERCDBASEICPR" then
											if SN == "MARINERCDBASEMIGHT" then
												return CalcStat("ClassBaseMightM",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseICPR",L);
										end
									elseif SN > "MARINERCDBASEMORALE" then
										if SN < "MARINERCDBASENCPR" then
											if SN == "MARINERCDBASENCMR" then
												return CalcStat("ClassBaseNCMRL",L);
											else
												return 0;
											end
										elseif SN > "MARINERCDBASENCPR" then
											if SN == "MARINERCDBASEPOWER" then
												return CalcStat("ClassBasePower",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseNCPR",L);
										end
									else
										return CalcStat("ClassBaseMorale",L);
									end
								else
									return CalcStat("ClassBaseFate",L);
								end
							else
								return 2;
							end
						else
							return "Man of the Fourth Age";
						end
					elseif SN > "MARINERCDBASEVITALITY" then
						if SN < "MINCALLEARENDILLIGHTMIT" then
							if SN < "MARINERCDVITALITYTONCMR" then
								if SN < "MARINERCDMIGHTTOFINESSE" then
									if SN < "MARINERCDFATETOPOWER" then
										if SN > "MARINERCDBASEWILL" then
											if SN == "MARINERCDFATETONCPR" then
												return 0.07;
											else
												return 0;
											end
										elseif SN == "MARINERCDBASEWILL" then
											return CalcStat("ClassBaseWillM",L);
										else
											return 0;
										end
									elseif SN > "MARINERCDFATETOPOWER" then
										if SN < "MARINERCDMIGHTTOCRITHIT" then
											if SN == "MARINERCDHASPOWER" then
												return 1;
											else
												return 0;
											end
										elseif SN > "MARINERCDMIGHTTOCRITHIT" then
											if SN == "MARINERCDMIGHTTOEVADE" then
												return 2;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 1;
									end
								elseif SN > "MARINERCDMIGHTTOFINESSE" then
									if SN < "MARINERCDPHYMITTOOFMIT" then
										if SN > "MARINERCDMIGHTTOPARRY" then
											if SN == "MARINERCDMIGHTTOPHYMAS" then
												return 6;
											else
												return 0;
											end
										elseif SN == "MARINERCDMIGHTTOPARRY" then
											return 4;
										else
											return 0;
										end
									elseif SN > "MARINERCDPHYMITTOOFMIT" then
										if SN < "MARINERCDVITALITYTOICMR" then
											if SN == "MARINERCDTACMASTOOUTHEAL" then
												return 1;
											else
												return 0;
											end
										elseif SN > "MARINERCDVITALITYTOICMR" then
											if SN == "MARINERCDVITALITYTOMORALE" then
												return 4;
											else
												return 0;
											end
										else
											return 0.012;
										end
									else
										return 1;
									end
								else
									return 1;
								end
							elseif SN > "MARINERCDVITALITYTONCMR" then
								if SN < "MARINERCDWILLTOTACMIT" then
									if SN < "MARINERCDWILLTOPARRY" then
										if SN > "MARINERCDWILLTOFINESSE" then
											if SN == "MARINERCDWILLTOOUTHEAL" then
												return 2;
											else
												return 0;
											end
										elseif SN == "MARINERCDWILLTOFINESSE" then
											return 2;
										else
											return 0;
										end
									elseif SN > "MARINERCDWILLTOPARRY" then
										if SN < "MARINERCDWILLTOPHYMIT" then
											if SN == "MARINERCDWILLTOPHYMAS" then
												return 2;
											else
												return 0;
											end
										elseif SN > "MARINERCDWILLTOPHYMIT" then
											if SN == "MARINERCDWILLTORESIST" then
												return 3;
											else
												return 0;
											end
										else
											return 2;
										end
									else
										return 1;
									end
								elseif SN > "MARINERCDWILLTOTACMIT" then
									if SN < "MASTERYT" then
										if SN > "MASTERY" then
											if SN == "MASTERYOLD" then
												return CalcStat("Mastery",L,N);
											else
												return 0;
											end
										elseif SN == "MASTERY" then
											return StatLinInter("PntMPMastery","ItemProg","ProgBMastery","",L,N,0);
										else
											return 0;
										end
									elseif SN > "MASTERYT" then
										if SN < "MIGHT" then
											if SN == "MATHOMLVLTOILVL" then
												return CalcStat("AwardLvlToILvl",L);
											else
												return 0;
											end
										elseif SN > "MIGHT" then
											if SN == "MIGHTT" then
												return CalcStat("MainT",L,N);
											else
												return 0;
											end
										else
											return CalcStat("Main",L,N);
										end
									else
										return StatLinInter("PntMPMastery","TraitProg","ProgBMastery","",L,N,0);
									end
								else
									return 2;
								end
							else
								return 0.12;
							end
						elseif SN > "MINCALLEARENDILLIGHTMIT" then
							if SN < "MINSTRELCDARMOURTOPHYMIT" then
								if SN < "MINECHOESBATTLERESIST" then
									if SN < "MINCOMPOSURERESIST" then
										if SN > "MINCALLOROMELIGHTMIT" then
											if SN == "MINCALLOROMEMELLIGHTMIT" then
												return -CalcStat("LightMitT",L,1.4);
											else
												return 0;
											end
										elseif SN == "MINCALLOROMELIGHTMIT" then
											return -CalcStat("LightMitT",L,1.2);
										else
											return 0;
										end
									elseif SN > "MINCOMPOSURERESIST" then
										if SN < "MINCOURAGERESIST" then
											if SN == "MINCOMPOSURETACMIT" then
												return CalcStat("TacMitT",L,1.0);
											else
												return 0;
											end
										elseif SN > "MINCOURAGERESIST" then
											if SN == "MINECHOESBATTLECRITDEF" then
												return -CalcStat("CritDefT",L,2.0);
											else
												return 0;
											end
										else
											return CalcStat("FearResistT",L,1.0);
										end
									else
										return CalcStat("ResistT",L,1.6);
									end
								elseif SN > "MINECHOESBATTLERESIST" then
									if SN < "MINSTRELCDAGILITYTOCRITHIT" then
										if SN > "MINENDMORALE" then
											if SN == "MINPIERCINGBALFINESSE" then
												return CalcStat("FinesseT",L,CalcStat("Trait12345Choice",N)*0.4);
											else
												return 0;
											end
										elseif SN == "MINENDMORALE" then
											return CalcStat("MoraleT",L,CalcStat("Trait12345Choice",N)*0.8);
										else
											return 0;
										end
									elseif SN > "MINSTRELCDAGILITYTOCRITHIT" then
										if SN < "MINSTRELCDAGILITYTOPARRY" then
											if SN == "MINSTRELCDAGILITYTOEVADE" then
												return 3;
											else
												return 0;
											end
										elseif SN > "MINSTRELCDAGILITYTOPARRY" then
											if SN == "MINSTRELCDARMOURTOOFMIT" then
												return 0.2;
											else
												return 0;
											end
										else
											return 2;
										end
									else
										return 1;
									end
								else
									return -CalcStat("SongResistT",L,1.0);
								end
							elseif SN > "MINSTRELCDARMOURTOPHYMIT" then
								if SN < "MINSTRELCDBASEMIGHT" then
									if SN < "MINSTRELCDBASEAGILITY" then
										if SN > "MINSTRELCDARMOURTOTACMIT" then
											if SN == "MINSTRELCDARMOURTYPE" then
												return 1;
											else
												return 0;
											end
										elseif SN == "MINSTRELCDARMOURTOTACMIT" then
											return 0.2;
										else
											return 0;
										end
									elseif SN > "MINSTRELCDBASEAGILITY" then
										if SN < "MINSTRELCDBASEICMR" then
											if SN == "MINSTRELCDBASEFATE" then
												return CalcStat("ClassBaseFate",L);
											else
												return 0;
											end
										elseif SN > "MINSTRELCDBASEICMR" then
											if SN == "MINSTRELCDBASEICPR" then
												return CalcStat("ClassBaseICPR",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseICMRL",L);
										end
									else
										return CalcStat("ClassBaseAgilityM",L);
									end
								elseif SN > "MINSTRELCDBASEMIGHT" then
									if SN < "MINSTRELCDBASEPOWER" then
										if SN < "MINSTRELCDBASENCMR" then
											if SN == "MINSTRELCDBASEMORALE" then
												return CalcStat("ClassBaseMorale",L);
											else
												return 0;
											end
										elseif SN > "MINSTRELCDBASENCMR" then
											if SN == "MINSTRELCDBASENCPR" then
												return CalcStat("ClassBaseNCPR",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseNCMRL",L);
										end
									elseif SN > "MINSTRELCDBASEPOWER" then
										if SN < "MINSTRELCDBASEWILL" then
											if SN == "MINSTRELCDBASEVITALITY" then
												return CalcStat("ClassBaseVitality",L);
											else
												return 0;
											end
										elseif SN > "MINSTRELCDBASEWILL" then
											if SN == "MINSTRELCDCANBLOCK" then
												if Lm <= 19 then
													return 0;
												else
													return 1;
												end
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseWillH",L);
										end
									else
										return CalcStat("ClassBasePower",L);
									end
								else
									return CalcStat("ClassBaseMightL",L);
								end
							else
								return 1;
							end
						else
							return -CalcStat("LightMitT",L,1.2);
						end
					else
						return CalcStat("ClassBaseVitality",L);
					end
				else
					return 0.5;
				end
			else
				return CalcStat("ClassBaseAgilityH",L);
			end
		else
			if Lm <= 0 then
				return 0;
			elseif Lm <= 5 then
				return LinFmod(1,0.02,0.1,1,5,L);
			elseif Lm <= 10 then
				return LinFmod(1,0.1,0.15,5,10,L);
			elseif Lm <= 15 then
				return LinFmod(1,0.15,0.2,10,15,L);
			else
				return CalcStat("CreepBatPromOutHealP",15);
			end
		end
	elseif SN > "MINSTRELCDFATETOCRITHIT" then
		if SN < "TPENBPE" then
			if SN < "PNTMPMAIN" then
				if SN < "PARTBPEPRATP" then
					if SN < "MITMEDIUMPRATPCAPR" then
						if SN < "MINTOTRESIST" then
							if SN < "MINSTRELCDVITALITYTOTACMIT" then
								if SN < "MINSTRELCDMIGHTTOBLOCK" then
									if SN < "MINSTRELCDFATETOPOWER" then
										if SN > "MINSTRELCDFATETOICMR" then
											if SN == "MINSTRELCDFATETONCPR" then
												return 0.05;
											else
												return 0;
											end
										elseif SN == "MINSTRELCDFATETOICMR" then
											return 0.025;
										else
											return 0;
										end
									elseif SN > "MINSTRELCDFATETOPOWER" then
										if SN < "MINSTRELCDFATETOTACMIT" then
											if SN == "MINSTRELCDFATETORESIST" then
												return 0.5;
											else
												return 0;
											end
										elseif SN > "MINSTRELCDFATETOTACMIT" then
											if SN == "MINSTRELCDHASPOWER" then
												return 1;
											else
												return 0;
											end
										else
											return 0.5;
										end
									else
										return 1;
									end
								elseif SN > "MINSTRELCDMIGHTTOBLOCK" then
									if SN < "MINSTRELCDTACMASTOOUTHEAL" then
										if SN > "MINSTRELCDMIGHTTOPARRY" then
											if SN == "MINSTRELCDPHYMITTOOFMIT" then
												return 1;
											else
												return 0;
											end
										elseif SN == "MINSTRELCDMIGHTTOPARRY" then
											return 2;
										else
											return 0;
										end
									elseif SN > "MINSTRELCDTACMASTOOUTHEAL" then
										if SN < "MINSTRELCDVITALITYTONCMR" then
											if SN == "MINSTRELCDVITALITYTOMORALE" then
												return 4;
											else
												return 0;
											end
										elseif SN > "MINSTRELCDVITALITYTONCMR" then
											if SN == "MINSTRELCDVITALITYTORESIST" then
												return 0.5;
											else
												return 0;
											end
										else
											return 0.12;
										end
									else
										return 1;
									end
								else
									return 3;
								end
							elseif SN > "MINSTRELCDVITALITYTOTACMIT" then
								if SN < "MINTIMEECHOESBATTLERESIST" then
									if SN < "MINSTRELCDWILLTORESIST" then
										if SN > "MINSTRELCDWILLTOEVADE" then
											if SN == "MINSTRELCDWILLTOFINESSE" then
												return 1;
											else
												return 0;
											end
										elseif SN == "MINSTRELCDWILLTOEVADE" then
											return 1;
										else
											return 0;
										end
									elseif SN > "MINSTRELCDWILLTORESIST" then
										if SN < "MINSTRELCDWILLTOTACMIT" then
											if SN == "MINSTRELCDWILLTOTACMAS" then
												return 3;
											else
												return 0;
											end
										elseif SN > "MINSTRELCDWILLTOTACMIT" then
											if SN == "MINTACMAS" then
												return CalcStat("TacMasT",L,CalcStat("Trait12345Choice",N)*0.4);
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 2;
									end
								elseif SN > "MINTIMEECHOESBATTLERESIST" then
									if SN < "MINTOTFATE" then
										if SN > "MINTOTCRITHIT" then
											if SN == "MINTOTCRITHITSEL" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({0,0,0,0.4,0.6,0},L);
												end
											else
												return 0;
											end
										elseif SN == "MINTOTCRITHIT" then
											return CalcStat("CritHitT",L,CalcStat("MinToTCritHitSel",N));
										else
											return 0;
										end
									elseif SN > "MINTOTFATE" then
										if SN < "MINTOTFINESSE" then
											if SN == "MINTOTFATESEL" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({0.2,0.3,0.4,0.5,0.6,0},L);
												end
											else
												return 0;
											end
										elseif SN > "MINTOTFINESSE" then
											if SN == "MINTOTFINESSESEL" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({0,0,0.2,0.4,0.6,0},L);
												end
											else
												return 0;
											end
										else
											return CalcStat("FinesseT",L,CalcStat("MinToTFinesseSel",N));
										end
									else
										return CalcStat("FateT",L,CalcStat("MinToTFateSel",N));
									end
								else
									return -CalcStat("SongResistT",L,0.6);
								end
							else
								return 0.5;
							end
						elseif SN > "MINTOTRESIST" then
							if SN < "MITLIGHTPPRAT" then
								if SN < "MITHEAVYPRATPA" then
									if SN < "MINTOTVITALITYSEL" then
										if SN > "MINTOTRESISTSEL" then
											if SN == "MINTOTVITALITY" then
												return CalcStat("VitalityT",L,CalcStat("MinToTVitalitySel",N));
											else
												return 0;
											end
										elseif SN == "MINTOTRESISTSEL" then
											if Lm <= 0 then
												return 0;
											else
												return DataTableValue({0,0.2,0.3,0.4,0.5,0},L);
											end
										else
											return 0;
										end
									elseif SN > "MINTOTVITALITYSEL" then
										if SN < "MITHEAVYPPRAT" then
											if SN == "MITHEAVYPBONUS" then
												return 0;
											else
												return 0;
											end
										elseif SN > "MITHEAVYPPRAT" then
											if SN == "MITHEAVYPRATP" then
												return CalcPercAB(CalcStat("MitHeavyPRatPA",L),CalcStat("MitHeavyPRatPB",L),CalcStat("MitHeavyPRatPCap",L),N);
											else
												return 0;
											end
										else
											return CalcRatAB(CalcStat("MitHeavyPRatPA",L),CalcStat("MitHeavyPRatPB",L),CalcStat("MitHeavyPRatPCapR",L),N);
										end
									else
										if Lm <= 0 then
											return 0;
										else
											return DataTableValue({0,0,0,0,0.4,0},L);
										end
									end
								elseif SN > "MITHEAVYPRATPA" then
									if SN < "MITHEAVYPRATPCAP" then
										if SN > "MITHEAVYPRATPB" then
											if SN == "MITHEAVYPRATPC" then
												return 0.5;
											else
												return 0;
											end
										elseif SN == "MITHEAVYPRATPB" then
											return CalcStat("BRatRounded",L,"BRatMitHeavy");
										else
											return 0;
										end
									elseif SN > "MITHEAVYPRATPCAP" then
										if SN < "MITIGATION" then
											if SN == "MITHEAVYPRATPCAPR" then
												return CalcStat("MitHeavyPRatPB",L)*CalcStat("MitHeavyPRatPC",L);
											else
												return 0;
											end
										elseif SN > "MITIGATION" then
											if SN == "MITLIGHTPBONUS" then
												return 0;
											else
												return 0;
											end
										else
											return StatLinInter("PntMPMitigation","ItemProg","ProgBMitigation","",L,N,0);
										end
									else
										return 60;
									end
								else
									return 180;
								end
							elseif SN > "MITLIGHTPPRAT" then
								if SN < "MITMEDIUMPBONUS" then
									if SN < "MITLIGHTPRATPB" then
										if SN > "MITLIGHTPRATP" then
											if SN == "MITLIGHTPRATPA" then
												return 120;
											else
												return 0;
											end
										elseif SN == "MITLIGHTPRATP" then
											return CalcPercAB(CalcStat("MitLightPRatPA",L),CalcStat("MitLightPRatPB",L),CalcStat("MitLightPRatPCap",L),N);
										else
											return 0;
										end
									elseif SN > "MITLIGHTPRATPB" then
										if SN < "MITLIGHTPRATPCAP" then
											if SN == "MITLIGHTPRATPC" then
												return 0.5;
											else
												return 0;
											end
										elseif SN > "MITLIGHTPRATPCAP" then
											if SN == "MITLIGHTPRATPCAPR" then
												return CalcStat("MitLightPRatPB",L)*CalcStat("MitLightPRatPC",L);
											else
												return 0;
											end
										else
											return 40;
										end
									else
										return CalcStat("BRatRounded",L,"BRatMitLight");
									end
								elseif SN > "MITMEDIUMPBONUS" then
									if SN < "MITMEDIUMPRATPA" then
										if SN > "MITMEDIUMPPRAT" then
											if SN == "MITMEDIUMPRATP" then
												return CalcPercAB(CalcStat("MitMediumPRatPA",L),CalcStat("MitMediumPRatPB",L),CalcStat("MitMediumPRatPCap",L),N);
											else
												return 0;
											end
										elseif SN == "MITMEDIUMPPRAT" then
											return CalcRatAB(CalcStat("MitMediumPRatPA",L),CalcStat("MitMediumPRatPB",L),CalcStat("MitMediumPRatPCapR",L),N);
										else
											return 0;
										end
									elseif SN > "MITMEDIUMPRATPA" then
										if SN < "MITMEDIUMPRATPC" then
											if SN == "MITMEDIUMPRATPB" then
												return CalcStat("BRatRounded",L,"BRatMitMedium");
											else
												return 0;
											end
										elseif SN > "MITMEDIUMPRATPC" then
											if SN == "MITMEDIUMPRATPCAP" then
												return 50;
											else
												return 0;
											end
										else
											return 0.5;
										end
									else
										return 150;
									end
								else
									return 0;
								end
							else
								return CalcRatAB(CalcStat("MitLightPRatPA",L),CalcStat("MitLightPRatPB",L),CalcStat("MitLightPRatPCapR",L),N);
							end
						else
							return CalcStat("ResistT",L,CalcStat("MinToTResistSel",N));
						end
					elseif SN > "MITMEDIUMPRATPCAPR" then
						if SN < "OUTHEALT" then
							if SN < "OUTDMGPRATPA" then
								if SN < "NCMRT" then
									if SN < "MORALET" then
										if SN > "MORALE" then
											if SN == "MORALEADJ" then
												if Lm <= 25 then
													return 0.5;
												elseif Lm <= 50 then
													return 0.6;
												elseif Lm <= 60 then
													return 0.7;
												elseif Lm <= 79 then
													return 0.8;
												elseif Lm <= 80 then
													return 0.9;
												else
													return 1;
												end
											else
												return 0;
											end
										elseif SN == "MORALE" then
											return StatLinInter("PntMPMorale","ItemProgVital","ProgBMorale","MoraleAdj",L,N,0);
										else
											return 0;
										end
									elseif SN > "MORALET" then
										if SN < "N" then
											if SN == "MORALETADJ" then
												if Lm <= 25 then
													return 0.5;
												elseif Lm <= 50 then
													return 0.6;
												elseif Lm <= 60 then
													return 0.7;
												elseif Lm <= 65 then
													return 0.8;
												elseif Lm <= 75 then
													return 0.9;
												else
													return 1;
												end
											else
												return 0;
											end
										elseif SN > "N" then
											if SN == "NCMR" then
												return StatLinInter("PntMPNCMR","ItemProgVital","ProgBNCMR","",L,N,1);
											else
												return 0;
											end
										else
											return N;
										end
									else
										return StatLinInter("PntMPMorale","TraitProgVital","ProgBMorale","MoraleTAdj",L,N,0);
									end
								elseif SN > "NCMRT" then
									if SN < "OFFSET" then
										if SN > "NCPR" then
											if SN == "NCPRT" then
												return StatLinInter("PntMPNCPR","TraitProgVital","ProgBNCPR","",L,N,99);
											else
												return 0;
											end
										elseif SN == "NCPR" then
											return StatLinInter("PntMPNCPR","ItemProgClassic","ProgBNCPR","",L,N,99);
										else
											return 0;
										end
									elseif SN > "OFFSET" then
										if SN < "OUTDMGPPRAT" then
											if SN == "OUTDMGPBONUS" then
												return 0;
											else
												return 0;
											end
										elseif SN > "OUTDMGPPRAT" then
											if SN == "OUTDMGPRATP" then
												return CalcPercAB(CalcStat("OutDmgPRatPA",L),CalcStat("OutDmgPRatPB",L),CalcStat("OutDmgPRatPCap",L),N);
											else
												return 0;
											end
										else
											return CalcRatAB(CalcStat("OutDmgPRatPA",L),CalcStat("OutDmgPRatPB",L),CalcStat("OutDmgPRatPCapR",L),N);
										end
									else
										return L+N;
									end
								else
									return StatLinInter("PntMPNCMR","TraitProgVital","ProgBNCMR","",L,N,1);
								end
							elseif SN > "OUTDMGPRATPA" then
								if SN < "OUTHEALPPRAT" then
									if SN < "OUTDMGPRATPCAP" then
										if SN > "OUTDMGPRATPB" then
											if SN == "OUTDMGPRATPC" then
												return 0.5;
											else
												return 0;
											end
										elseif SN == "OUTDMGPRATPB" then
											return CalcStat("BRatRounded",L,"BRatExtra");
										else
											return 0;
										end
									elseif SN > "OUTDMGPRATPCAP" then
										if SN < "OUTHEAL" then
											if SN == "OUTDMGPRATPCAPR" then
												return CalcStat("OutDmgPRatPB",L)*CalcStat("OutDmgPRatPC",L);
											else
												return 0;
											end
										elseif SN > "OUTHEAL" then
											if SN == "OUTHEALPBONUS" then
												return 0;
											else
												return 0;
											end
										else
											return StatLinInter("PntMPOutHeal","ItemProg","ProgBOutHeal","",L,N,0);
										end
									else
										return 200;
									end
								elseif SN > "OUTHEALPPRAT" then
									if SN < "OUTHEALPRATPB" then
										if SN > "OUTHEALPRATP" then
											if SN == "OUTHEALPRATPA" then
												return 210;
											else
												return 0;
											end
										elseif SN == "OUTHEALPRATP" then
											return CalcPercAB(CalcStat("OutHealPRatPA",L),CalcStat("OutHealPRatPB",L),CalcStat("OutHealPRatPCap",L),N);
										else
											return 0;
										end
									elseif SN > "OUTHEALPRATPB" then
										if SN < "OUTHEALPRATPCAP" then
											if SN == "OUTHEALPRATPC" then
												return 0.5;
											else
												return 0;
											end
										elseif SN > "OUTHEALPRATPCAP" then
											if SN == "OUTHEALPRATPCAPR" then
												return CalcStat("OutHealPRatPB",L)*CalcStat("OutHealPRatPC",L);
											else
												return 0;
											end
										else
											return 70;
										end
									else
										return CalcStat("BRatRounded",L,"BRatOutHeal");
									end
								else
									return CalcRatAB(CalcStat("OutHealPRatPA",L),CalcStat("OutHealPRatPB",L),CalcStat("OutHealPRatPCapR",L),N);
								end
							else
								return 600;
							end
						elseif SN > "OUTHEALT" then
							if SN < "PARTBLOCKMITPRATPA" then
								if SN < "PARRYPRATPC" then
									if SN < "PARRYPPRAT" then
										if SN > "PARRY" then
											if SN == "PARRYPBONUS" then
												return CalcStat("BPEPBonus",L);
											else
												return 0;
											end
										elseif SN == "PARRY" then
											return CalcStat("BPE",L,N);
										else
											return 0;
										end
									elseif SN > "PARRYPPRAT" then
										if SN < "PARRYPRATPA" then
											if SN == "PARRYPRATP" then
												return CalcStat("BPEPRatP",L,N);
											else
												return 0;
											end
										elseif SN > "PARRYPRATPA" then
											if SN == "PARRYPRATPB" then
												return CalcStat("BPEPRatPB",L);
											else
												return 0;
											end
										else
											return CalcStat("BPEPRatPA",L);
										end
									else
										return CalcStat("BPEPPRat",L,N);
									end
								elseif SN > "PARRYPRATPC" then
									if SN < "PARRYT" then
										if SN > "PARRYPRATPCAP" then
											if SN == "PARRYPRATPCAPR" then
												return CalcStat("BPEPRatPCapR",L);
											else
												return 0;
											end
										elseif SN == "PARRYPRATPCAP" then
											return CalcStat("BPEPRatPCap",L);
										else
											return 0;
										end
									elseif SN > "PARRYT" then
										if SN < "PARTBLOCKMITPPRAT" then
											if SN == "PARTBLOCKMITPBONUS" then
												return CalcStat("PartMitPBonus",L);
											else
												return 0;
											end
										elseif SN > "PARTBLOCKMITPPRAT" then
											if SN == "PARTBLOCKMITPRATP" then
												return CalcStat("PartMitPRatP",L,N);
											else
												return 0;
											end
										else
											return CalcStat("PartMitPPRat",L,N);
										end
									else
										return CalcStat("BPET",L,N);
									end
								else
									return CalcStat("BPEPRatPC",L);
								end
							elseif SN > "PARTBLOCKMITPRATPA" then
								if SN < "PARTBLOCKPRATP" then
									if SN < "PARTBLOCKMITPRATPCAP" then
										if SN > "PARTBLOCKMITPRATPB" then
											if SN == "PARTBLOCKMITPRATPC" then
												return CalcStat("PartMitPRatPC",L);
											else
												return 0;
											end
										elseif SN == "PARTBLOCKMITPRATPB" then
											return CalcStat("PartMitPRatPB",L);
										else
											return 0;
										end
									elseif SN > "PARTBLOCKMITPRATPCAP" then
										if SN < "PARTBLOCKPBONUS" then
											if SN == "PARTBLOCKMITPRATPCAPR" then
												return CalcStat("PartMitPRatPCapR",L);
											else
												return 0;
											end
										elseif SN > "PARTBLOCKPBONUS" then
											if SN == "PARTBLOCKPPRAT" then
												return CalcStat("PartBPEPPRat",L,N);
											else
												return 0;
											end
										else
											return CalcStat("PartBPEPBonus",L);
										end
									else
										return CalcStat("PartMitPRatPCap",L);
									end
								elseif SN > "PARTBLOCKPRATP" then
									if SN < "PARTBLOCKPRATPCAP" then
										if SN < "PARTBLOCKPRATPB" then
											if SN == "PARTBLOCKPRATPA" then
												return CalcStat("PartBPEPRatPA",L);
											else
												return 0;
											end
										elseif SN > "PARTBLOCKPRATPB" then
											if SN == "PARTBLOCKPRATPC" then
												return CalcStat("PartBPEPRatPC",L);
											else
												return 0;
											end
										else
											return CalcStat("PartBPEPRatPB",L);
										end
									elseif SN > "PARTBLOCKPRATPCAP" then
										if SN < "PARTBPEPBONUS" then
											if SN == "PARTBLOCKPRATPCAPR" then
												return CalcStat("PartBPEPRatPCapR",L);
											else
												return 0;
											end
										elseif SN > "PARTBPEPBONUS" then
											if SN == "PARTBPEPPRAT" then
												return CalcRatAB(CalcStat("PartBPEPRatPA",L),CalcStat("PartBPEPRatPB",L),CalcStat("PartBPEPRatPCapR",L),N);
											else
												return 0;
											end
										else
											return 0;
										end
									else
										return CalcStat("PartBPEPRatPCap",L);
									end
								else
									return CalcStat("PartBPEPRatP",L,N);
								end
							else
								return CalcStat("PartMitPRatPA",L);
							end
						else
							return StatLinInter("PntMPOutHeal","TraitProg","ProgBOutHeal","",L,N,0);
						end
					else
						return CalcStat("MitMediumPRatPB",L)*CalcStat("MitMediumPRatPC",L);
					end
				elseif SN > "PARTBPEPRATP" then
					if SN < "PHYDMGPRATPB" then
						if SN < "PARTMITPRATPCAP" then
							if SN < "PARTEVADEPBONUS" then
								if SN < "PARTEVADEMITPPRAT" then
									if SN < "PARTBPEPRATPC" then
										if SN > "PARTBPEPRATPA" then
											if SN == "PARTBPEPRATPB" then
												return CalcStat("BRatRounded",L,"BRatPartBPE");
											else
												return 0;
											end
										elseif SN == "PARTBPEPRATPA" then
											return 75;
										else
											return 0;
										end
									elseif SN > "PARTBPEPRATPC" then
										if SN < "PARTBPEPRATPCAPR" then
											if SN == "PARTBPEPRATPCAP" then
												return 25;
											else
												return 0;
											end
										elseif SN > "PARTBPEPRATPCAPR" then
											if SN == "PARTEVADEMITPBONUS" then
												return CalcStat("PartMitPBonus",L);
											else
												return 0;
											end
										else
											return CalcStat("PartBPEPRatPB",L)*CalcStat("PartBPEPRatPC",L);
										end
									else
										return 0.5;
									end
								elseif SN > "PARTEVADEMITPPRAT" then
									if SN < "PARTEVADEMITPRATPB" then
										if SN > "PARTEVADEMITPRATP" then
											if SN == "PARTEVADEMITPRATPA" then
												return CalcStat("PartMitPRatPA",L);
											else
												return 0;
											end
										elseif SN == "PARTEVADEMITPRATP" then
											return CalcStat("PartMitPRatP",L,N);
										else
											return 0;
										end
									elseif SN > "PARTEVADEMITPRATPB" then
										if SN < "PARTEVADEMITPRATPCAP" then
											if SN == "PARTEVADEMITPRATPC" then
												return CalcStat("PartMitPRatPC",L);
											else
												return 0;
											end
										elseif SN > "PARTEVADEMITPRATPCAP" then
											if SN == "PARTEVADEMITPRATPCAPR" then
												return CalcStat("PartMitPRatPCapR",L);
											else
												return 0;
											end
										else
											return CalcStat("PartMitPRatPCap",L);
										end
									else
										return CalcStat("PartMitPRatPB",L);
									end
								else
									return CalcStat("PartMitPPRat",L,N);
								end
							elseif SN > "PARTEVADEPBONUS" then
								if SN < "PARTEVADEPRATPCAPR" then
									if SN < "PARTEVADEPRATPA" then
										if SN > "PARTEVADEPPRAT" then
											if SN == "PARTEVADEPRATP" then
												return CalcStat("PartBPEPRatP",L,N);
											else
												return 0;
											end
										elseif SN == "PARTEVADEPPRAT" then
											return CalcStat("PartBPEPPRat",L,N);
										else
											return 0;
										end
									elseif SN > "PARTEVADEPRATPA" then
										if SN < "PARTEVADEPRATPC" then
											if SN == "PARTEVADEPRATPB" then
												return CalcStat("PartBPEPRatPB",L);
											else
												return 0;
											end
										elseif SN > "PARTEVADEPRATPC" then
											if SN == "PARTEVADEPRATPCAP" then
												return CalcStat("PartBPEPRatPCap",L);
											else
												return 0;
											end
										else
											return CalcStat("PartBPEPRatPC",L);
										end
									else
										return CalcStat("PartBPEPRatPA",L);
									end
								elseif SN > "PARTEVADEPRATPCAPR" then
									if SN < "PARTMITPRATP" then
										if SN > "PARTMITPBONUS" then
											if SN == "PARTMITPPRAT" then
												return CalcRatAB(CalcStat("PartMitPRatPA",L),CalcStat("PartMitPRatPB",L),CalcStat("PartMitPRatPCapR",L),N);
											else
												return 0;
											end
										elseif SN == "PARTMITPBONUS" then
											return 0.1;
										else
											return 0;
										end
									elseif SN > "PARTMITPRATP" then
										if SN < "PARTMITPRATPB" then
											if SN == "PARTMITPRATPA" then
												return 105;
											else
												return 0;
											end
										elseif SN > "PARTMITPRATPB" then
											if SN == "PARTMITPRATPC" then
												return 0.5;
											else
												return 0;
											end
										else
											return CalcStat("BRatRounded",L,"BRatPartBPE");
										end
									else
										return CalcPercAB(CalcStat("PartMitPRatPA",L),CalcStat("PartMitPRatPB",L),CalcStat("PartMitPRatPCap",L),N);
									end
								else
									return CalcStat("PartBPEPRatPCapR",L);
								end
							else
								return CalcStat("PartBPEPBonus",L);
							end
						elseif SN > "PARTMITPRATPCAP" then
							if SN < "PARTPARRYPRATPB" then
								if SN < "PARTPARRYMITPRATPC" then
									if SN < "PARTPARRYMITPPRAT" then
										if SN > "PARTMITPRATPCAPR" then
											if SN == "PARTPARRYMITPBONUS" then
												return CalcStat("PartMitPBonus",L);
											else
												return 0;
											end
										elseif SN == "PARTMITPRATPCAPR" then
											return CalcStat("PartMitPRatPB",L)*CalcStat("PartMitPRatPC",L);
										else
											return 0;
										end
									elseif SN > "PARTPARRYMITPPRAT" then
										if SN < "PARTPARRYMITPRATPA" then
											if SN == "PARTPARRYMITPRATP" then
												return CalcStat("PartMitPRatP",L,N);
											else
												return 0;
											end
										elseif SN > "PARTPARRYMITPRATPA" then
											if SN == "PARTPARRYMITPRATPB" then
												return CalcStat("PartMitPRatPB",L);
											else
												return 0;
											end
										else
											return CalcStat("PartMitPRatPA",L);
										end
									else
										return CalcStat("PartMitPPRat",L,N);
									end
								elseif SN > "PARTPARRYMITPRATPC" then
									if SN < "PARTPARRYPBONUS" then
										if SN > "PARTPARRYMITPRATPCAP" then
											if SN == "PARTPARRYMITPRATPCAPR" then
												return CalcStat("PartMitPRatPCapR",L);
											else
												return 0;
											end
										elseif SN == "PARTPARRYMITPRATPCAP" then
											return CalcStat("PartMitPRatPCap",L);
										else
											return 0;
										end
									elseif SN > "PARTPARRYPBONUS" then
										if SN < "PARTPARRYPRATP" then
											if SN == "PARTPARRYPPRAT" then
												return CalcStat("PartBPEPPRat",L,N);
											else
												return 0;
											end
										elseif SN > "PARTPARRYPRATP" then
											if SN == "PARTPARRYPRATPA" then
												return CalcStat("PartBPEPRatPA",L);
											else
												return 0;
											end
										else
											return CalcStat("PartBPEPRatP",L,N);
										end
									else
										return CalcStat("PartBPEPBonus",L);
									end
								else
									return CalcStat("PartMitPRatPC",L);
								end
							elseif SN > "PARTPARRYPRATPB" then
								if SN < "PERKPHYMIT" then
									if SN < "PARTPARRYPRATPCAPR" then
										if SN > "PARTPARRYPRATPC" then
											if SN == "PARTPARRYPRATPCAP" then
												return CalcStat("PartBPEPRatPCap",L);
											else
												return 0;
											end
										elseif SN == "PARTPARRYPRATPC" then
											return CalcStat("PartBPEPRatPC",L);
										else
											return 0;
										end
									elseif SN > "PARTPARRYPRATPCAPR" then
										if SN < "PERKNCMR" then
											if SN == "PERKMORALE" then
												return DataTableValue({10,20,30,40},L);
											else
												return 0;
											end
										elseif SN > "PERKNCMR" then
											if SN == "PERKNCPR" then
												return CalcStat("FoodNCPRL",L);
											else
												return 0;
											end
										else
											return CalcStat("FoodNCMRL",L);
										end
									else
										return CalcStat("PartBPEPRatPCapR",L);
									end
								elseif SN > "PERKPHYMIT" then
									if SN < "PHYDMGPBONUS" then
										if SN > "PERKPOWER" then
											if SN == "PERKTACMIT" then
												return CalcStat("TacMitT",L,0.2*N);
											else
												return 0;
											end
										elseif SN == "PERKPOWER" then
											return DataTableValue({10,20,30,40},L);
										else
											return 0;
										end
									elseif SN > "PHYDMGPBONUS" then
										if SN < "PHYDMGPRATP" then
											if SN == "PHYDMGPPRAT" then
												return CalcStat("OutDmgPPRat",L,N);
											else
												return 0;
											end
										elseif SN > "PHYDMGPRATP" then
											if SN == "PHYDMGPRATPA" then
												return CalcStat("OutDmgPRatPA",L);
											else
												return 0;
											end
										else
											return CalcStat("OutDmgPRatP",L,N);
										end
									else
										return CalcStat("OutDmgPBonus",L);
									end
								else
									return CalcStat("PhyMitT",L,0.2*N);
								end
							else
								return CalcStat("PartBPEPRatPB",L);
							end
						else
							return 35;
						end
					elseif SN > "PHYDMGPRATPB" then
						if SN < "PHYMITMPRATPB" then
							if SN < "PHYMITHPRATPCAP" then
								if SN < "PHYMIT" then
									if SN < "PHYDMGPRATPCAPR" then
										if SN > "PHYDMGPRATPC" then
											if SN == "PHYDMGPRATPCAP" then
												return CalcStat("OutDmgPRatPCap",L);
											else
												return 0;
											end
										elseif SN == "PHYDMGPRATPC" then
											return CalcStat("OutDmgPRatPC",L);
										else
											return 0;
										end
									elseif SN > "PHYDMGPRATPCAPR" then
										if SN < "PHYMASOLD" then
											if SN == "PHYMAS" then
												return CalcStat("Mastery",L,N);
											else
												return 0;
											end
										elseif SN > "PHYMASOLD" then
											if SN == "PHYMAST" then
												return CalcStat("MasteryT",L,N);
											else
												return 0;
											end
										else
											return CalcStat("Mastery",L,N);
										end
									else
										return CalcStat("OutDmgPRatPCapR",L);
									end
								elseif SN > "PHYMIT" then
									if SN < "PHYMITHPRATP" then
										if SN > "PHYMITHPBONUS" then
											if SN == "PHYMITHPPRAT" then
												return CalcStat("MitHeavyPPRat",L,N);
											else
												return 0;
											end
										elseif SN == "PHYMITHPBONUS" then
											return CalcStat("MitHeavyPBonus",L);
										else
											return 0;
										end
									elseif SN > "PHYMITHPRATP" then
										if SN < "PHYMITHPRATPB" then
											if SN == "PHYMITHPRATPA" then
												return CalcStat("MitHeavyPRatPA",L);
											else
												return 0;
											end
										elseif SN > "PHYMITHPRATPB" then
											if SN == "PHYMITHPRATPC" then
												return CalcStat("MitHeavyPRatPC",L);
											else
												return 0;
											end
										else
											return CalcStat("MitHeavyPRatPB",L);
										end
									else
										return CalcStat("MitHeavyPRatP",L,N);
									end
								else
									return StatLinInter("PntMPPhyMit","ItemProg","ProgBMitigation","",L,N,0);
								end
							elseif SN > "PHYMITHPRATPCAP" then
								if SN < "PHYMITLPRATPC" then
									if SN < "PHYMITLPPRAT" then
										if SN > "PHYMITHPRATPCAPR" then
											if SN == "PHYMITLPBONUS" then
												return CalcStat("MitLightPBonus",L);
											else
												return 0;
											end
										elseif SN == "PHYMITHPRATPCAPR" then
											return CalcStat("MitHeavyPRatPCapR",L);
										else
											return 0;
										end
									elseif SN > "PHYMITLPPRAT" then
										if SN < "PHYMITLPRATPA" then
											if SN == "PHYMITLPRATP" then
												return CalcStat("MitLightPRatP",L,N);
											else
												return 0;
											end
										elseif SN > "PHYMITLPRATPA" then
											if SN == "PHYMITLPRATPB" then
												return CalcStat("MitLightPRatPB",L);
											else
												return 0;
											end
										else
											return CalcStat("MitLightPRatPA",L);
										end
									else
										return CalcStat("MitLightPPRat",L,N);
									end
								elseif SN > "PHYMITLPRATPC" then
									if SN < "PHYMITMPBONUS" then
										if SN > "PHYMITLPRATPCAP" then
											if SN == "PHYMITLPRATPCAPR" then
												return CalcStat("MitLightPRatPCapR",L);
											else
												return 0;
											end
										elseif SN == "PHYMITLPRATPCAP" then
											return CalcStat("MitLightPRatPCap",L);
										else
											return 0;
										end
									elseif SN > "PHYMITMPBONUS" then
										if SN < "PHYMITMPRATP" then
											if SN == "PHYMITMPPRAT" then
												return CalcStat("MitMediumPPRat",L,N);
											else
												return 0;
											end
										elseif SN > "PHYMITMPRATP" then
											if SN == "PHYMITMPRATPA" then
												return CalcStat("MitMediumPRatPA",L);
											else
												return 0;
											end
										else
											return CalcStat("MitMediumPRatP",L,N);
										end
									else
										return CalcStat("MitMediumPBonus",L);
									end
								else
									return CalcStat("MitLightPRatPC",L);
								end
							else
								return CalcStat("MitHeavyPRatPCap",L);
							end
						elseif SN > "PHYMITMPRATPB" then
							if SN < "PNTMPARMOURVIRTUES" then
								if SN < "PLAYERBASEMORALE" then
									if SN < "PHYMITMPRATPCAPR" then
										if SN > "PHYMITMPRATPC" then
											if SN == "PHYMITMPRATPCAP" then
												return CalcStat("MitMediumPRatPCap",L);
											else
												return 0;
											end
										elseif SN == "PHYMITMPRATPC" then
											return CalcStat("MitMediumPRatPC",L);
										else
											return 0;
										end
									elseif SN > "PHYMITMPRATPCAPR" then
										if SN < "PHYRESISTT" then
											if SN == "PHYMITT" then
												return StatLinInter("PntMPPhyMit","TraitProg","ProgBMitigation","",L,N,0);
											else
												return 0;
											end
										elseif SN > "PHYRESISTT" then
											if SN == "PLAYERBASEEVADE" then
												return 1;
											else
												return 0;
											end
										else
											return CalcStat("ResistAddT",L,N);
										end
									else
										return CalcStat("MitMediumPRatPCapR",L);
									end
								elseif SN > "PLAYERBASEMORALE" then
									if SN < "PLAYERBASETACMAS" then
										if SN > "PLAYERBASEPARRY" then
											if SN == "PLAYERBASEPHYMAS" then
												return 11.5;
											else
												return 0;
											end
										elseif SN == "PLAYERBASEPARRY" then
											return 3;
										else
											return 0;
										end
									elseif SN > "PLAYERBASETACMAS" then
										if SN < "PNTMPARMOURPENT" then
											if SN == "PNTMPARMOUR" then
												return 4.4/1200;
											else
												return 0;
											end
										elseif SN > "PNTMPARMOURPENT" then
											if SN == "PNTMPARMOURT" then
												return 25/1200;
											else
												return 0;
											end
										else
											return 72/1200;
										end
									else
										return 11.5;
									end
								else
									return 52.5;
								end
							elseif SN > "PNTMPARMOURVIRTUES" then
								if SN < "PNTMPDMGTYPEMIT" then
									if SN < "PNTMPCLASSBASENCPR" then
										if SN > "PNTMPBPE" then
											if SN == "PNTMPCLASSBASEICPR" then
												return 0.125;
											else
												return 0;
											end
										elseif SN == "PNTMPBPE" then
											return 42/1200;
										else
											return 0;
										end
									elseif SN > "PNTMPCLASSBASENCPR" then
										if SN < "PNTMPCRITDEF" then
											if SN == "PNTMPCOMBATDAMAGEMOD" then
												return 300/1200;
											else
												return 0;
											end
										elseif SN > "PNTMPCRITDEF" then
											if SN == "PNTMPCRITHIT" then
												return 24/1200;
											else
												return 0;
											end
										else
											return 40/1200;
										end
									else
										return 0.25;
									end
								elseif SN > "PNTMPDMGTYPEMIT" then
									if SN < "PNTMPFOODRESIST" then
										if SN < "PNTMPFINESSE" then
											if SN == "PNTMPFATE" then
												return 1.25;
											else
												return 0;
											end
										elseif SN > "PNTMPFINESSE" then
											if SN == "PNTMPFINESSET" then
												return 36/1200;
											else
												return 0;
											end
										else
											return 40/1200;
										end
									elseif SN > "PNTMPFOODRESIST" then
										if SN < "PNTMPICPR" then
											if SN == "PNTMPICMR" then
												return 0.015;
											else
												return 0;
											end
										elseif SN > "PNTMPICPR" then
											if SN == "PNTMPINHEAL" then
												return 40/1200;
											else
												return 0;
											end
										else
											return 0.0625;
										end
									else
										return 30/1200;
									end
								else
									return 60/1200;
								end
							else
								return 24/1200;
							end
						else
							return CalcStat("MitMediumPRatPB",L);
						end
					else
						return CalcStat("OutDmgPRatPB",L);
					end
				else
					return CalcPercAB(CalcStat("PartBPEPRatPA",L),CalcStat("PartBPEPRatPB",L),CalcStat("PartBPEPRatPCap",L),N);
				end
			elseif SN > "PNTMPMAIN" then
				if SN < "RUNEKEEPERCDBASEPOWER" then
					if SN < "REPCRITDEF" then
						if SN < "PROGBENERGY" then
							if SN < "PNTMPTACMIT" then
								if SN < "PNTMPOUTHEAL" then
									if SN < "PNTMPMORALE" then
										if SN > "PNTMPMASTERY" then
											if SN == "PNTMPMITIGATION" then
												return 28/1200;
											else
												return 0;
											end
										elseif SN == "PNTMPMASTERY" then
											return 17/1200;
										else
											return 0;
										end
									elseif SN > "PNTMPMORALE" then
										if SN < "PNTMPNCMR" then
											if SN == "PNTMPMORALEVIRTUES" then
												return 0.75;
											else
												return 0;
											end
										elseif SN > "PNTMPNCMR" then
											if SN == "PNTMPNCPR" then
												return 0.5;
											else
												return 0;
											end
										else
											return 0.15;
										end
									else
										return 1;
									end
								elseif SN > "PNTMPOUTHEAL" then
									if SN < "PNTMPPOWERT" then
										if SN > "PNTMPPHYMIT" then
											if SN == "PNTMPPOWER" then
												return 1;
											else
												return 0;
											end
										elseif SN == "PNTMPPHYMIT" then
											return 36/1200;
										else
											return 0;
										end
									elseif SN > "PNTMPPOWERT" then
										if SN < "PNTMPSHIELDBLOCK" then
											if SN == "PNTMPRESIST" then
												return 36/1200;
											else
												return 0;
											end
										elseif SN > "PNTMPSHIELDBLOCK" then
											if SN == "PNTMPTACHPS" then
												return 0.0875;
											else
												return 0;
											end
										else
											return 80/1200;
										end
									else
										return 0.6666;
									end
								else
									return 30/1200;
								end
							elseif SN > "PNTMPTACMIT" then
								if SN < "PROGBARMOURHEAVY" then
									if SN < "POISONRESISTT" then
										if SN > "PNTMPVITALITY" then
											if SN == "PNTMPVITALITYT" then
												return 0.225;
											else
												return 0;
											end
										elseif SN == "PNTMPVITALITY" then
											return 0.175;
										else
											return 0;
										end
									elseif SN > "POISONRESISTT" then
										if SN < "POWERT" then
											if SN == "POWER" then
												return StatLinInter("PntMPPower","ItemProgVital","ProgBPower","",L,N,0);
											else
												return 0;
											end
										elseif SN > "POWERT" then
											if SN == "PROGBARMOUR" then
												return CalcStat("BRatProgB",L,"BRatMitMedium");
											else
												return 0;
											end
										else
											return StatLinInter("PntMPPowerT","TraitProgVital","ProgBPower","",L,N,0);
										end
									else
										return CalcStat("ResistAddT",L,N);
									end
								elseif SN > "PROGBARMOURHEAVY" then
									if SN < "PROGBBPE" then
										if SN > "PROGBARMOURLIGHT" then
											if SN == "PROGBARMOURMEDIUM" then
												return CalcStat("BRatProgB",L,"BRatMitMedium");
											else
												return 0;
											end
										elseif SN == "PROGBARMOURLIGHT" then
											return CalcStat("BRatProgB",L,"BRatMitLight");
										else
											return 0;
										end
									elseif SN > "PROGBBPE" then
										if SN < "PROGBCRITDEF" then
											if SN == "PROGBCOMBATDAMAGEMOD" then
												if Lm <= 25 then
													return LinFmod(1,5,45.58309,1,25,L);
												elseif Lm <= 50 then
													return LinFmod(1,45.58309,120,25,50,L);
												elseif Lm <= 60 then
													return LinFmod(1,120,175.56,50,60,L);
												elseif Lm <= 65 then
													return LinFmod(1,175.56,200.79,60,65,L);
												elseif Lm <= 75 then
													return LinFmod(1,200.79,292.5,65,75,L);
												elseif Lm <= 85 then
													return LinFmod(1,292.5,421.2,75,85,L);
												elseif Lm <= 95 then
													return LinFmod(1,421.2,600.3,85,95,L);
												elseif Lm <= 100 then
													return LinFmod(1,600.3,774,95,100,L);
												elseif Lm <= 105 then
													return LinFmod(1,774,985.5,100,105,L);
												elseif Lm <= 115 then
													return LinFmod(1,985.5,1350,105,115,L);
												elseif Lm <= 120 then
													return LinFmod(1,1350,1710,115,120,L);
												elseif Lm <= 130 then
													return LinFmod(1,1710,2330.25,120,130,L);
												elseif Lm <= 140 then
													return LinFmod(1,2330.25,3600,130,140,L);
												else
													return LinFmod(1,3600,5400,140,150,L);
												end
											else
												return 0;
											end
										elseif SN > "PROGBCRITDEF" then
											if SN == "PROGBCRITHIT" then
												return CalcStat("BRatProgB",L,"BRatStandard");
											else
												return 0;
											end
										else
											return CalcStat("BRatProgB",L,"BRatStandard");
										end
									else
										return CalcStat("BRatProgB",L,"BRatStandard");
									end
								else
									return CalcStat("BRatProgB",L,"BRatMitHeavy");
								end
							else
								return 36/1200;
							end
						elseif SN > "PROGBENERGY" then
							if SN < "PROGBPOWER" then
								if SN < "PROGBMAIN" then
									if SN < "PROGBHEALTH" then
										if SN > "PROGBFATE" then
											if SN == "PROGBFINESSE" then
												return CalcStat("BRatProgB",L,"BRatStandard");
											else
												return 0;
											end
										elseif SN == "PROGBFATE" then
											return CalcStat("ProgBEnergy",L);
										else
											return 0;
										end
									elseif SN > "PROGBHEALTH" then
										if SN < "PROGBICPR" then
											if SN == "PROGBICMR" then
												return CalcStat("ProgBHealth",L);
											else
												return 0;
											end
										elseif SN > "PROGBICPR" then
											if SN == "PROGBINHEAL" then
												return CalcStat("BRatProgB",L,"BRatStandard");
											else
												return 0;
											end
										else
											return CalcStat("ProgBEnergy",L);
										end
									else
										if Lm <= 50 then
											return LinFmod(1,8,60,1,50,L);
										elseif Lm <= 60 then
											return LinFmod(1,60,80,50,60,L);
										elseif Lm <= 65 then
											return LinFmod(1,80,100,60,65,L);
										elseif Lm <= 75 then
											return LinFmod(1,100,150,65,75,L);
										elseif Lm <= 85 then
											return LinFmod(1,150,226,75,85,L);
										elseif Lm <= 95 then
											return LinFmod(1,226,300,85,95,L);
										elseif Lm <= 100 then
											return LinFmod(1,300,450,95,100,L);
										elseif Lm <= 105 then
											return LinFmod(1,450,600,100,105,L);
										elseif Lm <= 115 then
											return LinFmod(1,660,900,106,115,L);
										elseif Lm <= 120 then
											return LinFmod(1,1040,1120,116,120,L);
										elseif Lm <= 130 then
											return LinFmod(1,1280,1680,121,130,L);
										elseif Lm <= 140 then
											return LinFmod(1,1940,3360,131,140,L);
										else
											return LinFmod(1,3860,6800,141,150,L);
										end
									end
								elseif SN > "PROGBMAIN" then
									if SN < "PROGBMORALE" then
										if SN > "PROGBMASTERY" then
											if SN == "PROGBMITIGATION" then
												return CalcStat("BRatProgB",L,"BRatMitMedium");
											else
												return 0;
											end
										elseif SN == "PROGBMASTERY" then
											return CalcStat("BRatProgB",L,"BRatExtra");
										else
											return 0;
										end
									elseif SN > "PROGBMORALE" then
										if SN < "PROGBNCPR" then
											if SN == "PROGBNCMR" then
												return CalcStat("ProgBHealth",L);
											else
												return 0;
											end
										elseif SN > "PROGBNCPR" then
											if SN == "PROGBOUTHEAL" then
												return CalcStat("BRatProgB",L,"BRatOutHeal");
											else
												return 0;
											end
										else
											return CalcStat("ProgBEnergy",L);
										end
									else
										return CalcStat("ProgBHealth",L);
									end
								else
									return CalcStat("StdProg",L);
								end
							elseif SN > "PROGBPOWER" then
								if SN < "PROGEXTCOMLOWRND" then
									if SN < "PROGBVITALITY" then
										if SN > "PROGBRESIST" then
											if SN == "PROGBTACHPS" then
												return CalcStat("ProgBHealth",L);
											else
												return 0;
											end
										elseif SN == "PROGBRESIST" then
											return CalcStat("BRatProgB",L,"BRatExtra");
										else
											return 0;
										end
									elseif SN > "PROGBVITALITY" then
										if SN < "PROGEXTCOMHIGHRND" then
											if SN == "PROGEXTCOMHIGHRAW" then
												if Lm <= 121 then
													return ExpFmod(N,121,20,L,nil);
												elseif Lm <= 125 then
													return ExpFmod(CalcStat("ProgExtComHighRaw",121,N),122,5.5,L,nil);
												elseif Lm <= 126 then
													return ExpFmod(CalcStat("ProgExtComHighRaw",125,N),126,20,L,nil);
												elseif Lm <= 130 then
													return ExpFmod(CalcStat("ProgExtComHighRaw",126,N),127,5.5,L,nil);
												elseif Lm <= 131 then
													return ExpFmod(CalcStat("ProgExtComHighRaw",130,N),131,20,L,nil);
												elseif Lm <= 140 then
													return ExpFmod(CalcStat("ProgExtComHighRaw",131,N),132,5.5,L,nil);
												else
													return CalcStat("ProgExtComHighRaw",140,N);
												end
											else
												return 0;
											end
										elseif SN > "PROGEXTCOMHIGHRND" then
											if SN == "PROGEXTCOMLOWRAW" then
												if Lm <= 116 then
													return ExpFmod(N,116,20,L,nil);
												elseif Lm <= 120 then
													return ExpFmod(CalcStat("ProgExtComLowRaw",116,N),117,5.5,L,nil);
												else
													return CalcStat("ProgExtComHighRaw",L,CalcStat("ProgExtComLowRaw",120,N));
												end
											else
												return 0;
											end
										else
											if Lm <= 121 then
												return ExpFmod(N,121,20,L,0);
											elseif Lm <= 125 then
												return ExpFmod(CalcStat("ProgExtComHighRnd",121,N),122,5.5,L,0);
											elseif Lm <= 126 then
												return ExpFmod(CalcStat("ProgExtComHighRnd",125,N),126,20,L,0);
											elseif Lm <= 130 then
												return ExpFmod(CalcStat("ProgExtComHighRnd",126,N),127,5.5,L,0);
											elseif Lm <= 131 then
												return ExpFmod(CalcStat("ProgExtComHighRnd",130,N),131,20,L,0);
											elseif Lm <= 140 then
												return ExpFmod(CalcStat("ProgExtComHighRnd",131,N),132,5.5,L,0);
											else
												return CalcStat("ProgExtComHighRnd",140,N);
											end
										end
									else
										return CalcStat("ProgBHealth",L);
									end
								elseif SN > "PROGEXTCOMLOWRND" then
									if SN < "PROGEXTMEDEXPRAW" then
										if SN > "PROGEXTHIGHLINEXPRND" then
											if SN == "PROGEXTLOWEXPRND" then
												if Lm <= 115 then
													return ExpFmod(N,106,5.5,L,0);
												else
													return CalcStat("ProgExtComLowRnd",L,CalcStat("ProgExtLowExpRnd",115,N));
												end
											else
												return 0;
											end
										elseif SN == "PROGEXTHIGHLINEXPRND" then
											if Lm <= 115 then
												return LinFmod(N,2,4,106,115,L,-1);
											else
												return CalcStat("ProgExtComLowRnd",L,CalcStat("ProgExtHighLinExpRnd",115,N));
											end
										else
											return 0;
										end
									elseif SN > "PROGEXTMEDEXPRAW" then
										if SN < "REPAGILITYH" then
											if SN == "PROGEXTMPEXPRND" then
												if Lm <= 114 then
													return ExpFmod(N,106,7.5,L,0);
												elseif Lm <= 115 then
													return 2*N;
												elseif Lm <= 119 then
													return ExpFmod(CalcStat("ProgExtMpExpRnd",115,N),116,25,L,0);
												elseif Lm <= 120 then
													return RoundDbl((16/3)*N);
												else
													return CalcStat("ProgExtComHighRnd",L,CalcStat("ProgExtMpExpRnd",120,N));
												end
											else
												return 0;
											end
										elseif SN > "REPAGILITYH" then
											if SN == "REPAGILITYL" then
												return CalcStat("RepMainL",L);
											else
												return 0;
											end
										else
											return CalcStat("RepMainH",L);
										end
									else
										if Lm <= 106 then
											return ExpFmod(N,106,10,L,nil);
										elseif Lm <= 115 then
											return ExpFmod(CalcStat("ProgExtMedExpRaw",106,N),107,5.5,L,nil);
										else
											return CalcStat("ProgExtComLowRaw",L,CalcStat("ProgExtMedExpRaw",115,N));
										end
									end
								else
									if Lm <= 116 then
										return ExpFmod(N,116,20,L,0);
									elseif Lm <= 120 then
										return ExpFmod(CalcStat("ProgExtComLowRnd",116,N),117,5.5,L,0);
									else
										return CalcStat("ProgExtComHighRnd",L,CalcStat("ProgExtComLowRnd",120,N));
									end
								end
							else
								return CalcStat("ProgBEnergy",L);
							end
						else
							if Lm <= 50 then
								return LinFmod(1,4,8,1,50,L);
							elseif Lm <= 60 then
								return LinFmod(1,8,10.6,50,60,L);
							elseif Lm <= 65 then
								return LinFmod(1,10.6,13.2,60,65,L);
							elseif Lm <= 75 then
								return LinFmod(1,13.2,19.8,65,75,L);
							elseif Lm <= 85 then
								return LinFmod(1,19.8,29.8,75,85,L);
							elseif Lm <= 95 then
								return LinFmod(1,29.8,39.6,85,95,L);
							elseif Lm <= 100 then
								return LinFmod(1,39.6,52,95,100,L);
							elseif Lm <= 105 then
								return LinFmod(1,52,70,100,105,L);
							elseif Lm <= 115 then
								return LinFmod(1,78,106,106,115,L);
							elseif Lm <= 120 then
								return LinFmod(1,122,132,116,120,L);
							elseif Lm <= 130 then
								return LinFmod(1,152,198,121,130,L);
							elseif Lm <= 140 then
								return LinFmod(1,228,396,131,140,L);
							else
								return LinFmod(1,456,800,141,150,L);
							end
						end
					elseif SN > "REPCRITDEF" then
						if SN < "RIVERHOBBITRDPSVONENAME" then
							if SN < "REPWILLH" then
								if SN < "REPMIGHTH" then
									if SN < "REPFATEL" then
										if SN > "REPCRITHIT" then
											if SN == "REPFATEH" then
												return CalcStat("RepMainH",L);
											else
												return 0;
											end
										elseif SN == "REPCRITHIT" then
											if Lm <= 50 then
												return LinFmod(1,300,496,1,50,L);
											elseif Lm <= 85 then
												return LinFmod(1,496,636,50,85,L);
											elseif Lm <= 105 then
												return LinFmod(1,636,716,85,105,L);
											else
												return LinFmod(1,716,776,105,120,L);
											end
										else
											return 0;
										end
									elseif SN > "REPFATEL" then
										if SN < "REPMAINH" then
											if SN == "REPFINESSE" then
												if Lm <= 50 then
													return LinFmod(1,322,557,1,50,L);
												elseif Lm <= 85 then
													return LinFmod(1,557,749.9477,50,85,L);
												elseif Lm <= 105 then
													return LinFmod(1,749.9477,859.1634,85,105,L);
												else
													return LinFmod(1,859.1634,939.2549,105,120,L);
												end
											else
												return 0;
											end
										elseif SN > "REPMAINH" then
											if SN == "REPMAINL" then
												if Lm <= 50 then
													return RoundDblDown(LinFmod(1,53,102,1,50,L));
												elseif Lm <= 85 then
													return RoundDblDown(LinFmod(1,102,137,50,85,L));
												elseif Lm <= 105 then
													return RoundDblDown(LinFmod(1,137,157,85,105,L));
												else
													return RoundDblDown(LinFmod(1,157,172,105,120,L));
												end
											else
												return 0;
											end
										else
											if Lm <= 50 then
												return RoundDblDown(LinFmod(1,80,153,1,50,L));
											elseif Lm <= 85 then
												return RoundDblDown(LinFmod(1,153,206,50,85,L));
											elseif Lm <= 105 then
												return RoundDblDown(LinFmod(1,206,236,85,105,L));
											else
												return RoundDblDown(LinFmod(1,236,258,105,120,L));
											end
										end
									else
										return CalcStat("RepMainL",L);
									end
								elseif SN > "REPMIGHTH" then
									if SN < "REPPOWER" then
										if SN > "REPMIGHTL" then
											if SN == "REPMORALE" then
												if Lm <= 50 then
													return LinFmod(1,187,427,1,50,L);
												elseif Lm <= 85 then
													return LinFmod(1,427,599,50,85,L);
												elseif Lm <= 105 then
													return LinFmod(1,599,697,85,105,L);
												else
													return LinFmod(1,697,770,105,120,L);
												end
											else
												return 0;
											end
										elseif SN == "REPMIGHTL" then
											return CalcStat("RepMainL",L);
										else
											return 0;
										end
									elseif SN > "REPPOWER" then
										if SN < "REPVITALITYH" then
											if SN == "REPTACMIT" then
												if Lm <= 50 then
													return LinFmod(1,675,1116,1,50,L);
												elseif Lm <= 85 then
													return LinFmod(1,1116,1431,50,85,L);
												elseif Lm <= 105 then
													return LinFmod(1,1431,1611,85,105,L);
												else
													return LinFmod(1,1611,1746,105,120,L);
												end
											else
												return 0;
											end
										elseif SN > "REPVITALITYH" then
											if SN == "REPVITALITYL" then
												return CalcStat("RepMainL",L);
											else
												return 0;
											end
										else
											return CalcStat("RepMainH",L);
										end
									else
										if Lm <= 50 then
											return LinFmod(1,94,212,1,50,L);
										elseif Lm <= 85 then
											return LinFmod(1,212,296,50,85,L);
										elseif Lm <= 105 then
											return LinFmod(1,296,344,85,105,L);
										else
											return LinFmod(1,344,380,105,120,L);
										end
									end
								else
									return CalcStat("RepMainH",L);
								end
							elseif SN > "REPWILLH" then
								if SN < "RESISTPRATPA" then
									if SN < "RESISTADDT" then
										if SN > "REPWILLL" then
											if SN == "RESIST" then
												return StatLinInter("PntMPResist","ItemProg","ProgBResist","",L,N,0);
											else
												return 0;
											end
										elseif SN == "REPWILLL" then
											return CalcStat("RepMainL",L);
										else
											return 0;
										end
									elseif SN > "RESISTADDT" then
										if SN < "RESISTPPRAT" then
											if SN == "RESISTPBONUS" then
												return 0;
											else
												return 0;
											end
										elseif SN > "RESISTPPRAT" then
											if SN == "RESISTPRATP" then
												return CalcPercAB(CalcStat("ResistPRatPA",L),CalcStat("ResistPRatPB",L),CalcStat("ResistPRatPCap",L),N);
											else
												return 0;
											end
										else
											return CalcRatAB(CalcStat("ResistPRatPA",L),CalcStat("ResistPRatPB",L),CalcStat("ResistPRatPCapR",L),N);
										end
									else
										return CalcStat("ResistT",L,N);
									end
								elseif SN > "RESISTPRATPA" then
									if SN < "RESISTPRATPCAP" then
										if SN > "RESISTPRATPB" then
											if SN == "RESISTPRATPC" then
												return 0.5;
											else
												return 0;
											end
										elseif SN == "RESISTPRATPB" then
											return CalcStat("BRatRounded",L,"BRatExtra");
										else
											return 0;
										end
									elseif SN > "RESISTPRATPCAP" then
										if SN < "RESISTT" then
											if SN == "RESISTPRATPCAPR" then
												return CalcStat("ResistPRatPB",L)*CalcStat("ResistPRatPC",L);
											else
												return 0;
											end
										elseif SN > "RESISTT" then
											if SN == "REVERSE" then
												return ReverseCalc(C,L);
											else
												return 0;
											end
										else
											return StatLinInter("PntMPResist","TraitProg","ProgBResist","",L,N,0);
										end
									else
										return 50;
									end
								else
									return 150;
								end
							else
								return CalcStat("RepMainH",L);
							end
						elseif SN > "RIVERHOBBITRDPSVONENAME" then
							if SN < "RUNEKEEPERCDAGILITYTOCRITHIT" then
								if SN < "RIVHOBHARDYHOLBMORALE" then
									if SN < "RIVERHOBBITRDTRAITAGILITY" then
										if SN > "RIVERHOBBITRDPSVONEWILL" then
											if SN == "RIVERHOBBITRDPSVTWONAME" then
												return "";
											else
												return 0;
											end
										elseif SN == "RIVERHOBBITRDPSVONEWILL" then
											return CalcStat("RivHobSeenWorldWill",L);
										else
											return 0;
										end
									elseif SN > "RIVERHOBBITRDTRAITAGILITY" then
										if SN < "RIVERHOBBITRDTRAITMORALE" then
											if SN == "RIVERHOBBITRDTRAITFROSTMITP" then
												return CalcStat("RivHobSwimmerFrostMitP",L);
											else
												return 0;
											end
										elseif SN > "RIVERHOBBITRDTRAITMORALE" then
											if SN == "RIVERHOBBITRDTRAITWILL" then
												return CalcStat("RivHobSeclusionWill",L);
											else
												return 0;
											end
										else
											return CalcStat("RivHobHardyHolbMorale",L);
										end
									else
										return CalcStat("RivHobSlipperyAgility",L);
									end
								elseif SN > "RIVHOBHARDYHOLBMORALE" then
									if SN < "RIVHOBSLIPPERYAGILITY" then
										if SN > "RIVHOBSECLUSIONWILL" then
											if SN == "RIVHOBSEENWORLDWILL" then
												return CalcStat("WillT",L,1.0);
											else
												return 0;
											end
										elseif SN == "RIVHOBSECLUSIONWILL" then
											return -CalcStat("WillT",L,0.4);
										else
											return 0;
										end
									elseif SN > "RIVHOBSLIPPERYAGILITY" then
										if SN < "RKDETERMINATIONWILL" then
											if SN == "RIVHOBSWIMMERFROSTMITP" then
												return 1;
											else
												return 0;
											end
										elseif SN > "RKDETERMINATIONWILL" then
											if SN == "RKFORTUNESMILESFATE" then
												return CalcStat("FateT",L,CalcStat("Trait567810Choice",N)*0.4);
											else
												return 0;
											end
										else
											return CalcStat("WillT",L,CalcStat("Trait567810Choice",N)*0.4);
										end
									else
										return CalcStat("AgilityT",L,1.0);
									end
								else
									return CalcStat("MoraleT",L,1.0);
								end
							elseif SN > "RUNEKEEPERCDAGILITYTOCRITHIT" then
								if SN < "RUNEKEEPERCDBASEAGILITY" then
									if SN < "RUNEKEEPERCDARMOURTOOFMIT" then
										if SN > "RUNEKEEPERCDAGILITYTOEVADE" then
											if SN == "RUNEKEEPERCDAGILITYTOPARRY" then
												return 2;
											else
												return 0;
											end
										elseif SN == "RUNEKEEPERCDAGILITYTOEVADE" then
											return 3;
										else
											return 0;
										end
									elseif SN > "RUNEKEEPERCDARMOURTOOFMIT" then
										if SN < "RUNEKEEPERCDARMOURTOTACMIT" then
											if SN == "RUNEKEEPERCDARMOURTOPHYMIT" then
												return 1;
											else
												return 0;
											end
										elseif SN > "RUNEKEEPERCDARMOURTOTACMIT" then
											if SN == "RUNEKEEPERCDARMOURTYPE" then
												return 1;
											else
												return 0;
											end
										else
											return 0.2;
										end
									else
										return 0.2;
									end
								elseif SN > "RUNEKEEPERCDBASEAGILITY" then
									if SN < "RUNEKEEPERCDBASEMIGHT" then
										if SN < "RUNEKEEPERCDBASEICMR" then
											if SN == "RUNEKEEPERCDBASEFATE" then
												return CalcStat("ClassBaseFate",L);
											else
												return 0;
											end
										elseif SN > "RUNEKEEPERCDBASEICMR" then
											if SN == "RUNEKEEPERCDBASEICPR" then
												return CalcStat("ClassBaseICPR",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseICMRL",L);
										end
									elseif SN > "RUNEKEEPERCDBASEMIGHT" then
										if SN < "RUNEKEEPERCDBASENCMR" then
											if SN == "RUNEKEEPERCDBASEMORALE" then
												return CalcStat("ClassBaseMorale",L);
											else
												return 0;
											end
										elseif SN > "RUNEKEEPERCDBASENCMR" then
											if SN == "RUNEKEEPERCDBASENCPR" then
												return CalcStat("ClassBaseNCPR",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseNCMRL",L);
										end
									else
										return CalcStat("ClassBaseMightM",L);
									end
								else
									return CalcStat("ClassBaseAgilityL",L);
								end
							else
								return 1;
							end
						else
							return "Seen the World";
						end
					else
						if Lm <= 50 then
							return LinFmod(1,900,1488,1,50,L);
						elseif Lm <= 85 then
							return LinFmod(1,1488,1908,50,85,L);
						elseif Lm <= 105 then
							return LinFmod(1,1908,2148,85,105,L);
						else
							return LinFmod(1,2148,2328,105,120,L);
						end
					end
				elseif SN > "RUNEKEEPERCDBASEPOWER" then
					if SN < "STOUTWRBLACKLSHADOWMITP" then
						if SN < "SONGRESISTT" then
							if SN < "RUNEKEEPERCDVITALITYTOMORALE" then
								if SN < "RUNEKEEPERCDFATETORESIST" then
									if SN < "RUNEKEEPERCDFATETOCRITHIT" then
										if SN > "RUNEKEEPERCDBASEVITALITY" then
											if SN == "RUNEKEEPERCDBASEWILL" then
												return CalcStat("ClassBaseWillH",L);
											else
												return 0;
											end
										elseif SN == "RUNEKEEPERCDBASEVITALITY" then
											return CalcStat("ClassBaseVitality",L);
										else
											return 0;
										end
									elseif SN > "RUNEKEEPERCDFATETOCRITHIT" then
										if SN < "RUNEKEEPERCDFATETONCPR" then
											if SN == "RUNEKEEPERCDFATETOICMR" then
												return 0.025;
											else
												return 0;
											end
										elseif SN > "RUNEKEEPERCDFATETONCPR" then
											if SN == "RUNEKEEPERCDFATETOPOWER" then
												return 1;
											else
												return 0;
											end
										else
											return 0.05;
										end
									else
										return 1.5;
									end
								elseif SN > "RUNEKEEPERCDFATETORESIST" then
									if SN < "RUNEKEEPERCDMIGHTTOBLOCK" then
										if SN > "RUNEKEEPERCDFATETOTACMIT" then
											if SN == "RUNEKEEPERCDHASPOWER" then
												return 1;
											else
												return 0;
											end
										elseif SN == "RUNEKEEPERCDFATETOTACMIT" then
											return 0.5;
										else
											return 0;
										end
									elseif SN > "RUNEKEEPERCDMIGHTTOBLOCK" then
										if SN < "RUNEKEEPERCDPHYMITTOOFMIT" then
											if SN == "RUNEKEEPERCDMIGHTTOPARRY" then
												return 2;
											else
												return 0;
											end
										elseif SN > "RUNEKEEPERCDPHYMITTOOFMIT" then
											if SN == "RUNEKEEPERCDTACMASTOOUTHEAL" then
												return 1;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 3;
									end
								else
									return 0.5;
								end
							elseif SN > "RUNEKEEPERCDVITALITYTOMORALE" then
								if SN < "RUNEKEEPERCDWILLTOTACMAS" then
									if SN < "RUNEKEEPERCDVITALITYTOTACMIT" then
										if SN > "RUNEKEEPERCDVITALITYTONCMR" then
											if SN == "RUNEKEEPERCDVITALITYTORESIST" then
												return 0.5;
											else
												return 0;
											end
										elseif SN == "RUNEKEEPERCDVITALITYTONCMR" then
											return 0.12;
										else
											return 0;
										end
									elseif SN > "RUNEKEEPERCDVITALITYTOTACMIT" then
										if SN < "RUNEKEEPERCDWILLTOFINESSE" then
											if SN == "RUNEKEEPERCDWILLTOEVADE" then
												return 1;
											else
												return 0;
											end
										elseif SN > "RUNEKEEPERCDWILLTOFINESSE" then
											if SN == "RUNEKEEPERCDWILLTORESIST" then
												return 2;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 0.5;
									end
								elseif SN > "RUNEKEEPERCDWILLTOTACMAS" then
									if SN < "SHADOWMITT" then
										if SN > "RUNEKEEPERCDWILLTOTACMIT" then
											if SN == "SHADOWMIT" then
												return CalcStat("DmgTypeMit",L,N);
											else
												return 0;
											end
										elseif SN == "RUNEKEEPERCDWILLTOTACMIT" then
											return 1;
										else
											return 0;
										end
									elseif SN > "SHADOWMITT" then
										if SN < "SHIELDBRAWLERBLOCK" then
											if SN == "SHIELDBLOCK" then
												return StatLinInter("PntMPShieldBlock","ItemProg","ProgBBPE","",L,N,0);
											else
												return 0;
											end
										elseif SN > "SHIELDBRAWLERBLOCK" then
											if SN == "SKILLPOWERCOST" then
												return CalcStat("CombatDamageModEnergy",L,0.64*N);
											else
												return 0;
											end
										else
											return CalcStat("DwarfShieldBrwlBlock",L);
										end
									else
										return CalcStat("DmgTypeMitT",L,N);
									end
								else
									return 3;
								end
							else
								return 4;
							end
						elseif SN > "SONGRESISTT" then
							if SN < "STOUTAXERDTRAITDISEASERESISTP" then
								if SN < "STDPOWERL" then
									if SN < "STDMORALEL" then
										if SN > "STATC" then
											if SN == "STDMORALEH" then
												if Lm <= 105 then
													return RoundDbl(2*CalcStat("StdMoraleRaw",L));
												else
													return CalcStat("ProgExtLowExpRnd",L,CalcStat("StdMoraleH",105));
												end
											else
												return 0;
											end
										elseif SN == "STATC" then
											return CalcStat(C,L);
										else
											return 0;
										end
									elseif SN > "STDMORALEL" then
										if SN < "STDMORALERAW" then
											if SN == "STDMORALEM" then
												if Lm <= 105 then
													return RoundDbl(1.5*CalcStat("StdMoraleRaw",L));
												else
													return CalcStat("ProgExtLowExpRnd",L,CalcStat("StdMoraleM",105));
												end
											else
												return 0;
											end
										elseif SN > "STDMORALERAW" then
											if SN == "STDPOWERH" then
												if Lm <= 105 then
													return RoundDbl(2*CalcStat("StdPowerRaw",L));
												else
													return CalcStat("ProgExtLowExpRnd",L,CalcStat("StdPowerH",105));
												end
											else
												return 0;
											end
										else
											if Lm <= 50 then
												return LinFmod(1,7.51,227.59,1,50,L);
											else
												return LinFmod(1,227.59,1327.5,50,105,L);
											end
										end
									else
										if Lm <= 105 then
											return RoundDbl(CalcStat("StdMoraleRaw",L));
										else
											return CalcStat("ProgExtLowExpRnd",L,CalcStat("StdMoraleL",105));
										end
									end
								elseif SN > "STDPOWERL" then
									if SN < "STOUTAXERDPSVONENAME" then
										if SN > "STDPOWERRAW" then
											if SN == "STDPROG" then
												if Lm <= 50 then
													return LinFmod(1,1,10,1,50,L);
												elseif Lm <= 60 then
													return LinFmod(1,10,15,50,60,L);
												elseif Lm <= 65 then
													return LinFmod(1,15,20,60,65,L);
												elseif Lm <= 75 then
													return LinFmod(1,20,30,65,75,L);
												elseif Lm <= 85 then
													return LinFmod(1,30,45,75,85,L);
												elseif Lm <= 95 then
													return LinFmod(1,45,65,85,95,L);
												elseif Lm <= 100 then
													return LinFmod(1,65,90,95,100,L);
												elseif Lm <= 105 then
													return LinFmod(1,90,120,100,105,L);
												elseif Lm <= 115 then
													return LinFmod(1,132,180,106,115,L);
												elseif Lm <= 120 then
													return LinFmod(1,207,225,116,120,L);
												elseif Lm <= 130 then
													return LinFmod(1,259,340,121,130,L);
												elseif Lm <= 140 then
													return LinFmod(1,390,680,131,140,L);
												else
													return LinFmod(1,780,1360,141,150,L);
												end
											else
												return 0;
											end
										elseif SN == "STDPOWERRAW" then
											if Lm <= 50 then
												return LinFmod(1,3,91.0375,1,50,L);
											else
												return LinFmod(1,91.0375,421,50,105,L);
											end
										else
											return 0;
										end
									elseif SN > "STOUTAXERDPSVONENAME" then
										if SN < "STOUTAXERDPSVTWONAME" then
											if SN == "STOUTAXERDPSVONEVITALITY" then
												return CalcStat("StoutUnwritDestVitality",L);
											else
												return 0;
											end
										elseif SN > "STOUTAXERDPSVTWONAME" then
											if SN == "STOUTAXERDTRAITAGILITY" then
												return CalcStat("StoutWrBlackLAgility",L);
											else
												return 0;
											end
										else
											return "";
										end
									else
										return "Unwritten Destiny";
									end
								else
									if Lm <= 105 then
										return RoundDbl(CalcStat("StdPowerRaw",L));
									else
										return CalcStat("ProgExtLowExpRnd",L,CalcStat("StdPowerL",105));
									end
								end
							elseif SN > "STOUTAXERDTRAITDISEASERESISTP" then
								if SN < "STOUTDOOMDRASAFATE" then
									if SN < "STOUTAXERDTRAITPHYMITP" then
										if SN > "STOUTAXERDTRAITFATE" then
											if SN == "STOUTAXERDTRAITMIGHT" then
												return CalcStat("StoutWrBlackLMight",L);
											else
												return 0;
											end
										elseif SN == "STOUTAXERDTRAITFATE" then
											return CalcStat("StoutDoomDrasaFate",L);
										else
											return 0;
										end
									elseif SN > "STOUTAXERDTRAITPHYMITP" then
										if SN < "STOUTAXERDTRAITVITALITY" then
											if SN == "STOUTAXERDTRAITSHADOWMITP" then
												return CalcStat("StoutWrBlackLShadowMitP",L);
											else
												return 0;
											end
										elseif SN > "STOUTAXERDTRAITVITALITY" then
											if SN == "STOUTAXERDTRAITWILL" then
												return CalcStat("StoutUnyieldingWill",L);
											else
												return 0;
											end
										else
											return CalcStat("StoutShadowEyeVitality",L);
										end
									else
										return CalcStat("StoutUnyieldingPhyMitP",L);
									end
								elseif SN > "STOUTDOOMDRASAFATE" then
									if SN < "STOUTUNYIELDINGWILL" then
										if SN < "STOUTUNWRITDESTVITALITY" then
											if SN == "STOUTSHADOWEYEVITALITY" then
												return -CalcStat("VitalityT",L,0.4);
											else
												return 0;
											end
										elseif SN > "STOUTUNWRITDESTVITALITY" then
											if SN == "STOUTUNYIELDINGPHYMITP" then
												return 1;
											else
												return 0;
											end
										else
											return CalcStat("VitalityT",L,1.0);
										end
									elseif SN > "STOUTUNYIELDINGWILL" then
										if SN < "STOUTWRBLACKLDISEASERESISTP" then
											if SN == "STOUTWRBLACKLAGILITY" then
												return CalcStat("AgilityT",L,1.0);
											else
												return 0;
											end
										elseif SN > "STOUTWRBLACKLDISEASERESISTP" then
											if SN == "STOUTWRBLACKLMIGHT" then
												return CalcStat("MightT",L,1.0);
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return CalcStat("WillT",L,1.0);
									end
								else
									return -CalcStat("FateT",L,0.4);
								end
							else
								return CalcStat("StoutWrBlackLDiseaseResistP",L);
							end
						else
							return CalcStat("ResistAddT",L,N);
						end
					elseif SN > "STOUTWRBLACKLSHADOWMITP" then
						if SN < "TACMITLPRATPA" then
							if SN < "TACMASOLD" then
								if SN < "TACDMGPRATP" then
									if SN < "T2PENMIT" then
										if SN > "T2PENARMOUR" then
											if SN == "T2PENBPE" then
												if Lm <= 115 then
													return (-40)*L;
												else
													return CalcStat("ProgExtComLowRaw",L,CalcStat("T2PenBPE",115));
												end
											else
												return 0;
											end
										elseif SN == "T2PENARMOUR" then
											return CalcStat("T2penMit",L);
										else
											return 0;
										end
									elseif SN > "T2PENMIT" then
										if SN < "TACDMGPBONUS" then
											if SN == "T2PENRESIST" then
												if Lm <= 115 then
													return (-90)*L;
												else
													return CalcStat("ProgExtComLowRaw",L,CalcStat("T2PenResist",115));
												end
											else
												return 0;
											end
										elseif SN > "TACDMGPBONUS" then
											if SN == "TACDMGPPRAT" then
												return CalcStat("OutDmgPPRat",L,N);
											else
												return 0;
											end
										else
											return CalcStat("OutDmgPBonus",L);
										end
									else
										if Lm <= 115 then
											return RoundDblDown(L*13.5)*-5;
										else
											return CalcStat("ProgExtComLowRaw",L,CalcStat("T2PenMit",115));
										end
									end
								elseif SN > "TACDMGPRATP" then
									if SN < "TACDMGPRATPC" then
										if SN > "TACDMGPRATPA" then
											if SN == "TACDMGPRATPB" then
												return CalcStat("OutDmgPRatPB",L);
											else
												return 0;
											end
										elseif SN == "TACDMGPRATPA" then
											return CalcStat("OutDmgPRatPA",L);
										else
											return 0;
										end
									elseif SN > "TACDMGPRATPC" then
										if SN < "TACDMGPRATPCAPR" then
											if SN == "TACDMGPRATPCAP" then
												return CalcStat("OutDmgPRatPCap",L);
											else
												return 0;
											end
										elseif SN > "TACDMGPRATPCAPR" then
											if SN == "TACMAS" then
												return CalcStat("Mastery",L,N);
											else
												return 0;
											end
										else
											return CalcStat("OutDmgPRatPCapR",L);
										end
									else
										return CalcStat("OutDmgPRatPC",L);
									end
								else
									return CalcStat("OutDmgPRatP",L,N);
								end
							elseif SN > "TACMASOLD" then
								if SN < "TACMITHPRATPB" then
									if SN < "TACMITHPBONUS" then
										if SN > "TACMAST" then
											if SN == "TACMIT" then
												return StatLinInter("PntMPTacMit","ItemProg","ProgBMitigation","",L,N,0);
											else
												return 0;
											end
										elseif SN == "TACMAST" then
											return CalcStat("MasteryT",L,N);
										else
											return 0;
										end
									elseif SN > "TACMITHPBONUS" then
										if SN < "TACMITHPRATP" then
											if SN == "TACMITHPPRAT" then
												return CalcStat("MitHeavyPPRat",L,N);
											else
												return 0;
											end
										elseif SN > "TACMITHPRATP" then
											if SN == "TACMITHPRATPA" then
												return CalcStat("MitHeavyPRatPA",L);
											else
												return 0;
											end
										else
											return CalcStat("MitHeavyPRatP",L,N);
										end
									else
										return CalcStat("MitHeavyPBonus",L);
									end
								elseif SN > "TACMITHPRATPB" then
									if SN < "TACMITHPRATPCAPR" then
										if SN > "TACMITHPRATPC" then
											if SN == "TACMITHPRATPCAP" then
												return CalcStat("MitHeavyPRatPCap",L);
											else
												return 0;
											end
										elseif SN == "TACMITHPRATPC" then
											return CalcStat("MitHeavyPRatPC",L);
										else
											return 0;
										end
									elseif SN > "TACMITHPRATPCAPR" then
										if SN < "TACMITLPPRAT" then
											if SN == "TACMITLPBONUS" then
												return CalcStat("MitLightPBonus",L);
											else
												return 0;
											end
										elseif SN > "TACMITLPPRAT" then
											if SN == "TACMITLPRATP" then
												return CalcStat("MitLightPRatP",L,N);
											else
												return 0;
											end
										else
											return CalcStat("MitLightPPRat",L,N);
										end
									else
										return CalcStat("MitHeavyPRatPCapR",L);
									end
								else
									return CalcStat("MitHeavyPRatPB",L);
								end
							else
								return CalcStat("Mastery",L,N);
							end
						elseif SN > "TACMITLPRATPA" then
							if SN < "TACRESISTT" then
								if SN < "TACMITMPRATP" then
									if SN < "TACMITLPRATPCAP" then
										if SN > "TACMITLPRATPB" then
											if SN == "TACMITLPRATPC" then
												return CalcStat("MitLightPRatPC",L);
											else
												return 0;
											end
										elseif SN == "TACMITLPRATPB" then
											return CalcStat("MitLightPRatPB",L);
										else
											return 0;
										end
									elseif SN > "TACMITLPRATPCAP" then
										if SN < "TACMITMPBONUS" then
											if SN == "TACMITLPRATPCAPR" then
												return CalcStat("MitLightPRatPCapR",L);
											else
												return 0;
											end
										elseif SN > "TACMITMPBONUS" then
											if SN == "TACMITMPPRAT" then
												return CalcStat("MitMediumPPRat",L,N);
											else
												return 0;
											end
										else
											return CalcStat("MitMediumPBonus",L);
										end
									else
										return CalcStat("MitLightPRatPCap",L);
									end
								elseif SN > "TACMITMPRATP" then
									if SN < "TACMITMPRATPC" then
										if SN > "TACMITMPRATPA" then
											if SN == "TACMITMPRATPB" then
												return CalcStat("MitMediumPRatPB",L);
											else
												return 0;
											end
										elseif SN == "TACMITMPRATPA" then
											return CalcStat("MitMediumPRatPA",L);
										else
											return 0;
										end
									elseif SN > "TACMITMPRATPC" then
										if SN < "TACMITMPRATPCAPR" then
											if SN == "TACMITMPRATPCAP" then
												return CalcStat("MitMediumPRatPCap",L);
											else
												return 0;
											end
										elseif SN > "TACMITMPRATPCAPR" then
											if SN == "TACMITT" then
												return StatLinInter("PntMPTacMit","TraitProg","ProgBMitigation","",L,N,0);
											else
												return 0;
											end
										else
											return CalcStat("MitMediumPRatPCapR",L);
										end
									else
										return CalcStat("MitMediumPRatPC",L);
									end
								else
									return CalcStat("MitMediumPRatP",L,N);
								end
							elseif SN > "TACRESISTT" then
								if SN < "TOMETOTALLEVEL" then
									if SN < "TOMEMAIN" then
										if SN > "TOMEFATE" then
											if SN == "TOMEFATEDEC" then
												return CalcStat("TomeTotalFateDec",L)-CalcStat("TomeTotalFateDec",L-1);
											else
												return 0;
											end
										elseif SN == "TOMEFATE" then
											return CalcStat("TomeFateDec",RomanRankDecode(C));
										else
											return 0;
										end
									elseif SN > "TOMEMAIN" then
										if SN < "TOMETOTALFATE" then
											if SN == "TOMEMAINDEC" then
												return CalcStat("TomeTotalMainDec",L)-CalcStat("TomeTotalMainDec",L-1);
											else
												return 0;
											end
										elseif SN > "TOMETOTALFATE" then
											if SN == "TOMETOTALFATEDEC" then
												return CalcStat("FateT",CalcStat("TomeTotalLevel",L),2.0);
											else
												return 0;
											end
										else
											return CalcStat("TomeTotalFateDec",RomanRankDecode(C));
										end
									else
										return CalcStat("TomeMainDec",RomanRankDecode(C));
									end
								elseif SN > "TOMETOTALLEVEL" then
									if SN < "TOMETOTALVITALITYDEC" then
										if SN < "TOMETOTALMAINDEC" then
											if SN == "TOMETOTALMAIN" then
												return CalcStat("TomeTotalMainDec",RomanRankDecode(C));
											else
												return 0;
											end
										elseif SN > "TOMETOTALMAINDEC" then
											if SN == "TOMETOTALVITALITY" then
												return CalcStat("TomeTotalVitalityDec",RomanRankDecode(C));
											else
												return 0;
											end
										else
											return CalcStat("MainT",CalcStat("TomeTotalLevel",L),2.0);
										end
									elseif SN > "TOMETOTALVITALITYDEC" then
										if SN < "TOMEVITALITYDEC" then
											if SN == "TOMEVITALITY" then
												return CalcStat("TomeVitalityDec",RomanRankDecode(C));
											else
												return 0;
											end
										elseif SN > "TOMEVITALITYDEC" then
											if SN == "TPENARMOUR" then
												return -CalcStat("ArmourPenT",L,CalcStat("TpenChoice",N));
											else
												return 0;
											end
										else
											return CalcStat("TomeTotalVitalityDec",L)-CalcStat("TomeTotalVitalityDec",L-1);
										end
									else
										return CalcStat("VitalityT",CalcStat("TomeTotalLevel",L),2.0);
									end
								else
									if Lm <= 0 then
										return 0;
									elseif Lm <= 3 then
										return 6*L+1;
									elseif Lm <= 10 then
										return 5*L+5;
									elseif Lm <= 11 then
										return 5*L+10;
									elseif Lm <= 18 then
										return 5*L+15;
									elseif Lm <= 22 then
										return RoundDbl(3.2*L+45.4);
									else
										return 0*L+118;
									end
								end
							else
								return CalcStat("ResistAddT",L,N);
							end
						else
							return CalcStat("MitLightPRatPA",L);
						end
					else
						return 1;
					end
				else
					return CalcStat("ClassBasePower",L);
				end
			else
				return 0.5;
			end
		elseif SN > "TPENBPE" then
			if SN < "WARDENCDWILLTORESIST" then
				if SN < "VITALITY" then
					if SN < "VIRTHONESTYVPTACMAS" then
						if SN < "VIRTCONFIDENCEEVADE" then
							if SN < "TRAIT58121620CHOICE" then
								if SN < "TRAIT13510CHOICE" then
									if SN < "TRAIT12345CHOICE" then
										if SN > "TPENCHOICE" then
											if SN == "TPENRESIST" then
												return -CalcStat("ResistT",L,CalcStat("TpenChoice",N)*2);
											else
												return 0;
											end
										elseif SN == "TPENCHOICE" then
											return DataTableValue({0,1,2},L);
										else
											return 0;
										end
									elseif SN > "TRAIT12345CHOICE" then
										if SN < "TRAIT1234CHOICE" then
											if SN == "TRAIT12347CHOICE" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({1,2,3,4,7},L);
												end
											else
												return 0;
											end
										elseif SN > "TRAIT1234CHOICE" then
											if SN == "TRAIT123CHOICE" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({1,2,3},L);
												end
											else
												return 0;
											end
										else
											if Lm <= 0 then
												return 0;
											else
												return DataTableValue({1,2,3,4},L);
											end
										end
									else
										if Lm <= 0 then
											return 0;
										else
											return DataTableValue({1,2,3,4,5},L);
										end
									end
								elseif SN > "TRAIT13510CHOICE" then
									if SN < "TRAIT357912CHOICE" then
										if SN > "TRAIT23456CHOICE" then
											if SN == "TRAIT234CHOICE" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({2,3,4},L);
												end
											else
												return 0;
											end
										elseif SN == "TRAIT23456CHOICE" then
											if Lm <= 0 then
												return 0;
											else
												return DataTableValue({2,3,4,5,6},L);
											end
										else
											return 0;
										end
									elseif SN > "TRAIT357912CHOICE" then
										if SN < "TRAIT567810CHOICE" then
											if SN == "TRAIT47101316CHOICE" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({4,7,10,13,16},L);
												end
											else
												return 0;
											end
										elseif SN > "TRAIT567810CHOICE" then
											if SN == "TRAIT56789CHOICE" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({5,6,7,8,9},L);
												end
											else
												return 0;
											end
										else
											if Lm <= 0 then
												return 0;
											else
												return DataTableValue({5,6,7,8,10},L);
											end
										end
									else
										if Lm <= 0 then
											return 0;
										else
											return DataTableValue({3,5,7,9,12},L);
										end
									end
								else
									if Lm <= 0 then
										return 0;
									else
										return DataTableValue({1,3,5,10},L);
									end
								end
							elseif SN > "TRAIT58121620CHOICE" then
								if SN < "VIRTCHARITYVITALITY" then
									if SN < "U371LEGACYSTATFIX" then
										if SN > "TRAITPROG" then
											if SN == "TRAITPROGVITAL" then
												return {{1,25,50,60,65,75,85,95,100,105,115,120,130,140,150},{1,25,50,60,65,75,85,95,100,105,115,120,130,140,150}};
											else
												return 0;
											end
										elseif SN == "TRAITPROG" then
											return {{1,25,50,60,65,75,85,95,100,105,115,120,130,131,140,141},{1,25,50,60,65,75,85,95,100,105,115,120,130,131,140,141}};
										else
											return 0;
										end
									elseif SN > "U371LEGACYSTATFIX" then
										if SN < "VIRTCHARITYPHYMIT" then
											if SN == "VARMOUR" then
												return RoundDbl(StatLinInter("PntMPArmourVirtues","ItemProg","ProgBArmourLight","",L,N,0));
											else
												return 0;
											end
										elseif SN > "VIRTCHARITYPHYMIT" then
											if SN == "VIRTCHARITYRESIST" then
												return CalcStat("VSResistH",L);
											else
												return 0;
											end
										else
											return CalcStat("VSPhyMitM",L);
										end
									else
										if Lm <= 2 then
											return 0;
										else
											return RoundDblUp(CalcStat(C,L-2)*0.07);
										end
									end
								elseif SN > "VIRTCHARITYVITALITY" then
									if SN < "VIRTCOMPASSIONPHYMIT" then
										if SN > "VIRTCHARITYVPMORALE" then
											if SN == "VIRTCOMPASSIONARMOUR" then
												return CalcStat("VSArmourL",L);
											else
												return 0;
											end
										elseif SN == "VIRTCHARITYVPMORALE" then
											return CalcStat("VSVPMorale",L);
										else
											return 0;
										end
									elseif SN > "VIRTCOMPASSIONPHYMIT" then
										if SN < "VIRTCOMPASSIONVPMORALE" then
											if SN == "VIRTCOMPASSIONTACMIT" then
												return CalcStat("VSTacMitM",L);
											else
												return 0;
											end
										elseif SN > "VIRTCOMPASSIONVPMORALE" then
											if SN == "VIRTCONFIDENCECRITHIT" then
												return CalcStat("VSCritHitH",L);
											else
												return 0;
											end
										else
											return CalcStat("VSVPMorale",L);
										end
									else
										return CalcStat("VSPhyMitH",L);
									end
								else
									return CalcStat("VSVitalityL",L);
								end
							else
								if Lm <= 0 then
									return 0;
								else
									return DataTableValue({5,8,12,16,20},L);
								end
							end
						elseif SN > "VIRTCONFIDENCEEVADE" then
							if SN < "VIRTEMPATHYCRITDEF" then
								if SN < "VIRTDETERMINATIONVPPHYMAS" then
									if SN < "VIRTCONFIDENCEVPTACMAS" then
										if SN > "VIRTCONFIDENCEFINESSE" then
											if SN == "VIRTCONFIDENCEVPPHYMAS" then
												return CalcStat("VSVPPhyMas",L);
											else
												return 0;
											end
										elseif SN == "VIRTCONFIDENCEFINESSE" then
											return CalcStat("VSFinesseM",L);
										else
											return 0;
										end
									elseif SN > "VIRTCONFIDENCEVPTACMAS" then
										if SN < "VIRTDETERMINATIONCRITHIT" then
											if SN == "VIRTDETERMINATIONAGILITY" then
												return CalcStat("VSAgilityH",L);
											else
												return 0;
											end
										elseif SN > "VIRTDETERMINATIONCRITHIT" then
											if SN == "VIRTDETERMINATIONPHYMAS" then
												return CalcStat("VSPhyMasM",L);
											else
												return 0;
											end
										else
											return CalcStat("VSCritHitL",L);
										end
									else
										return CalcStat("VSVPTacMas",L);
									end
								elseif SN > "VIRTDETERMINATIONVPPHYMAS" then
									if SN < "VIRTDISCIPLINEPHYMIT" then
										if SN > "VIRTDETERMINATIONVPTACMAS" then
											if SN == "VIRTDISCIPLINEINHEAL" then
												return CalcStat("VSInHealM",L);
											else
												return 0;
											end
										elseif SN == "VIRTDETERMINATIONVPTACMAS" then
											return CalcStat("VSVPTacMas",L);
										else
											return 0;
										end
									elseif SN > "VIRTDISCIPLINEPHYMIT" then
										if SN < "VIRTDISCIPLINEVPMORALE" then
											if SN == "VIRTDISCIPLINERESIST" then
												return CalcStat("VSResistH",L);
											else
												return 0;
											end
										elseif SN > "VIRTDISCIPLINEVPMORALE" then
											if SN == "VIRTEMPATHYARMOUR" then
												return CalcStat("VSArmourH",L);
											else
												return 0;
											end
										else
											return CalcStat("VSVPMorale",L);
										end
									else
										return CalcStat("VSPhyMitL",L);
									end
								else
									return CalcStat("VSVPPhyMas",L);
								end
							elseif SN > "VIRTEMPATHYCRITDEF" then
								if SN < "VIRTFORTITUDECRITDEF" then
									if SN < "VIRTFIDELITYPHYMIT" then
										if SN > "VIRTEMPATHYRESIST" then
											if SN == "VIRTEMPATHYVPMORALE" then
												return CalcStat("VSVPMorale",L);
											else
												return 0;
											end
										elseif SN == "VIRTEMPATHYRESIST" then
											return CalcStat("VSResistL",L);
										else
											return 0;
										end
									elseif SN > "VIRTFIDELITYPHYMIT" then
										if SN < "VIRTFIDELITYVITALITY" then
											if SN == "VIRTFIDELITYTACMIT" then
												return CalcStat("VSTacMitH",L);
											else
												return 0;
											end
										elseif SN > "VIRTFIDELITYVITALITY" then
											if SN == "VIRTFIDELITYVPMORALE" then
												return CalcStat("VSVPMorale",L);
											else
												return 0;
											end
										else
											return CalcStat("VSVitalityM",L);
										end
									else
										return CalcStat("VSPhyMitL",L);
									end
								elseif SN > "VIRTFORTITUDECRITDEF" then
									if SN < "VIRTFORTITUDEVPMORALE" then
										if SN > "VIRTFORTITUDEMORALE" then
											if SN == "VIRTFORTITUDERESIST" then
												return CalcStat("VSResistL",L);
											else
												return 0;
											end
										elseif SN == "VIRTFORTITUDEMORALE" then
											return CalcStat("VSMoraleH",L);
										else
											return 0;
										end
									elseif SN > "VIRTFORTITUDEVPMORALE" then
										if SN < "VIRTHONESTYTACMAS" then
											if SN == "VIRTHONESTYCRITHIT" then
												return CalcStat("VSCritHitL",L);
											else
												return 0;
											end
										elseif SN > "VIRTHONESTYTACMAS" then
											if SN == "VIRTHONESTYVPPHYMAS" then
												return CalcStat("VSVPPhyMas",L);
											else
												return 0;
											end
										else
											return CalcStat("VSTacMasH",L);
										end
									else
										return CalcStat("VSVPMorale",L);
									end
								else
									return CalcStat("VSCritDefM",L);
								end
							else
								return CalcStat("VSCritDefM",L);
							end
						else
							return CalcStat("VSEvadeL",L);
						end
					elseif SN > "VIRTHONESTYVPTACMAS" then
						if SN < "VIRTPATIENCEPOWER" then
							if SN < "VIRTJUSTICEICMR" then
								if SN < "VIRTIDEALISMINHEAL" then
									if SN < "VIRTHONOURMORALE" then
										if SN > "VIRTHONESTYWILL" then
											if SN == "VIRTHONOURCRITDEF" then
												return CalcStat("VSCritDefL",L);
											else
												return 0;
											end
										elseif SN == "VIRTHONESTYWILL" then
											return CalcStat("VSWillM",L);
										else
											return 0;
										end
									elseif SN > "VIRTHONOURMORALE" then
										if SN < "VIRTHONOURVPMORALE" then
											if SN == "VIRTHONOURTACMIT" then
												return CalcStat("VSTacMitM",L);
											else
												return 0;
											end
										elseif SN > "VIRTHONOURVPMORALE" then
											if SN == "VIRTIDEALISMFATE" then
												return CalcStat("VSFateH",L);
											else
												return 0;
											end
										else
											return CalcStat("VSVPMorale",L);
										end
									else
										return CalcStat("VSMoraleH",L);
									end
								elseif SN > "VIRTIDEALISMINHEAL" then
									if SN < "VIRTINNOCENCEPHYMIT" then
										if SN > "VIRTIDEALISMMORALE" then
											if SN == "VIRTIDEALISMVPMORALE" then
												return CalcStat("VSVPMorale",L);
											else
												return 0;
											end
										elseif SN == "VIRTIDEALISMMORALE" then
											return CalcStat("VSMoraleL",L);
										else
											return 0;
										end
									elseif SN > "VIRTINNOCENCEPHYMIT" then
										if SN < "VIRTINNOCENCETACMIT" then
											if SN == "VIRTINNOCENCERESIST" then
												return CalcStat("VSResistM",L);
											else
												return 0;
											end
										elseif SN > "VIRTINNOCENCETACMIT" then
											if SN == "VIRTINNOCENCEVPMORALE" then
												return CalcStat("VSVPMorale",L);
											else
												return 0;
											end
										else
											return CalcStat("VSTacMitL",L);
										end
									else
										return CalcStat("VSPhyMitH",L);
									end
								else
									return CalcStat("VSInHealM",L);
								end
							elseif SN > "VIRTJUSTICEICMR" then
								if SN < "VIRTLOYALTYVPMORALE" then
									if SN < "VIRTJUSTICEVPMORALE" then
										if SN > "VIRTJUSTICEMORALE" then
											if SN == "VIRTJUSTICETACMIT" then
												return CalcStat("VSTacMitL",L);
											else
												return 0;
											end
										elseif SN == "VIRTJUSTICEMORALE" then
											return CalcStat("VSMoraleM",L);
										else
											return 0;
										end
									elseif SN > "VIRTJUSTICEVPMORALE" then
										if SN < "VIRTLOYALTYINHEAL" then
											if SN == "VIRTLOYALTYARMOUR" then
												return CalcStat("VSArmourM",L);
											else
												return 0;
											end
										elseif SN > "VIRTLOYALTYINHEAL" then
											if SN == "VIRTLOYALTYVITALITY" then
												return CalcStat("VSVitalityH",L);
											else
												return 0;
											end
										else
											return CalcStat("VSInHealL",L);
										end
									else
										return CalcStat("VSVPMorale",L);
									end
								elseif SN > "VIRTLOYALTYVPMORALE" then
									if SN < "VIRTMERCYVITALITY" then
										if SN > "VIRTMERCYEVADE" then
											if SN == "VIRTMERCYFATE" then
												return CalcStat("VSFateM",L);
											else
												return 0;
											end
										elseif SN == "VIRTMERCYEVADE" then
											return CalcStat("VSEvadeH",L);
										else
											return 0;
										end
									elseif SN > "VIRTMERCYVITALITY" then
										if SN < "VIRTPATIENCECRITHIT" then
											if SN == "VIRTMERCYVPMORALE" then
												return CalcStat("VSVPMorale",L);
											else
												return 0;
											end
										elseif SN > "VIRTPATIENCECRITHIT" then
											if SN == "VIRTPATIENCEEVADE" then
												return CalcStat("VSEvadeM",L);
											else
												return 0;
											end
										else
											return CalcStat("VSCritHitL",L);
										end
									else
										return CalcStat("VSVitalityL",L);
									end
								else
									return CalcStat("VSVPMorale",L);
								end
							else
								return CalcStat("VSICMRH",L);
							end
						elseif SN > "VIRTPATIENCEPOWER" then
							if SN < "VIRTWISDOMTACMAS" then
								if SN < "VIRTTOLERANCEVPMORALE" then
									if SN < "VIRTRNKCOSTTOT" then
										if SN > "VIRTPATIENCEVPMORALE" then
											if SN == "VIRTRNKCOST" then
												if Lm <= 0 then
													return 0;
												elseif Lm <= 10 then
													return 1000;
												elseif Lm <= 60 then
													return RoundDbl(18*L+878,-2);
												elseif Lm <= 73 then
													return RoundDbl(18.75*L+878,-2);
												else
													return RoundDbl(17.45*L+878,-2);
												end
											else
												return 0;
											end
										elseif SN == "VIRTPATIENCEVPMORALE" then
											return CalcStat("VSVPMorale",L);
										else
											return 0;
										end
									elseif SN > "VIRTRNKCOSTTOT" then
										if SN < "VIRTTOLERANCERESIST" then
											if SN == "VIRTTOLERANCEPHYMIT" then
												return CalcStat("VSPhyMitL",L);
											else
												return 0;
											end
										elseif SN > "VIRTTOLERANCERESIST" then
											if SN == "VIRTTOLERANCETACMIT" then
												return CalcStat("VSTacMitH",L);
											else
												return 0;
											end
										else
											return CalcStat("VSResistM",L);
										end
									else
										if Lm <= 0 then
											return 0;
										else
											return CalcStat("VirtRnkCostTot",L-1)+CalcStat("VirtRnkCost",L);
										end
									end
								elseif SN > "VIRTTOLERANCEVPMORALE" then
									if SN < "VIRTVALOURPHYMAS" then
										if SN > "VIRTVALOURCRITHIT" then
											if SN == "VIRTVALOURFINESSE" then
												return CalcStat("VSFinesseM",L);
											else
												return 0;
											end
										elseif SN == "VIRTVALOURCRITHIT" then
											return CalcStat("VSCritHitL",L);
										else
											return 0;
										end
									elseif SN > "VIRTVALOURPHYMAS" then
										if SN < "VIRTVALOURVPTACMAS" then
											if SN == "VIRTVALOURVPPHYMAS" then
												return CalcStat("VSVPPhyMas",L);
											else
												return 0;
											end
										elseif SN > "VIRTVALOURVPTACMAS" then
											if SN == "VIRTWISDOMFINESSE" then
												return CalcStat("VSFinesseL",L);
											else
												return 0;
											end
										else
											return CalcStat("VSVPTacMas",L);
										end
									else
										return CalcStat("VSPhyMasH",L);
									end
								else
									return CalcStat("VSVPMorale",L);
								end
							elseif SN > "VIRTWISDOMTACMAS" then
								if SN < "VIRTWITTACMAS" then
									if SN < "VIRTWISDOMWILL" then
										if SN > "VIRTWISDOMVPPHYMAS" then
											if SN == "VIRTWISDOMVPTACMAS" then
												return CalcStat("VSVPTacMas",L);
											else
												return 0;
											end
										elseif SN == "VIRTWISDOMVPPHYMAS" then
											return CalcStat("VSVPPhyMas",L);
										else
											return 0;
										end
									elseif SN > "VIRTWISDOMWILL" then
										if SN < "VIRTWITFINESSE" then
											if SN == "VIRTWITCRITHIT" then
												return CalcStat("VSCritHitM",L);
											else
												return 0;
											end
										elseif SN > "VIRTWITFINESSE" then
											if SN == "VIRTWITPHYMAS" then
												return CalcStat("VSPhyMasL",L);
											else
												return 0;
											end
										else
											return CalcStat("VSFinesseH",L);
										end
									else
										return CalcStat("VSWillH",L);
									end
								elseif SN > "VIRTWITTACMAS" then
									if SN < "VIRTZEALMIGHT" then
										if SN < "VIRTWITVPTACMAS" then
											if SN == "VIRTWITVPPHYMAS" then
												return CalcStat("VSVPPhyMas",L);
											else
												return 0;
											end
										elseif SN > "VIRTWITVPTACMAS" then
											if SN == "VIRTZEALCRITHIT" then
												return CalcStat("VSCritHitL",L);
											else
												return 0;
											end
										else
											return CalcStat("VSVPTacMas",L);
										end
									elseif SN > "VIRTZEALMIGHT" then
										if SN < "VIRTZEALVPPHYMAS" then
											if SN == "VIRTZEALPHYMAS" then
												return CalcStat("VSPhyMasM",L);
											else
												return 0;
											end
										elseif SN > "VIRTZEALVPPHYMAS" then
											if SN == "VIRTZEALVPTACMAS" then
												return CalcStat("VSVPTacMas",L);
											else
												return 0;
											end
										else
											return CalcStat("VSVPPhyMas",L);
										end
									else
										return CalcStat("VSMightH",L);
									end
								else
									return CalcStat("VSTacMasL",L);
								end
							else
								return CalcStat("VSTacMasM",L);
							end
						else
							return CalcStat("VSPowerH",L);
						end
					else
						return CalcStat("VSVPTacMas",L);
					end
				elseif SN > "VITALITY" then
					if SN < "VSPOWERH" then
						if SN < "VSCRITHITM" then
							if SN < "VRNKCAP" then
								if SN < "VMHIGH" then
									if SN < "VITALITYTADJ" then
										if SN > "VITALITYADJ" then
											if SN == "VITALITYT" then
												return RoundDblDown(StatLinInter("PntMPVitalityT","TraitProgVital","ProgBVitality","VitalityTAdj",L,N,0));
											else
												return 0;
											end
										elseif SN == "VITALITYADJ" then
											if Lm <= 25 then
												return 0.5;
											elseif Lm <= 50 then
												return 0.6;
											elseif Lm <= 60 then
												return 0.7;
											elseif Lm <= 79 then
												return 0.8;
											elseif Lm <= 80 then
												return 0.9;
											else
												return 1;
											end
										else
											return 0;
										end
									elseif SN > "VITALITYTADJ" then
										if SN < "VMASTERYADJ" then
											if SN == "VMASTERY" then
												return StatLinInter("PntMPMastery","ItemProgVirtueMastery","ProgBMastery","VMasteryAdj",L,N,0);
											else
												return 0;
											end
										elseif SN > "VMASTERYADJ" then
											if SN == "VMASTERYOLD" then
												return CalcStat("VMastery",L,N);
											else
												return 0;
											end
										else
											if Lm <= 0 then
												return 0;
											else
												return 1;
											end
										end
									else
										if Lm <= 25 then
											return 0.5;
										elseif Lm <= 50 then
											return 0.6;
										elseif Lm <= 60 then
											return 0.7;
										elseif Lm <= 65 then
											return 0.8;
										elseif Lm <= 75 then
											return 0.9;
										else
											return 1;
										end
									end
								elseif SN > "VMHIGH" then
									if SN < "VMMEDIUM" then
										if SN > "VMLOW" then
											if SN == "VMMASTERYPSV" then
												if Lm <= 0 then
													return 0;
												else
													return CalcStat("VMastery",CalcStat("VRnkToILvl",L),0.2);
												end
											else
												return 0;
											end
										elseif SN == "VMLOW" then
											if Lm <= 0 then
												return 0;
											else
												return CalcStat(C,CalcStat("VRnkToILvl",L),0.6);
											end
										else
											return 0;
										end
									elseif SN > "VMMEDIUM" then
										if SN < "VMORALE" then
											if SN == "VMMORALEPSV" then
												if Lm <= 0 then
													return 0;
												else
													return CalcStat("VMorale",CalcStat("VRnkToILvl",L),0.3);
												end
											else
												return 0;
											end
										elseif SN > "VMORALE" then
											if SN == "VMORALEADJ" then
												if Lm <= 0 then
													return 0;
												elseif Lm <= 2 then
													return 1;
												elseif Lm <= 50 then
													return 1.5;
												elseif Lm <= 80 then
													return 1.028;
												elseif Lm <= 349 then
													return 1;
												elseif Lm <= 350 then
													return 1.042;
												elseif Lm <= 399 then
													return 93.72/99;
												elseif Lm <= 400 then
													return 86/99;
												elseif Lm <= 449 then
													return 0.9475;
												elseif Lm <= 450 then
													return 86/99;
												elseif Lm <= 499 then
													return 0.9475;
												else
													return 86/99;
												end
											else
												return 0;
											end
										else
											return StatLinInter("PntMPMoraleVirtues","ItemProgVirtueMorale","ProgBMorale","VMoraleAdj",L,N,0);
										end
									else
										if Lm <= 0 then
											return 0;
										else
											return CalcStat(C,CalcStat("VRnkToILvl",L),1.0);
										end
									end
								else
									if Lm <= 0 then
										return 0;
									else
										return CalcStat(C,CalcStat("VRnkToILvl",L),2.0);
									end
								end
							elseif SN > "VRNKCAP" then
								if SN < "VSARMOURL" then
									if SN < "VSAGILITYH" then
										if SN > "VRNKLVLCAP" then
											if SN == "VRNKTOILVL" then
												if Lm <= 0 then
													return 0;
												elseif Lm <= 38 then
													return LinFmod(1,4,78,1,38,L);
												elseif Lm <= 48 then
													return LinFmod(1,78,178,38,48,L);
												elseif Lm <= 49 then
													return LinFmod(1,178,190,48,49,L);
												elseif Lm <= 50 then
													return LinFmod(1,190,210,49,50,L);
												elseif Lm <= 51 then
													return LinFmod(1,210,222,50,51,L);
												elseif Lm <= 52 then
													return LinFmod(1,222,236,51,52,L);
												elseif Lm <= 53 then
													return LinFmod(1,236,260,52,53,L);
												elseif Lm <= 55 then
													return LinFmod(1,260,292,53,55,L);
												elseif Lm <= 68 then
													return LinFmod(1,292,396,55,68,L);
												else
													return LinFmod(1,396,706,68,130,L);
												end
											else
												return 0;
											end
										elseif SN == "VRNKLVLCAP" then
											if Lm <= 4 then
												return 2;
											elseif Lm <= 110 then
												return RoundDblDown(L/2);
											elseif Lm <= 139 then
												return L-55;
											else
												return 88;
											end
										else
											return 0;
										end
									elseif SN > "VSAGILITYH" then
										if SN < "VSAGILITYM" then
											if SN == "VSAGILITYL" then
												return CalcStat("VMLow",L,"Agility");
											else
												return 0;
											end
										elseif SN > "VSAGILITYM" then
											if SN == "VSARMOURH" then
												return CalcStat("VMHigh",L,"Varmour");
											else
												return 0;
											end
										else
											return CalcStat("VMMedium",L,"Agility");
										end
									else
										return CalcStat("VMHigh",L,"Agility");
									end
								elseif SN > "VSARMOURL" then
									if SN < "VSCRITDEFL" then
										if SN > "VSARMOURM" then
											if SN == "VSCRITDEFH" then
												return CalcStat("VMHigh",L,"CritDef");
											else
												return 0;
											end
										elseif SN == "VSARMOURM" then
											return CalcStat("VMMedium",L,"Varmour");
										else
											return 0;
										end
									elseif SN > "VSCRITDEFL" then
										if SN < "VSCRITHITH" then
											if SN == "VSCRITDEFM" then
												return CalcStat("VMMedium",L,"CritDef");
											else
												return 0;
											end
										elseif SN > "VSCRITHITH" then
											if SN == "VSCRITHITL" then
												return CalcStat("VMLow",L,"CritHit");
											else
												return 0;
											end
										else
											return CalcStat("VMHigh",L,"CritHit");
										end
									else
										return CalcStat("VMLow",L,"CritDef");
									end
								else
									return CalcStat("VMLow",L,"Varmour");
								end
							else
								return 86;
							end
						elseif SN > "VSCRITHITM" then
							if SN < "VSINHEALL" then
								if SN < "VSFINESSEH" then
									if SN < "VSEVADEM" then
										if SN > "VSEVADEH" then
											if SN == "VSEVADEL" then
												return CalcStat("VMLow",L,"Evade");
											else
												return 0;
											end
										elseif SN == "VSEVADEH" then
											return CalcStat("VMHigh",L,"Evade");
										else
											return 0;
										end
									elseif SN > "VSEVADEM" then
										if SN < "VSFATEL" then
											if SN == "VSFATEH" then
												return CalcStat("VMHigh",L,"Fate");
											else
												return 0;
											end
										elseif SN > "VSFATEL" then
											if SN == "VSFATEM" then
												return CalcStat("VMMedium",L,"Fate");
											else
												return 0;
											end
										else
											return CalcStat("VMLow",L,"Fate");
										end
									else
										return CalcStat("VMMedium",L,"Evade");
									end
								elseif SN > "VSFINESSEH" then
									if SN < "VSICMRH" then
										if SN > "VSFINESSEL" then
											if SN == "VSFINESSEM" then
												return CalcStat("VMMedium",L,"Finesse");
											else
												return 0;
											end
										elseif SN == "VSFINESSEL" then
											return CalcStat("VMLow",L,"Finesse");
										else
											return 0;
										end
									elseif SN > "VSICMRH" then
										if SN < "VSICMRM" then
											if SN == "VSICMRL" then
												return CalcStat("VMLow",L,"ICMR");
											else
												return 0;
											end
										elseif SN > "VSICMRM" then
											if SN == "VSINHEALH" then
												return CalcStat("VMHigh",L,"InHeal");
											else
												return 0;
											end
										else
											return CalcStat("VMMedium",L,"ICMR");
										end
									else
										return CalcStat("VMHigh",L,"ICMR");
									end
								else
									return CalcStat("VMHigh",L,"Finesse");
								end
							elseif SN > "VSINHEALL" then
								if SN < "VSMORALEM" then
									if SN < "VSMIGHTL" then
										if SN > "VSINHEALM" then
											if SN == "VSMIGHTH" then
												return CalcStat("VMHigh",L,"Might");
											else
												return 0;
											end
										elseif SN == "VSINHEALM" then
											return CalcStat("VMMedium",L,"InHeal");
										else
											return 0;
										end
									elseif SN > "VSMIGHTL" then
										if SN < "VSMORALEH" then
											if SN == "VSMIGHTM" then
												return CalcStat("VMMedium",L,"Might");
											else
												return 0;
											end
										elseif SN > "VSMORALEH" then
											if SN == "VSMORALEL" then
												return CalcStat("VMLow",L,"VMorale");
											else
												return 0;
											end
										else
											return CalcStat("VMHigh",L,"VMorale");
										end
									else
										return CalcStat("VMLow",L,"Might");
									end
								elseif SN > "VSMORALEM" then
									if SN < "VSPHYMASM" then
										if SN > "VSPHYMASH" then
											if SN == "VSPHYMASL" then
												return CalcStat("VMLow",L,"PhyMas");
											else
												return 0;
											end
										elseif SN == "VSPHYMASH" then
											return CalcStat("VMHigh",L,"PhyMas");
										else
											return 0;
										end
									elseif SN > "VSPHYMASM" then
										if SN < "VSPHYMITL" then
											if SN == "VSPHYMITH" then
												return CalcStat("VMHigh",L,"PhyMit");
											else
												return 0;
											end
										elseif SN > "VSPHYMITL" then
											if SN == "VSPHYMITM" then
												return CalcStat("VMMedium",L,"PhyMit");
											else
												return 0;
											end
										else
											return CalcStat("VMLow",L,"PhyMit");
										end
									else
										return CalcStat("VMMedium",L,"PhyMas");
									end
								else
									return CalcStat("VMMedium",L,"VMorale");
								end
							else
								return CalcStat("VMLow",L,"InHeal");
							end
						else
							return CalcStat("VMMedium",L,"CritHit");
						end
					elseif SN > "VSPOWERH" then
						if SN < "WARDENCDARMOURTYPE" then
							if SN < "VSVITALITYM" then
								if SN < "VSTACMASL" then
									if SN < "VSRESISTH" then
										if SN > "VSPOWERL" then
											if SN == "VSPOWERM" then
												return CalcStat("VMMedium",L,"Power");
											else
												return 0;
											end
										elseif SN == "VSPOWERL" then
											return CalcStat("VMLow",L,"Power");
										else
											return 0;
										end
									elseif SN > "VSRESISTH" then
										if SN < "VSRESISTM" then
											if SN == "VSRESISTL" then
												return CalcStat("VMLow",L,"Resist");
											else
												return 0;
											end
										elseif SN > "VSRESISTM" then
											if SN == "VSTACMASH" then
												return CalcStat("VMHigh",L,"TacMas");
											else
												return 0;
											end
										else
											return CalcStat("VMMedium",L,"Resist");
										end
									else
										return CalcStat("VMHigh",L,"Resist");
									end
								elseif SN > "VSTACMASL" then
									if SN < "VSTACMITL" then
										if SN > "VSTACMASM" then
											if SN == "VSTACMITH" then
												return CalcStat("VMHigh",L,"TacMit");
											else
												return 0;
											end
										elseif SN == "VSTACMASM" then
											return CalcStat("VMMedium",L,"TacMas");
										else
											return 0;
										end
									elseif SN > "VSTACMITL" then
										if SN < "VSVITALITYH" then
											if SN == "VSTACMITM" then
												return CalcStat("VMMedium",L,"TacMit");
											else
												return 0;
											end
										elseif SN > "VSVITALITYH" then
											if SN == "VSVITALITYL" then
												return CalcStat("VMLow",L,"Vitality");
											else
												return 0;
											end
										else
											return CalcStat("VMHigh",L,"Vitality");
										end
									else
										return CalcStat("VMLow",L,"TacMit");
									end
								else
									return CalcStat("VMLow",L,"TacMas");
								end
							elseif SN > "VSVITALITYM" then
								if SN < "WARDENCDAGILITYTOCRITHIT" then
									if SN < "VSVPTACMAS" then
										if SN > "VSVPMORALE" then
											if SN == "VSVPPHYMAS" then
												return CalcStat("VMMasteryPsv",L);
											else
												return 0;
											end
										elseif SN == "VSVPMORALE" then
											return CalcStat("VMMoralePsv",L);
										else
											return 0;
										end
									elseif SN > "VSVPTACMAS" then
										if SN < "VSWILLL" then
											if SN == "VSWILLH" then
												return CalcStat("VMHigh",L,"Will");
											else
												return 0;
											end
										elseif SN > "VSWILLL" then
											if SN == "VSWILLM" then
												return CalcStat("VMMedium",L,"Will");
											else
												return 0;
											end
										else
											return CalcStat("VMLow",L,"Will");
										end
									else
										return CalcStat("VMMasteryPsv",L);
									end
								elseif SN > "WARDENCDAGILITYTOCRITHIT" then
									if SN < "WARDENCDAGILITYTOPHYMAS" then
										if SN > "WARDENCDAGILITYTOEVADE" then
											if SN == "WARDENCDAGILITYTOPARRY" then
												return 2;
											else
												return 0;
											end
										elseif SN == "WARDENCDAGILITYTOEVADE" then
											return 3;
										else
											return 0;
										end
									elseif SN > "WARDENCDAGILITYTOPHYMAS" then
										if SN < "WARDENCDARMOURTOPHYMIT" then
											if SN == "WARDENCDARMOURTOOFMIT" then
												return 0.2;
											else
												return 0;
											end
										elseif SN > "WARDENCDARMOURTOPHYMIT" then
											if SN == "WARDENCDARMOURTOTACMIT" then
												return 0.2;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 3;
									end
								else
									return 1;
								end
							else
								return CalcStat("VMMedium",L,"Vitality");
							end
						elseif SN > "WARDENCDARMOURTYPE" then
							if SN < "WARDENCDFATETOFINESSE" then
								if SN < "WARDENCDBASENCMR" then
									if SN < "WARDENCDBASEICMR" then
										if SN > "WARDENCDBASEAGILITY" then
											if SN == "WARDENCDBASEFATE" then
												return CalcStat("ClassBaseFate",L);
											else
												return 0;
											end
										elseif SN == "WARDENCDBASEAGILITY" then
											return CalcStat("ClassBaseAgilityH",L);
										else
											return 0;
										end
									elseif SN > "WARDENCDBASEICMR" then
										if SN < "WARDENCDBASEMIGHT" then
											if SN == "WARDENCDBASEICPR" then
												return CalcStat("ClassBaseICPR",L);
											else
												return 0;
											end
										elseif SN > "WARDENCDBASEMIGHT" then
											if SN == "WARDENCDBASEMORALE" then
												return CalcStat("ClassBaseMorale",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseMightM",L);
										end
									else
										return CalcStat("ClassBaseICMRH",L);
									end
								elseif SN > "WARDENCDBASENCMR" then
									if SN < "WARDENCDBASEVITALITY" then
										if SN > "WARDENCDBASENCPR" then
											if SN == "WARDENCDBASEPOWER" then
												return CalcStat("ClassBasePower",L);
											else
												return 0;
											end
										elseif SN == "WARDENCDBASENCPR" then
											return CalcStat("ClassBaseNCPR",L);
										else
											return 0;
										end
									elseif SN > "WARDENCDBASEVITALITY" then
										if SN < "WARDENCDCANBLOCK" then
											if SN == "WARDENCDBASEWILL" then
												return CalcStat("ClassBaseWillL",L);
											else
												return 0;
											end
										elseif SN > "WARDENCDCANBLOCK" then
											if SN == "WARDENCDFATETOCRITHIT" then
												return 1.5;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return CalcStat("ClassBaseVitality",L);
									end
								else
									return CalcStat("ClassBaseNCMRH",L);
								end
							elseif SN > "WARDENCDFATETOFINESSE" then
								if SN < "WARDENCDMIGHTTOBLOCK" then
									if SN < "WARDENCDFATETOPOWER" then
										if SN > "WARDENCDFATETOICMR" then
											if SN == "WARDENCDFATETONCPR" then
												return 0.05;
											else
												return 0;
											end
										elseif SN == "WARDENCDFATETOICMR" then
											return 0.025;
										else
											return 0;
										end
									elseif SN > "WARDENCDFATETOPOWER" then
										if SN < "WARDENCDFATETOTACMIT" then
											if SN == "WARDENCDFATETORESIST" then
												return 0.5;
											else
												return 0;
											end
										elseif SN > "WARDENCDFATETOTACMIT" then
											if SN == "WARDENCDHASPOWER" then
												return 1;
											else
												return 0;
											end
										else
											return 0.5;
										end
									else
										return 1;
									end
								elseif SN > "WARDENCDMIGHTTOBLOCK" then
									if SN < "WARDENCDTACMASTOOUTHEAL" then
										if SN < "WARDENCDMIGHTTOPHYMAS" then
											if SN == "WARDENCDMIGHTTOPARRY" then
												return 2;
											else
												return 0;
											end
										elseif SN > "WARDENCDMIGHTTOPHYMAS" then
											if SN == "WARDENCDPHYMITTOOFMIT" then
												return 1;
											else
												return 0;
											end
										else
											return 2;
										end
									elseif SN > "WARDENCDTACMASTOOUTHEAL" then
										if SN < "WARDENCDVITALITYTONCMR" then
											if SN == "WARDENCDVITALITYTOMORALE" then
												return 4.8;
											else
												return 0;
											end
										elseif SN > "WARDENCDVITALITYTONCMR" then
											if SN == "WARDENCDVITALITYTORESIST" then
												return 0.5;
											else
												return 0;
											end
										else
											return 0.12;
										end
									else
										return 1;
									end
								else
									return 3;
								end
							else
								return 0.5;
							end
						else
							return 2;
						end
					else
						return CalcStat("VMHigh",L,"Power");
					end
				else
					return RoundDblDown(StatLinInter("PntMPVitality","ItemProgVital","ProgBVitality","VitalityAdj",L,N,0));
				end
			elseif SN > "WARDENCDWILLTORESIST" then
				if SN < "WORTHTABY" then
					if SN < "WORTHTABBI" then
						if SN < "WORTHTABAJ" then
							if SN < "WORTHMPH" then
								if SN < "WORTHMPA" then
									if SN < "WARDINGLORETACMIT" then
										if SN > "WARDENCDWILLTOTACMIT" then
											if SN == "WARDINGLOREPHYMIT" then
												if Lm <= 105 then
													return CalcStat("Mitigation",L,1.6);
												elseif Lm <= 119 then
													return CalcStat("PhyMitT",L,1.2);
												elseif Lm <= 120 then
													return CalcStat("PhyMitT",L,1.6);
												elseif Lm <= 129 then
													return CalcStat("PhyMitT",L,1.2);
												elseif Lm <= 130 then
													return CalcStat("PhyMitT",L,1.6);
												elseif Lm <= 139 then
													return CalcStat("PhyMitT",L,1.2);
												else
													return CalcStat("PhyMitT",L,1.6);
												end
											else
												return 0;
											end
										elseif SN == "WARDENCDWILLTOTACMIT" then
											return 1;
										else
											return 0;
										end
									elseif SN > "WARDINGLORETACMIT" then
										if SN < "WILLT" then
											if SN == "WILL" then
												return CalcStat("Main",L,N);
											else
												return 0;
											end
										elseif SN > "WILLT" then
											if SN == "WORTHEXT" then
												if Lm <= 360 then
													return RoundDbl(CalcStat("StatC",L,C));
												elseif Lm <= 501 then
													return RoundDbl(CalcStat("StatC",L-1,C));
												else
													return 0;
												end
											else
												return 0;
											end
										else
											return CalcStat("MainT",L,N);
										end
									else
										if Lm <= 105 then
											return CalcStat("Mitigation",L,1.6);
										elseif Lm <= 119 then
											return CalcStat("TacMitT",L,1.2);
										elseif Lm <= 120 then
											return CalcStat("TacMitT",L,1.6);
										elseif Lm <= 129 then
											return CalcStat("TacMitT",L,1.2);
										elseif Lm <= 130 then
											return CalcStat("TacMitT",L,1.6);
										elseif Lm <= 139 then
											return CalcStat("TacMitT",L,1.2);
										else
											return CalcStat("TacMitT",L,1.6);
										end
									end
								elseif SN > "WORTHMPA" then
									if SN < "WORTHMPD" then
										if SN > "WORTHMPB" then
											if SN == "WORTHMPC" then
												if Lm <= 0 then
													return 0;
												else
													return 1;
												end
											else
												return 0;
											end
										elseif SN == "WORTHMPB" then
											if Lm <= 0 then
												return 0;
											else
												return DataTableValue({1,1.2,2,3,4},L);
											end
										else
											return 0;
										end
									elseif SN > "WORTHMPD" then
										if SN < "WORTHMPF" then
											if SN == "WORTHMPE" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({1,1.1,1.15,1.2,1.25},L);
												end
											else
												return 0;
											end
										elseif SN > "WORTHMPF" then
											if SN == "WORTHMPG" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({1,2,3,4,5},L);
												end
											else
												return 0;
											end
										else
											if Lm <= 0 then
												return 0;
											else
												return DataTableValue({0.5,1,1.25,1.5,2},L);
											end
										end
									else
										if Lm <= 0 then
											return 0;
										else
											return DataTableValue({1,1,1,2,3},L);
										end
									end
								else
									if Lm <= 0 then
										return 0;
									else
										return DataTableValue({1,1.1,1.15,1.2,1.3},L);
									end
								end
							elseif SN > "WORTHMPH" then
								if SN < "WORTHTABAC" then
									if SN < "WORTHMPK" then
										if SN > "WORTHMPI" then
											if SN == "WORTHMPJ" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({1,1,1,1,5},L);
												end
											else
												return 0;
											end
										elseif SN == "WORTHMPI" then
											if Lm <= 0 then
												return 0;
											else
												return DataTableValue({1,2,2.5,3,10},L);
											end
										else
											return 0;
										end
									elseif SN > "WORTHMPK" then
										if SN < "WORTHTABAA" then
											if SN == "WORTHTABA" then
												if Lm <= 1 then
													return 1;
												else
													return CalcStat("WorthTabAF",L);
												end
											else
												return 0;
											end
										elseif SN > "WORTHTABAA" then
											if SN == "WORTHTABAB" then
												return 7*L+50;
											else
												return 0;
											end
										else
											return CalcStat("WorthTabD",L)+20;
										end
									else
										if Lm <= 0 then
											return 0;
										else
											return DataTableValue({1,1.2,1.3,1.35,1.4},L);
										end
									end
								elseif SN > "WORTHTABAC" then
									if SN < "WORTHTABAF" then
										if SN > "WORTHTABAD" then
											if SN == "WORTHTABAE" then
												return CalcStat("WorthTabE",L);
											else
												return 0;
											end
										elseif SN == "WORTHTABAD" then
											return 7*L+100;
										else
											return 0;
										end
									elseif SN > "WORTHTABAF" then
										if SN < "WORTHTABAH" then
											if SN == "WORTHTABAG" then
												return CalcStat("WorthTabAF",L)+25;
											else
												return 0;
											end
										elseif SN > "WORTHTABAH" then
											if SN == "WORTHTABAI" then
												if Lm <= 16 then
													return 10.73*L+54.3;
												elseif Lm <= 34 then
													return 10.735*L+54;
												elseif Lm <= 35 then
													return 429;
												elseif Lm <= 41 then
													return 10.65*L+57.1;
												elseif Lm <= 49 then
													return 10.7*L+55.3;
												else
													return 9*L+141;
												end
											else
												return 0;
											end
										else
											if Lm <= 49 then
												return CalcStat("WorthTabALBase",L+7)-66;
											else
												return 9*L+72;
											end
										end
									else
										if Lm <= 49 then
											return 2.5*L;
										else
											return 3*L-25;
										end
									end
								else
									return CalcStat("WorthTabE",L)-30;
								end
							else
								if Lm <= 0 then
									return 0;
								else
									return DataTableValue({1,1.2,1.8,3.2,5},L);
								end
							end
						elseif SN > "WORTHTABAJ" then
							if SN < "WORTHTABAW" then
								if SN < "WORTHTABAP" then
									if SN < "WORTHTABALBASE" then
										if SN > "WORTHTABAK" then
											if SN == "WORTHTABAL" then
												if Lm <= 49 then
													return CalcStat("WorthTabALBase",L);
												else
													return 9*L+69;
												end
											else
												return 0;
											end
										elseif SN == "WORTHTABAK" then
											if Lm <= 49 then
												return CalcStat("WorthTabALBase",L+1);
											else
												return 9*L+78;
											end
										else
											return 0;
										end
									elseif SN > "WORTHTABALBASE" then
										if SN < "WORTHTABAN" then
											if SN == "WORTHTABAM" then
												return CalcStat("WorthTabB",L);
											else
												return 0;
											end
										elseif SN > "WORTHTABAN" then
											if SN == "WORTHTABAO" then
												return CalcStat("WorthTabE",L);
											else
												return 0;
											end
										else
											if Lm <= 50 then
												return 8.25*L+27.75;
											else
												return 8*L+40;
											end
										end
									else
										return 9.86*L+23.51+RoundDbl(L*0.1+0.3)*0.4;
									end
								elseif SN > "WORTHTABAP" then
									if SN < "WORTHTABAS" then
										if SN > "WORTHTABAQ" then
											if SN == "WORTHTABAR" then
												return CalcStat("WorthTabD",L)+37.5;
											else
												return 0;
											end
										elseif SN == "WORTHTABAQ" then
											if Lm <= 10 then
												return 7.5*L+60;
											elseif Lm <= 49 then
												return 5.5*L+80;
											else
												return 6*L+55;
											end
										else
											return 0;
										end
									elseif SN > "WORTHTABAS" then
										if SN < "WORTHTABAU" then
											if SN == "WORTHTABAT" then
												if Lm <= 4 then
													return RoundDbl(3*L+20,-1);
												elseif Lm <= 7 then
													return 60;
												elseif Lm <= 10 then
													return 100;
												elseif Lm <= 13 then
													return 180;
												elseif Lm <= 16 then
													return 270;
												elseif Lm <= 19 then
													return 310;
												elseif Lm <= 40 then
													return RoundDbl(3.2*L+274,-1);
												elseif Lm <= 43 then
													return 400;
												elseif Lm <= 53 then
													return RoundDbl(1.9*L+334,-1);
												elseif Lm <= 56 then
													return 460;
												elseif Lm <= 59 then
													return 480;
												else
													return 20*L-680;
												end
											else
												return 0;
											end
										elseif SN > "WORTHTABAU" then
											if SN == "WORTHTABAV" then
												if Lm <= 4 then
													return RoundDbl(3*L+20,-1);
												elseif Lm <= 7 then
													return 70;
												elseif Lm <= 10 then
													return 140;
												elseif Lm <= 13 then
													return 270;
												elseif Lm <= 16 then
													return 400;
												elseif Lm <= 19 then
													return 460;
												elseif Lm <= 25 then
													return RoundDbl(3*L+447.5,-1);
												elseif Lm <= 46 then
													return RoundDbl(3.2*L+455,-1);
												elseif Lm <= 50 then
													return 620;
												else
													return 20*L-360;
												end
											else
												return 0;
											end
										else
											return CalcStat("WorthTabBN",L)*1.25;
										end
									else
										return CalcStat("WorthTabU",L);
									end
								else
									return CalcStat("WorthTabAQ",L);
								end
							elseif SN > "WORTHTABAW" then
								if SN < "WORTHTABBB" then
									if SN < "WORTHTABAZ" then
										if SN > "WORTHTABAX" then
											if SN == "WORTHTABAY" then
												return 5*L;
											else
												return 0;
											end
										elseif SN == "WORTHTABAX" then
											if Lm <= 16 then
												return 10.75*L+54;
											elseif Lm <= 34 then
												return 10.75*L+53.7;
											elseif Lm <= 35 then
												return 429;
											elseif Lm <= 41 then
												return 10.65*L+57.1;
											elseif Lm <= 49 then
												return 10.68*L+56.25;
											else
												return 12*L-9;
											end
										else
											return 0;
										end
									elseif SN > "WORTHTABAZ" then
										if SN < "WORTHTABB" then
											if SN == "WORTHTABAZBASE" then
												if Lm <= 44 then
													return RoundDbl(0.0525*L+0.7);
												elseif Lm <= 65 then
													return RoundDbl(0.19*L-4.85);
												elseif Lm <= 81 then
													return RoundDbl(0.19*L+28.15);
												else
													return RoundDbl(0.19*L+28.15+RoundDbl(L*0.05-4.55)*0.2);
												end
											else
												return 0;
											end
										elseif SN > "WORTHTABB" then
											if SN == "WORTHTABBA" then
												return 5*L+385;
											else
												return 0;
											end
										else
											if Lm <= 50 then
												return 8.25*L+22.25;
											else
												return 8*L+35;
											end
										end
									else
										if Lm <= 10 then
											return CalcStat("WorthTabAZBase",L)*62.5;
										elseif Lm <= 65 then
											return CalcStat("WorthTabAZBase",L)*125;
										else
											return CalcStat("WorthTabAZBase",L)*25;
										end
									end
								elseif SN > "WORTHTABBB" then
									if SN < "WORTHTABBE" then
										if SN > "WORTHTABBC" then
											if SN == "WORTHTABBD" then
												return CalcStat("WorthTabK",L)+1;
											else
												return 0;
											end
										elseif SN == "WORTHTABBC" then
											if Lm <= 34 then
												return RoundDbl(L/15+0.8)*3750+1250;
											elseif Lm <= 35 then
												return 100000;
											else
												return 125500;
											end
										else
											return 0;
										end
									elseif SN > "WORTHTABBE" then
										if SN < "WORTHTABBG" then
											if SN == "WORTHTABBF" then
												return CalcStat("WorthTabAV",L)+20;
											else
												return 0;
											end
										elseif SN > "WORTHTABBG" then
											if SN == "WORTHTABBH" then
												if Lm <= 80 then
													return RoundDbl(0.05*L-0.05)+RoundDbl(0.05*L+0.45);
												else
													return RoundDbl(0.05*L-0.1)+RoundDbl(0.05*L+0.45);
												end
											else
												return 0;
											end
										else
											if Lm <= 7 then
												return RoundDbl(0.2*L-0.4)*2+1;
											elseif Lm <= 16 then
												return RoundDbl(L/3-2)*6;
											elseif Lm <= 22 then
												return RoundDbl(L/3+1)*3;
											elseif Lm <= 25 then
												return RoundDbl(L/3+1)*3-2;
											elseif Lm <= 80 then
												return RoundDbl(L/3+18)-2;
											else
												return L-37;
											end
										end
									else
										if Lm <= 1 then
											return 60;
										elseif Lm <= 7 then
											return RoundDbl(L/3)*40+30;
										elseif Lm <= 13 then
											return RoundDbl(L/3)*130-210;
										elseif Lm <= 19 then
											return RoundDbl(L/3)*60+140;
										elseif Lm <= 25 then
											return RoundDbl(L/3)*10+480;
										elseif Lm <= 46 then
											return RoundDbl(L/3)*10+490;
										elseif Lm <= 50 then
											return 660;
										else
											return 20*L-320;
										end
									end
								else
									if Lm <= 1 then
										return 40;
									elseif Lm <= 10 then
										return RoundDbl(0.35*L-0.1)*20;
									elseif Lm <= 13 then
										return 100;
									elseif Lm <= 19 then
										return RoundDbl(0.35*L+1.6)*20;
									elseif Lm <= 25 then
										return RoundDbl(2*L+150,-1);
									elseif Lm <= 40 then
										return RoundDbl(2*L+164,-1);
									elseif Lm <= 49 then
										return RoundDbl(L+201,-1);
									else
										return 20*L-740;
									end
								end
							else
								if Lm <= 1 then
									return 50;
								elseif Lm <= 49 then
									return 2.5*L+102.5;
								elseif Lm <= 80 then
									return 3*L+78;
								elseif Lm <= 120 then
									return 2.97*L+79.5;
								else
									return 3*L+75;
								end
							end
						else
							return CalcStat("WorthTabAH",L);
						end
					elseif SN > "WORTHTABBI" then
						if SN < "WORTHTABCJ" then
							if SN < "WORTHTABBW" then
								if SN < "WORTHTABBP" then
									if SN < "WORTHTABBL" then
										if SN > "WORTHTABBJ" then
											if SN == "WORTHTABBK" then
												if Lm <= 10 then
													return RoundDbl(0.1*L+0.45);
												else
													return RoundDbl(0.1*L-0.6)*2;
												end
											else
												return 0;
											end
										elseif SN == "WORTHTABBJ" then
											return RoundDbl(0.1*L+0.45)*3;
										else
											return 0;
										end
									elseif SN > "WORTHTABBL" then
										if SN < "WORTHTABBN" then
											if SN == "WORTHTABBM" then
												if Lm <= 1 then
													return 10;
												else
													return 20*L-20;
												end
											else
												return 0;
											end
										elseif SN > "WORTHTABBN" then
											if SN == "WORTHTABBO" then
												if Lm <= 10 then
													return 10;
												elseif Lm <= 40 then
													return RoundDbl(0.1*L-0.55)*50-25;
												elseif Lm <= 80 then
													return RoundDbl(0.1*L-0.55)*50;
												else
													return RoundDbl(0.1*L-0.55)*25+175;
												end
											else
												return 0;
											end
										else
											if Lm <= 1 then
												return 400;
											else
												return 20*L+800;
											end
										end
									else
										if Lm <= 35 then
											return 126600;
										elseif Lm <= 46 then
											return 150600;
										elseif Lm <= 47 then
											return 180720;
										else
											return 150600;
										end
									end
								elseif SN > "WORTHTABBP" then
									if SN < "WORTHTABBS" then
										if SN > "WORTHTABBQ" then
											if SN == "WORTHTABBR" then
												if Lm <= 49 then
													return CalcStat("WorthTabBQ",L)*2;
												else
													return 4*L+250;
												end
											else
												return 0;
											end
										elseif SN == "WORTHTABBQ" then
											if Lm <= 1 then
												return 50;
											elseif Lm <= 49 then
												return RoundDbl(2.5*L+100);
											else
												return 3*L+75;
											end
										else
											return 0;
										end
									elseif SN > "WORTHTABBS" then
										if SN < "WORTHTABBU" then
											if SN == "WORTHTABBT" then
												return 62500;
											else
												return 0;
											end
										elseif SN > "WORTHTABBU" then
											if SN == "WORTHTABBV" then
												if Lm <= 29 then
													return 1250;
												else
													return RoundDbl(0.05*L-1)*2500;
												end
											else
												return 0;
											end
										else
											if Lm <= 80 then
												return 20*L+300;
											else
												return 10*L+1100;
											end
										end
									else
										return CalcStat("WorthTabBR",L)*2;
									end
								else
									if Lm <= 10 then
										return 156.25*L+1562.5;
									elseif Lm <= 20 then
										return 312.5*L;
									elseif Lm <= 30 then
										return 625*L-6250;
									elseif Lm <= 40 then
										return 1250*L-25000;
									else
										return 2500*L-75000;
									end
								end
							elseif SN > "WORTHTABBW" then
								if SN < "WORTHTABCC" then
									if SN < "WORTHTABBZ" then
										if SN > "WORTHTABBX" then
											if SN == "WORTHTABBY" then
												return 90000;
											else
												return 0;
											end
										elseif SN == "WORTHTABBX" then
											if Lm <= 10 then
												return RoundDbl(0.1*L+0.45)*3;
											elseif Lm <= 140 then
												return RoundDbl(0.1*L-0.6)*6;
											else
												return RoundDbl(0.1*L+0.4)*4+22;
											end
										else
											return 0;
										end
									elseif SN > "WORTHTABBZ" then
										if SN < "WORTHTABCA" then
											if SN == "WORTHTABC" then
												return CalcStat("WorthTabD",L)+20;
											else
												return 0;
											end
										elseif SN > "WORTHTABCA" then
											if SN == "WORTHTABCB" then
												return 0.1;
											else
												return 0;
											end
										else
											if Lm <= 10 then
												return 7.25*L+72.25;
											elseif Lm <= 20 then
												return 14.49*L+0.11;
											elseif Lm <= 30 then
												return 29*L-290;
											elseif Lm <= 35 then
												return 57.99*L-1160.16;
											elseif Lm <= 40 then
												return 57.99*L-1160.12;
											elseif Lm <= 80 then
												return 115.94*L-3478.2;
											else
												return 116*L-3483;
											end
										end
									else
										if Lm <= 10 then
											return 27.42*L+273.1;
										elseif Lm <= 20 then
											return 54.69*L;
										elseif Lm <= 30 then
											return 109.35*L-1093;
										elseif Lm <= 40 then
											return 218.75*L-4375;
										elseif Lm <= 80 then
											return 437.5*L-13125;
										else
											return 437*L-13085;
										end
									end
								elseif SN > "WORTHTABCC" then
									if SN < "WORTHTABCF" then
										if SN > "WORTHTABCD" then
											if SN == "WORTHTABCE" then
												return CalcStat("WorthTabBN",L)*10;
											else
												return 0;
											end
										elseif SN == "WORTHTABCD" then
											return CalcStat("WorthTabBK",L)*25;
										else
											return 0;
										end
									elseif SN > "WORTHTABCF" then
										if SN < "WORTHTABCH" then
											if SN == "WORTHTABCG" then
												return 20;
											else
												return 0;
											end
										elseif SN > "WORTHTABCH" then
											if SN == "WORTHTABCI" then
												return 40000;
											else
												return 0;
											end
										else
											return 50000;
										end
									else
										if Lm <= 20 then
											return RoundDbl(0.1*L-0.55)*600+200;
										elseif Lm <= 40 then
											return RoundDbl(0.1*L-0.55)*1000-500;
										elseif Lm <= 60 then
											return RoundDbl(0.1*L-0.55)*2000-4000;
										elseif Lm <= 70 then
											return 8500;
										elseif Lm <= 140 then
											return RoundDbl(0.1*L-0.55)*2500-7500;
										elseif Lm <= 200 then
											return RoundDbl(0.1*L-0.55)*3000-14000;
										else
											return RoundDbl(0.1*L-0.55)*2500-3500;
										end
									end
								else
									return CalcStat("WorthTabBK",L)*5;
								end
							else
								if Lm <= 1 then
									return 645;
								elseif Lm <= 9 then
									return 20*L+980;
								else
									return 50*L+920;
								end
							end
						elseif SN > "WORTHTABCJ" then
							if SN < "WORTHTABJ" then
								if SN < "WORTHTABCQ" then
									if SN < "WORTHTABCM" then
										if SN > "WORTHTABCK" then
											if SN == "WORTHTABCL" then
												return 12500;
											else
												return 0;
											end
										elseif SN == "WORTHTABCK" then
											return 15000;
										else
											return 0;
										end
									elseif SN > "WORTHTABCM" then
										if SN < "WORTHTABCO" then
											if SN == "WORTHTABCN" then
												return RoundDbl(0.1*L+0.5)*10000;
											else
												return 0;
											end
										elseif SN > "WORTHTABCO" then
											if SN == "WORTHTABCP" then
												return 1;
											else
												return 0;
											end
										else
											return CalcStat("WorthTabAB",L);
										end
									else
										return 7500;
									end
								elseif SN > "WORTHTABCQ" then
									if SN < "WORTHTABF" then
										if SN > "WORTHTABD" then
											if SN == "WORTHTABE" then
												if Lm <= 9 then
													return 30*L+50;
												else
													return 7*L+280;
												end
											else
												return 0;
											end
										elseif SN == "WORTHTABD" then
											if Lm <= 49 then
												return 7.5*L;
											else
												return 8*L-25;
											end
										else
											return 0;
										end
									elseif SN > "WORTHTABF" then
										if SN < "WORTHTABH" then
											if SN == "WORTHTABG" then
												if Lm <= 1 then
													return 50;
												else
													return CalcStat("WorthTabAF",L)+100;
												end
											else
												return 0;
											end
										elseif SN > "WORTHTABH" then
											if SN == "WORTHTABI" then
												if Lm <= 4 then
													return 1;
												elseif Lm <= 10 then
													return 0.7*L-1;
												elseif Lm <= 15 then
													return 1.4*L-7;
												elseif Lm <= 24 then
													return 1.25*L-5;
												elseif Lm <= 49 then
													return 2.5*L-35;
												else
													return 3*L-60;
												end
											else
												return 0;
											end
										else
											if Lm <= 2 then
												return L;
											elseif Lm <= 9 then
												return 1.48*L-2.85;
											elseif Lm <= 11 then
												return 2.5*L-13;
											elseif Lm <= 24 then
												return 2.5*L-10;
											else
												return 5*L-70;
											end
										end
									else
										return 0.1;
									end
								else
									if Lm <= 10 then
										return RoundDbl(0.3*L+0.2)*2;
									elseif Lm <= 16 then
										return RoundDbl(0.25*L-1)*6;
									elseif Lm <= 22 then
										return RoundDbl(0.3*L+5.5)*2;
									elseif Lm <= 50 then
										return RoundDbl((1/6)*L+8.75)*2;
									elseif Lm <= 80 then
										return RoundDbl((1/3)*L-11.5)*6;
									else
										return RoundDbl(0.25*L-4.6)*6;
									end
								end
							elseif SN > "WORTHTABJ" then
								if SN < "WORTHTABQ" then
									if SN < "WORTHTABM" then
										if SN > "WORTHTABK" then
											if SN == "WORTHTABL" then
												return CalcStat("WorthTabB",L)+19.25;
											else
												return 0;
											end
										elseif SN == "WORTHTABK" then
											if Lm <= 9 then
												return 12*L+24;
											else
												return 7*L+74;
											end
										else
											return 0;
										end
									elseif SN > "WORTHTABM" then
										if SN < "WORTHTABO" then
											if SN == "WORTHTABN" then
												return CalcStat("WorthTabAF",L)+25;
											else
												return 0;
											end
										elseif SN > "WORTHTABO" then
											if SN == "WORTHTABP" then
												return CalcStat("WorthTabE",L);
											else
												return 0;
											end
										else
											return CalcStat("WorthTabB",L)+11;
										end
									else
										if Lm <= 9 then
											return 10*L+11;
										elseif Lm <= 49 then
											return 4*L+65;
										else
											return 5*L+15;
										end
									end
								elseif SN > "WORTHTABQ" then
									if SN < "WORTHTABU" then
										if SN < "WORTHTABS" then
											if SN == "WORTHTABR" then
												if Lm <= 9 then
													return 12*L+30;
												else
													return 7*L+75;
												end
											else
												return 0;
											end
										elseif SN > "WORTHTABS" then
											if SN == "WORTHTABT" then
												if Lm <= 1 then
													return 54;
												elseif Lm <= 17 then
													return 10.73*L+43.57;
												elseif Lm <= 35 then
													return 10.735*L+43.265;
												elseif Lm <= 36 then
													return 429;
												elseif Lm <= 42 then
													return 10.65*L+46.45;
												elseif Lm <= 49 then
													return 10.7*L+44.6;
												else
													return 9*L+130;
												end
											else
												return 0;
											end
										else
											return CalcStat("WorthTabD",L)+17.5;
										end
									elseif SN > "WORTHTABU" then
										if SN < "WORTHTABW" then
											if SN == "WORTHTABV" then
												return CalcStat("WorthTabB",L)-2.75;
											else
												return 0;
											end
										elseif SN > "WORTHTABW" then
											if SN == "WORTHTABX" then
												return CalcStat("WorthTabAF",L)+75;
											else
												return 0;
											end
										else
											return CalcStat("WorthTabD",L)+25;
										end
									else
										if Lm <= 10 then
											return 9*L+20;
										elseif Lm <= 49 then
											return 5.5*L+55;
										else
											return 6*L+30;
										end
									end
								else
									return 7*L+25;
								end
							else
								if Lm <= 4 then
									return 2.5*L+7.5;
								elseif Lm <= 5 then
									return 23;
								else
									return 4*L;
								end
							end
						else
							return 20000;
						end
					else
						return CalcStat("WorthTabG",L)-25;
					end
				elseif SN > "WORTHTABY" then
					if SN < "WORTHVALCA" then
						if SN < "WORTHVALAZ" then
							if SN < "WORTHVALAL" then
								if SN < "WORTHVALAE" then
									if SN < "WORTHVALAA" then
										if SN > "WORTHTABZ" then
											if SN == "WORTHVALA" then
												return RoundDbl(CalcStat("WorthMpA",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabA"));
											else
												return 0;
											end
										elseif SN == "WORTHTABZ" then
											return CalcStat("WorthTabR",L)-15;
										else
											return 0;
										end
									elseif SN > "WORTHVALAA" then
										if SN < "WORTHVALAC" then
											if SN == "WORTHVALAB" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAB"));
											else
												return 0;
											end
										elseif SN > "WORTHVALAC" then
											if SN == "WORTHVALAD" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAD"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAC"));
										end
									else
										return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAA"));
									end
								elseif SN > "WORTHVALAE" then
									if SN < "WORTHVALAH" then
										if SN > "WORTHVALAF" then
											if SN == "WORTHVALAG" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAG"));
											else
												return 0;
											end
										elseif SN == "WORTHVALAF" then
											return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAF"));
										else
											return 0;
										end
									elseif SN > "WORTHVALAH" then
										if SN < "WORTHVALAJ" then
											if SN == "WORTHVALAI" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAI"));
											else
												return 0;
											end
										elseif SN > "WORTHVALAJ" then
											if SN == "WORTHVALAK" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAK"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAJ"));
										end
									else
										return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAH"));
									end
								else
									return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAE"));
								end
							elseif SN > "WORTHVALAL" then
								if SN < "WORTHVALAS" then
									if SN < "WORTHVALAO" then
										if SN > "WORTHVALAM" then
											if SN == "WORTHVALAN" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAN"));
											else
												return 0;
											end
										elseif SN == "WORTHVALAM" then
											return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAM"));
										else
											return 0;
										end
									elseif SN > "WORTHVALAO" then
										if SN < "WORTHVALAQ" then
											if SN == "WORTHVALAP" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAP"));
											else
												return 0;
											end
										elseif SN > "WORTHVALAQ" then
											if SN == "WORTHVALAR" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAR"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAQ"));
										end
									else
										return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAO"));
									end
								elseif SN > "WORTHVALAS" then
									if SN < "WORTHVALAV" then
										if SN > "WORTHVALAT" then
											if SN == "WORTHVALAU" then
												return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAU"));
											else
												return 0;
											end
										elseif SN == "WORTHVALAT" then
											return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAT"));
										else
											return 0;
										end
									elseif SN > "WORTHVALAV" then
										if SN < "WORTHVALAX" then
											if SN == "WORTHVALAW" then
												return RoundDbl(CalcStat("WorthMpA",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAW"));
											else
												return 0;
											end
										elseif SN > "WORTHVALAX" then
											if SN == "WORTHVALAY" then
												return RoundDbl(CalcStat("WorthMpA",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAY"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAX"));
										end
									else
										return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAV"));
									end
								else
									return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAS"));
								end
							else
								return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAL"));
							end
						elseif SN > "WORTHVALAZ" then
							if SN < "WORTHVALBM" then
								if SN < "WORTHVALBF" then
									if SN < "WORTHVALBB" then
										if SN > "WORTHVALB" then
											if SN == "WORTHVALBA" then
												return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBA"));
											else
												return 0;
											end
										elseif SN == "WORTHVALB" then
											return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabB"));
										else
											return 0;
										end
									elseif SN > "WORTHVALBB" then
										if SN < "WORTHVALBD" then
											if SN == "WORTHVALBC" then
												return RoundDbl(CalcStat("WorthMpE",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBC"));
											else
												return 0;
											end
										elseif SN > "WORTHVALBD" then
											if SN == "WORTHVALBE" then
												return RoundDbl(CalcStat("WorthMpJ",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBE"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBD"));
										end
									else
										return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBB"));
									end
								elseif SN > "WORTHVALBF" then
									if SN < "WORTHVALBI" then
										if SN > "WORTHVALBG" then
											if SN == "WORTHVALBH" then
												return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBH"));
											else
												return 0;
											end
										elseif SN == "WORTHVALBG" then
											return RoundDbl(CalcStat("WorthMpA",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBG"));
										else
											return 0;
										end
									elseif SN > "WORTHVALBI" then
										if SN < "WORTHVALBK" then
											if SN == "WORTHVALBJ" then
												return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBJ"));
											else
												return 0;
											end
										elseif SN > "WORTHVALBK" then
											if SN == "WORTHVALBL" then
												return RoundDbl(CalcStat("WorthMpE",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBL"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBK"));
										end
									else
										return RoundDbl(CalcStat("WorthMpA",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBI"));
									end
								else
									return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBF"));
								end
							elseif SN > "WORTHVALBM" then
								if SN < "WORTHVALBT" then
									if SN < "WORTHVALBP" then
										if SN > "WORTHVALBN" then
											if SN == "WORTHVALBO" then
												return RoundDbl(CalcStat("WorthMpG",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBO"));
											else
												return 0;
											end
										elseif SN == "WORTHVALBN" then
											return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBN"));
										else
											return 0;
										end
									elseif SN > "WORTHVALBP" then
										if SN < "WORTHVALBR" then
											if SN == "WORTHVALBQ" then
												return RoundDbl(CalcStat("WorthMpE",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBQ"));
											else
												return 0;
											end
										elseif SN > "WORTHVALBR" then
											if SN == "WORTHVALBS" then
												return RoundDbl(CalcStat("WorthMpE",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBS"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpE",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBR"));
										end
									else
										return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBP"));
									end
								elseif SN > "WORTHVALBT" then
									if SN < "WORTHVALBX" then
										if SN < "WORTHVALBV" then
											if SN == "WORTHVALBU" then
												return RoundDbl(CalcStat("WorthMpF",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBU"));
											else
												return 0;
											end
										elseif SN > "WORTHVALBV" then
											if SN == "WORTHVALBW" then
												return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBW"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpD",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBV"));
										end
									elseif SN > "WORTHVALBX" then
										if SN < "WORTHVALBZ" then
											if SN == "WORTHVALBY" then
												return RoundDbl(CalcStat("WorthMpE",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBY"));
											else
												return 0;
											end
										elseif SN > "WORTHVALBZ" then
											if SN == "WORTHVALC" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabC"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBZ"));
										end
									else
										return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBX"));
									end
								else
									return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBT"));
								end
							else
								return RoundDbl(CalcStat("WorthMpA",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBM"));
							end
						else
							return RoundDbl(CalcStat("WorthMpA",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAZ"));
						end
					elseif SN > "WORTHVALCA" then
						if SN < "WORTHVALO" then
							if SN < "WORTHVALCO" then
								if SN < "WORTHVALCH" then
									if SN < "WORTHVALCD" then
										if SN > "WORTHVALCB" then
											if SN == "WORTHVALCC" then
												return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabCC"));
											else
												return 0;
											end
										elseif SN == "WORTHVALCB" then
											return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabCB"));
										else
											return 0;
										end
									elseif SN > "WORTHVALCD" then
										if SN < "WORTHVALCF" then
											if SN == "WORTHVALCE" then
												return RoundDbl(CalcStat("WorthMpH",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabCE"));
											else
												return 0;
											end
										elseif SN > "WORTHVALCF" then
											if SN == "WORTHVALCG" then
												return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabCG"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpA",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabCF"));
										end
									else
										return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabCD"));
									end
								elseif SN > "WORTHVALCH" then
									if SN < "WORTHVALCK" then
										if SN > "WORTHVALCI" then
											if SN == "WORTHVALCJ" then
												return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabCJ"));
											else
												return 0;
											end
										elseif SN == "WORTHVALCI" then
											return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabCI"));
										else
											return 0;
										end
									elseif SN > "WORTHVALCK" then
										if SN < "WORTHVALCM" then
											if SN == "WORTHVALCL" then
												return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabCL"));
											else
												return 0;
											end
										elseif SN > "WORTHVALCM" then
											if SN == "WORTHVALCN" then
												return RoundDbl(CalcStat("WorthMpI",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabCN"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabCM"));
										end
									else
										return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabCK"));
									end
								else
									return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabCH"));
								end
							elseif SN > "WORTHVALCO" then
								if SN < "WORTHVALH" then
									if SN < "WORTHVALD" then
										if SN > "WORTHVALCP" then
											if SN == "WORTHVALCQ" then
												return RoundDbl(CalcStat("WorthMpK",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabCQ"));
											else
												return 0;
											end
										elseif SN == "WORTHVALCP" then
											return RoundDbl(CalcStat("WorthMpA",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabCP"));
										else
											return 0;
										end
									elseif SN > "WORTHVALD" then
										if SN < "WORTHVALF" then
											if SN == "WORTHVALE" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabE"));
											else
												return 0;
											end
										elseif SN > "WORTHVALF" then
											if SN == "WORTHVALG" then
												return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabG"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabF"));
										end
									else
										return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabD"));
									end
								elseif SN > "WORTHVALH" then
									if SN < "WORTHVALK" then
										if SN > "WORTHVALI" then
											if SN == "WORTHVALJ" then
												return RoundDbl(CalcStat("WorthMpA",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabJ"));
											else
												return 0;
											end
										elseif SN == "WORTHVALI" then
											return RoundDbl(CalcStat("WorthMpA",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabI"));
										else
											return 0;
										end
									elseif SN > "WORTHVALK" then
										if SN < "WORTHVALM" then
											if SN == "WORTHVALL" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabL"));
											else
												return 0;
											end
										elseif SN > "WORTHVALM" then
											if SN == "WORTHVALN" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabN"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabM"));
										end
									else
										return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabK"));
									end
								else
									return RoundDbl(CalcStat("WorthMpA",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabH"));
								end
							else
								return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabCO"));
							end
						elseif SN > "WORTHVALO" then
							if SN < "WPNDMGMIN" then
								if SN < "WORTHVALV" then
									if SN < "WORTHVALR" then
										if SN > "WORTHVALP" then
											if SN == "WORTHVALQ" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabQ"));
											else
												return 0;
											end
										elseif SN == "WORTHVALP" then
											return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabP"));
										else
											return 0;
										end
									elseif SN > "WORTHVALR" then
										if SN < "WORTHVALT" then
											if SN == "WORTHVALS" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabS"));
											else
												return 0;
											end
										elseif SN > "WORTHVALT" then
											if SN == "WORTHVALU" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabU"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabT"));
										end
									else
										return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabR"));
									end
								elseif SN > "WORTHVALV" then
									if SN < "WORTHVALY" then
										if SN > "WORTHVALW" then
											if SN == "WORTHVALX" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabX"));
											else
												return 0;
											end
										elseif SN == "WORTHVALW" then
											return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabW"));
										else
											return 0;
										end
									elseif SN > "WORTHVALY" then
										if SN < "WOUNDRESISTT" then
											if SN == "WORTHVALZ" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabZ"));
											else
												return 0;
											end
										elseif SN > "WOUNDRESISTT" then
											if SN == "WPNDMGMAX" then
												return (2/(2-CalcStat("WpnDPSVarianceType",WpnCodeIndex(C,2))))*CalcStat("CombatBase",L,C);
											else
												return 0;
											end
										else
											return CalcStat("ResistAddT",L,N);
										end
									else
										return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabY"));
									end
								else
									return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabV"));
								end
							elseif SN > "WPNDMGMIN" then
								if SN < "WRDPHYMAS" then
									if SN < "WRDBATSTRIKESCRITDEF" then
										if SN > "WPNDPS" then
											if SN == "WPNDPSVARIANCETYPE" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({0.25,0.25,0.25},L);
												end
											else
												return 0;
											end
										elseif SN == "WPNDPS" then
											return CalcStat("CombatBase",L,C);
										else
											return 0;
										end
									elseif SN > "WRDBATSTRIKESCRITDEF" then
										if SN < "WRDFINESSE" then
											if SN == "WRDCRITDEF" then
												return CalcStat("CritDefT",L,1.0);
											else
												return 0;
											end
										elseif SN > "WRDFINESSE" then
											if SN == "WRDIMPRBLADESPARRY" then
												return CalcStat("ParryT",L,2.8);
											else
												return 0;
											end
										else
											return CalcStat("FinesseT",L,CalcStat("Trait12345Choice",N)*0.4);
										end
									else
										return CalcStat("CritDefT",L,CalcStat("Trait123Choice",N)*0.4);
									end
								elseif SN > "WRDPHYMAS" then
									if SN < "WRDSSSHPIERCERBLOCK" then
										if SN < "WRDSHIELDMASBLOCK" then
											if SN == "WRDRECKLESSNCRITHIT" then
												return CalcStat("CritHitT",L,2.4);
											else
												return 0;
											end
										elseif SN > "WRDSHIELDMASBLOCK" then
											if SN == "WRDSHIELDTACTCRITDEF" then
												return CalcStat("CritDefT",L,2);
											else
												return 0;
											end
										else
											return CalcStat("BlockT",L,2.8);
										end
									elseif SN > "WRDSSSHPIERCERBLOCK" then
										if SN < "WRDSTDYOURGREVADE" then
											if SN == "WRDSTDYOURGRBLOCK" then
												return CalcStat("BlockT",L,CalcStat("Trait1234Choice",N)*0.4);
											else
												return 0;
											end
										elseif SN > "WRDSTDYOURGREVADE" then
											if SN == "WRDSTDYOURGRPARRY" then
												return CalcStat("ParryT",L,CalcStat("Trait1234Choice",N)*0.4);
											else
												return 0;
											end
										else
											return -CalcStat("EvadeT",L,CalcStat("Trait1234Choice",N)*0.4);
										end
									else
										return -CalcStat("BlockT",L,4);
									end
								else
									return CalcStat("PhyMasT",L,CalcStat("Trait12345Choice",N)*0.4);
								end
							else
								return ((2-2*CalcStat("WpnDPSVarianceType",WpnCodeIndex(C,2)))/(2-CalcStat("WpnDPSVarianceType",WpnCodeIndex(C,2))))*CalcStat("CombatBase",L,C);
							end
						else
							return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabO"));
						end
					else
						return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabCA"));
					end
				else
					return CalcStat("WorthTabD",L)+30;
				end
			else
				return 2;
			end
		else
			return -CalcStat("BPET",L,CalcStat("TpenChoice",N));
		end
	else
		return 1.5;
	end

end

-- Support functions for CalcStat. These consist of implementations of more complex calculation types, decode methods for parameter "C" and rounding/min/max/compare functions for floating point numbers.

-- ****************** Calculation Type support functions ******************

-- DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
-- DataTableValue: Takes a value from an array table.

function DataTableValue(vDataArray, dIndex)

	local lIndex = RoundDbl(dIndex);
	if lIndex <= 1 then
		return vDataArray[1];
	elseif lIndex > #vDataArray then
		return vDataArray[#vDataArray];
	else
		return vDataArray[lIndex];
	end

end

-- EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
-- ExpFmod: Exponential function based on percentage.
-- Common percentage values are around ~5.5% for between levels and ~20% jumps between level segments.

function ExpFmod(dVal, dLstart, dPlvl, dLvl, vDec, vAdd)

	local dRng = dLvl-dLstart+1;
	if dRng <= DblCalcDev then
		return dVal;
	else
		local dFac = 1+dPlvl/100;
		local dAdd;
		if vAdd == nil then
			dAdd = 0;
		else
			dAdd = vAdd;
		end
		if vDec == nil then
			local dFacExp = dFac^dRng;
			return dVal*dFacExp+dAdd*((dFacExp-1)/(dFac-1));
		else
			local dResult = dVal;
			local dLm = dLstart-DblCalcDev;
			while dLm <= dLvl do
				dResult = RoundDbl(dResult*dFac+dAdd,vDec);
				dLm = dLm+1;
			end
			return dResult;
		end
	end

end

-- PPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPP
-- CalcPercAB: Calculates the percentage out of a rating based on the AB formula.

function CalcPercAB(dA, dB, dPCap, dR)

	if dR <= DblCalcDev then
		return 0.0;
	else
		local dResult = dA/(1+dB/dR);
		if dResult >= dPCap-DblCalcDev then
			return dPCap;
		else
			return dResult;
		end
	end

end

-- RRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRR
-- CalcRatAB: Calculates the rating out of a percentage based on the AB formula.

function CalcRatAB(dA, dB, dCapR, dP)

	if dP <= DblCalcDev then
		return 0.0;
	else
		local dResult = dB/(dA/dP-1);
		if dResult >= dCapR-DblCalcDev then
			return dCapR;
		else
			return dResult;
		end
	end

end

-- SSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSS
-- StatLinInter: (Normalized) Stat Linear Interpolating

function StatLinInter(sPntMP, sProgScheme, sProgBase, sAdj, dLvl, vNorC, vRoundType)

	-- parameter processing
	local dN = 1;
	local sC = "";
	if type(vNorC) == "number" then
		dN = vNorC;
	elseif type(vNorC) == "string" then
		sC = vNorC;
	end

	local dRoundType = 0;
	if type(vRoundType) == "number" then
		dRoundType = vRoundType;
	end
	
	local dProgScheme = CalcStat(sProgScheme,dLvl);
	if type(dProgScheme) ~= "table" then
		return 0.0;
	end

	-- find level interval
	local dLvlMinus = dLvl-DblCalcDev;
	local iPointIndexHigh = 2;
	local iPointIndexMax = #dProgScheme[1];
	while iPointIndexHigh < iPointIndexMax do
		if dLvlMinus <= dProgScheme[1][iPointIndexHigh] then
			break;
		end
		iPointIndexHigh = iPointIndexHigh+1;
	end
	local iPointIndexLow = iPointIndexHigh-1;
		
	local dAccessLvlLow = dProgScheme[1][iPointIndexLow];
	local dAccessLvlHigh = dProgScheme[1][iPointIndexHigh];
	local dBaseLvlLow = dProgScheme[2][iPointIndexLow];
	local dBaseLvlHigh = dProgScheme[2][iPointIndexHigh];
	
	-- get values from base progression
	local dValLow = CalcStat(sProgBase,dBaseLvlLow,sC);
	local dValHigh = CalcStat(sProgBase,dBaseLvlHigh,sC);

	-- graph point multiplications
	if type(sPntMP) == "string" and sPntMP ~= "" then
		dValLow = dValLow*CalcStat(sPntMP,dAccessLvlLow,sC);
		dValHigh = dValHigh*CalcStat(sPntMP,dAccessLvlHigh,sC);
	end
	if type(sAdj) == "string" and sAdj ~= "" then
		dValLow = dValLow*CalcStat(sAdj,dAccessLvlLow,sC);
		dValHigh = dValHigh*CalcStat(sAdj,dAccessLvlHigh,sC);
	end
	dValLow = dValLow*dN;
	dValHigh = dValHigh*dN;

	-- graph point roundings
	if dRoundType == 0 then
		dValLow = RoundDblLotro(dValLow);
		dValHigh = RoundDblLotro(dValHigh);
	elseif dRoundType == 1 then
		if -1000.0 <= dValLow and dValLow <= 1000.0 then
			dValLow = RoundDblUp(dValLow,2);
		else
			dValLow = RoundDblLotro(dValLow);
		end
		if -1000.0 <= dValHigh and dValHigh <= 1000.0 then
			dValHigh = RoundDblUp(dValHigh,2);
		else
			dValHigh = RoundDblLotro(dValHigh);
		end
	elseif dRoundType == 2 then
		dValLow = RoundDblLotro(dValLow);
		if dValLow == -1 then
			dValLow = -2;
		end
		dValHigh = RoundDblLotro(dValHigh);
		if dValHigh == -1 then
			dValHigh = -2;
		end
	end

	-- return interpolated value from the calculated graph points
	return LinFmod(1,dValLow,dValHigh,dAccessLvlLow,dAccessLvlHigh,dLvl);

end

-- TTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTT
-- LinFmod: Linear line function between 2 points with some optional modifications.
-- Connects point (dLstart,dVal*dFstart) with (dLend,dVal*dFend).
-- Usually used with dVal=1 and dFstart/dFend containing unrelated points or dVal=# and dFstart/dFend containing multiplier factors.
-- Modification for in-between points on the line: rounding.

function LinFmod(dVal, dFstart, dFend, dLstart, dLend, dLvl, vDec)

	if dLstart-DblCalcDev <= dLvl and dLvl <= dLstart+DblCalcDev then
		return dVal*dFstart;
	elseif dLend-DblCalcDev <= dLvl and dLvl <= dLend+DblCalcDev then
		return dVal*dFend;
	elseif dLstart == dLend then
		return 0.0;
	elseif vDec == nil then
		return dVal*(dFstart*(dLend-dLvl)+(dLvl-dLstart)*dFend)/(dLend-dLstart);
	else
		return RoundDbl(dVal*(dFstart*(dLend-dLvl)+(dLvl-dLstart)*dFend)/(dLend-dLstart),vDec);
	end

end

-- ****************** Parameter "C" decode support functions ******************

-- ArmCodeIndex: returns a specified index from an Armour Code.
-- sACode string:
-- 1st position: H=heavy, M=medium, L=light
-- 2nd position: H=head, S=shoulders, CL=cloak/back, C=chest, G=gloves, L=leggings, B=boots, Sh=shield
-- 3rd position: W=white/common, Y=yellow/uncommon, P=purple/rare, T=teal/blue/incomparable, G=gold/legendary/epic
-- Note: no such thing exists as a heavy, medium or light cloak, so no H/M/L in cloak codes (cloaks go automatically in the M class since U23, although historically this was L)

function ArmCodeIndex(sACode, iI)

	if sACode == nil or iI == nil then
		return 0;
	end
	if type(sACode) ~= "string" or type(iI) ~= "number" then
		return 0;
	end

	local sArmCode = string.match(sACode,"(%a+)");
	if not sArmCode then
		return 0;
	end
	sArmCode = string.upper(sArmCode).."    ";
	
	local sArmCat = string.sub(sArmCode,1,1);
	local sArmType = string.sub(sArmCode,2,2);
	local sArmCol = string.sub(sArmCode,3,3);

	if sArmType == "S" and sArmCol == "H" then
		sArmType = "SH";
		sArmCol = string.sub(sArmCode,4,4);
	elseif sArmCat == "C" and sArmType == "L" then
		sArmCat = "M";
		sArmType = "CL";
	elseif sArmType then
		sArmType = " "..sArmType;
	end
	
	local result = 0;
	if iI == 1 then
		if sArmCat then
			result = string.find("HML",sArmCat);
		end
	elseif iI == 2 then
		if sArmType then
			result = string.find(" H SCL C G L BSH",sArmType);
			if result then
				result = (result+1)/2;
			end
		end
	elseif iI == 3 then
		if sArmCol then
			result = string.find("WYPTG",sArmCol);
		end
	end

	if result then
		return result;
	else
		return 0;
	end
	
end

-- QualityCodeIndex: returns a quality index from a Quality Code.
-- sQCode string: W=white/common, Y=yellow/uncommon, P=purple/rare, T=teal/blue/incomparable, G=gold/legendary/epic

function QualityCodeIndex(sQCode)

	if sQCode == nil then
		return 0;
	end
	if type(sQCode) ~= "string" then
		return 0;
	end

	local sQtyCode = string.match(sQCode,"(%a+)");
	if not sQtyCode then
		return 0;
	end
	sQtyCode = string.upper(sQtyCode).." ";

	local sQtyCol = string.sub(sQtyCode,1,1);
	
	local result = 0;
	if sQtyCol then
		result = string.find("WYPTG",sQtyCol);
	end
	
	if result then
		return result;
	else
		return 0;
	end
	
end

-- WpnCodeIndex: returns a specified index from a Weapon Code.
-- sWCode string:
-- 1st position: H=heavy, L=light
-- 2nd position: O=one-handed, T=two-handed, B=bow
-- 3rd position: W=white/common, Y=yellow/uncommon, P=purple/rare, T=teal/blue/incomparable, G=gold/legendary/epic

function WpnCodeIndex(sWCode, iI)

	if sWCode == nil or iI == nil then
		return 0;
	end
	if type(sWCode) ~= "string" or type(iI) ~= "number" then
		return 0;
	end

	local sWpnCode = string.match(sWCode,"(%a+)");
	if not sWpnCode then
		return 0;
	end
	sWpnCode = string.upper(sWpnCode).."   ";
	
	local sWpnCat = string.sub(sWpnCode,1,1);
	local sWpnType = string.sub(sWpnCode,2,2);
	local sWpnCol = string.sub(sWpnCode,3,3);

	local result = 0;
	if iI == 1 then
		if sWpnCat then
			result = string.find("HL",sWpnCat);
		end
	elseif iI == 2 then
		if sWpnType then
			result = string.find("OTB",sWpnType);
		end
	elseif iI == 3 then
		if sWpnCol then
			result = string.find("WYPTG",sWpnCol);
		end
	end

	if result then
		return result;
	else
		return 0;
	end
	
end

-- RomanRankDecode: converts a string with a Roman number in characters, to an integer number.
-- used for Legendary Item Title calculation.

local RomanCharsToValues = {["M"]=1000,["CM"]=900,["D"]=500,["CD"]=400,["C"]=100,["XC"]=90,["L"]=50,["XL"]=40,["X"]=10,["IX"]=9,["V"]=5,["IV"]=4,["I"]=1};

function RomanRankDecode(sNumber)

	if sNumber ~= nil and type(sNumber) == "string" and sNumber ~= "" then
		for sRomanRankChar, iRomanRankValue in pairs(RomanCharsToValues) do
			if string.sub(string.upper(sNumber),1,string.len(sRomanRankChar)) == sRomanRankChar then
				return iRomanRankValue+RomanRankDecode(string.sub(sNumber,string.len(sRomanRankChar)+1));
			end
		end
	end
	
	return 0;
	
end

-- ReverseCalc: tries to calculate back a calculation result to the original (integer) level.
-- Does not support N.

function ReverseCalc(sStat, dNum)

	if string.len(string.match(sStat,"([-%w]+)")) > 0 then
		local dNumMinus = dNum-DblCalcDev;
		local dNumPlus = dNum+DblCalcDev;
		local minlvl = 1;
		local maxlvl = 549;
		local devlvl = 3;
	
		local left = minlvl-1;
		local right = maxlvl;
		local middle;
		
		local count = minlvl;

		while right > left+1 and count <= maxlvl do
			count = count+1;
			middle = math.floor((left+right)/2);
			if CalcStat(sStat,middle) >= dNumMinus then
				right = middle;
			else
				left = middle;
			end
		end

		local mintest = right-devlvl;
		if mintest < minlvl then
			mintest = minlvl;
		end
		local maxtest = right+devlvl;
		if maxtest > maxlvl then
			maxtest = maxlvl;
		end

		local dFound;
	
		-- we check nearby in case the progression is not completely ascending/sorted.
		for test = mintest, maxtest do
			dFound = CalcStat(sStat,test);
			if dNumMinus <= dFound and dFound <= dNumPlus then
				return test;
			end
		end
	end

	return 0;

end

-- ****************** Misc. floating point support functions ******************

-- Misc. functions for floating point rounding.
-- 2nd parameter is number of decimals.

function RoundDbl(dNum, vDec)

	local dCorrection = 0.5+DblCalcDev;
	local iSign = 1;
	if dNum < 0 then
		iSign = -1;
	end
	if vDec == nil or (-DblCalcDev <= vDec and vDec <= DblCalcDev) then
		return iSign*math.floor(iSign*dNum+dCorrection);
	else
		local dFactor = 10^vDec;
		return iSign*math.floor(iSign*dNum*dFactor+dCorrection)/dFactor;
	end
	
end

function RoundDblDown(dNum, vDec)

	local dCorrection = DblCalcDev;
	local iSign = 1;
	if dNum < 0 then
		iSign = -1;
	end
	if vDec == nil or (-DblCalcDev <= vDec and vDec <= DblCalcDev) then
		return iSign*math.floor(iSign*dNum+dCorrection);
	else
		local dFactor = 10^vDec;
		return iSign*math.floor(iSign*dNum*dFactor+dCorrection)/dFactor;
	end
	
end

function RoundDblUp(dNum, vDec)

	local dCorrection = 1-DblCalcDev;
	local iSign = 1;
	if dNum < 0 then
		iSign = -1;
	end
	if vDec == nil or (-DblCalcDev <= vDec and vDec <= DblCalcDev) then
		return iSign*math.floor(iSign*dNum+dCorrection);
	else
		local dFactor = 10^vDec;
		return iSign*math.floor(iSign*dNum*dFactor+dCorrection)/dFactor;
	end
	
end

function RoundDblLotro(dNum)

	local dCorrection = 1-DblCalcDev;
	local iSign = 1;
	if dNum < 0 then
		iSign = -1;
	end

	local iNumCeiled = math.floor(iSign*dNum+dCorrection);
	if iNumCeiled <= 1000 then
		return iSign*iNumCeiled;
	end

	local iFactor = 1;
	local iTestNum = math.floor(iNumCeiled/1000);
	while iTestNum > 0 do
		iTestNum = math.floor(iTestNum/10);
		iFactor = iFactor*10;
	end

	return iSign*math.floor(iNumCeiled/iFactor+dCorrection)*iFactor;

end

-- ******************************* End CalcStat *******************************